/*     */ package com.googlecode.concurrentlinkedhashmap;
/*     */ 
/*     */ import java.io.Serializable;
/*     */ import java.util.Collection;
/*     */ import java.util.Iterator;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.Set;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public final class Weighers
/*     */ {
/*     */   private Weighers()
/*     */   {
/*  37 */     throw new AssertionError();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static <K, V> EntryWeigher<K, V> asEntryWeigher(Weigher<? super V> weigher)
/*     */   {
/*  49 */     return weigher == singleton() ? entrySingleton() : new EntryWeigherView(weigher);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static <K, V> EntryWeigher<K, V> entrySingleton()
/*     */   {
/*  63 */     return SingletonEntryWeigher.INSTANCE;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static <V> Weigher<V> singleton()
/*     */   {
/*  75 */     return SingletonWeigher.INSTANCE;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static Weigher<byte[]> byteArray()
/*     */   {
/*  94 */     return ByteArrayWeigher.INSTANCE;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static <E> Weigher<? super Iterable<E>> iterable()
/*     */   {
/* 113 */     return IterableWeigher.INSTANCE;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static <E> Weigher<? super Collection<E>> collection()
/*     */   {
/* 131 */     return CollectionWeigher.INSTANCE;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static <E> Weigher<? super List<E>> list()
/*     */   {
/* 149 */     return ListWeigher.INSTANCE;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static <E> Weigher<? super Set<E>> set()
/*     */   {
/* 167 */     return SetWeigher.INSTANCE;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static <A, B> Weigher<? super Map<A, B>> map()
/*     */   {
/* 185 */     return MapWeigher.INSTANCE;
/*     */   }
/*     */   
/*     */   static final class EntryWeigherView<K, V> implements EntryWeigher<K, V>, Serializable {
/*     */     static final long serialVersionUID = 1L;
/*     */     final Weigher<? super V> weigher;
/*     */     
/*     */     EntryWeigherView(Weigher<? super V> weigher) {
/* 193 */       ConcurrentLinkedHashMap.checkNotNull(weigher);
/* 194 */       this.weigher = weigher;
/*     */     }
/*     */     
/*     */     public int weightOf(K key, V value)
/*     */     {
/* 199 */       return this.weigher.weightOf(value);
/*     */     }
/*     */   }
/*     */   
/*     */   static enum SingletonEntryWeigher implements EntryWeigher<Object, Object> {
/* 204 */     INSTANCE;
/*     */     
/*     */     private SingletonEntryWeigher() {}
/*     */     
/* 208 */     public int weightOf(Object key, Object value) { return 1; }
/*     */   }
/*     */   
/*     */   static enum SingletonWeigher implements Weigher<Object>
/*     */   {
/* 213 */     INSTANCE;
/*     */     
/*     */     private SingletonWeigher() {}
/*     */     
/* 217 */     public int weightOf(Object value) { return 1; }
/*     */   }
/*     */   
/*     */   static enum ByteArrayWeigher implements Weigher<byte[]>
/*     */   {
/* 222 */     INSTANCE;
/*     */     
/*     */     private ByteArrayWeigher() {}
/*     */     
/* 226 */     public int weightOf(byte[] value) { return value.length; }
/*     */   }
/*     */   
/*     */   static enum IterableWeigher implements Weigher<Iterable<?>>
/*     */   {
/* 231 */     INSTANCE;
/*     */     
/*     */     private IterableWeigher() {}
/*     */     
/* 235 */     public int weightOf(Iterable<?> values) { if ((values instanceof Collection)) {
/* 236 */         return ((Collection)values).size();
/*     */       }
/* 238 */       int size = 0;
/* 239 */       for (Iterator<?> i = values.iterator(); i.hasNext();) {
/* 240 */         i.next();
/* 241 */         size++;
/*     */       }
/* 243 */       return size;
/*     */     }
/*     */   }
/*     */   
/*     */   static enum CollectionWeigher implements Weigher<Collection<?>> {
/* 248 */     INSTANCE;
/*     */     
/*     */     private CollectionWeigher() {}
/*     */     
/* 252 */     public int weightOf(Collection<?> values) { return values.size(); }
/*     */   }
/*     */   
/*     */   static enum ListWeigher implements Weigher<List<?>>
/*     */   {
/* 257 */     INSTANCE;
/*     */     
/*     */     private ListWeigher() {}
/*     */     
/* 261 */     public int weightOf(List<?> values) { return values.size(); }
/*     */   }
/*     */   
/*     */   static enum SetWeigher implements Weigher<Set<?>>
/*     */   {
/* 266 */     INSTANCE;
/*     */     
/*     */     private SetWeigher() {}
/*     */     
/* 270 */     public int weightOf(Set<?> values) { return values.size(); }
/*     */   }
/*     */   
/*     */   static enum MapWeigher implements Weigher<Map<?, ?>>
/*     */   {
/* 275 */     INSTANCE;
/*     */     
/*     */     private MapWeigher() {}
/*     */     
/* 279 */     public int weightOf(Map<?, ?> values) { return values.size(); }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\googlecode\concurrentlinkedhashmap\Weighers.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */