/*     */ package com.netapp.asup.data.objects;
/*     */ 
/*     */ import com.netapp.asup.client.arma.ARMAObject;
/*     */ import com.netapp.asup.client.arma.data.ARMAObjectInstance;
/*     */ import com.netapp.asup.client.arma.data.ARMAObjectInstanceIterator;
/*     */ import com.netapp.asup.common.AsupLogger;
/*     */ import com.netapp.asup.data.function.FunctionInputHandler;
/*     */ import com.netapp.asup.data.function.FunctionRegistry;
/*     */ import com.netapp.asup.data.function.RegisteredFunction;
/*     */ import com.netapp.asup.data.store.BaseContainer;
/*     */ import com.netapp.asup.data.store.DataStore;
/*     */ import com.netapp.asup.data.store.StoreContainer;
/*     */ import com.netapp.asup.exception.ARMAException;
/*     */ import com.netapp.asup.exception.ARMAFunctionExecutionException;
/*     */ import com.netapp.asupdata.client.ARMA.ARMAClientSession;
/*     */ import com.netapp.asupdata.client.ARMA.ARMAObjectFactoryImpl;
/*     */ import com.netapp.asupdata.client.ARMA.ARMAObjectImpl;
/*     */ import java.io.IOException;
/*     */ import java.util.ArrayList;
/*     */ import java.util.List;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.generic.GenericArray;
/*     */ import org.apache.avro.generic.GenericData.Array;
/*     */ import org.apache.avro.generic.GenericData.Record;
/*     */ import org.apache.avro.generic.GenericRecord;
/*     */ import org.apache.avro.util.Utf8;
/*     */ 
/*     */ public class FunctionRequest extends AvroObject<GenericArray<GenericRecord>>
/*     */ {
/*  30 */   public static DataStore dataStore = BaseContainer.getInstance().getStoreContainer().getStore();
/*     */   
/*  32 */   protected static final AsupLogger LOG = AsupLogger.getLogger(FunctionRequest.class);
/*     */   
/*     */ 
/*     */   private static final String FUNCTION_NAME = "functionName";
/*     */   
/*     */   private static final String NULL_SCHEMA_PATH = "com/netapp/asup/data/function/schema/NullFunctionInput";
/*     */   
/*  39 */   private static final FunctionRegistry functionRegistry = BaseContainer.getInstance().getFunctionRegistry();
/*     */   
/*  41 */   private final List<FunctionRequestData<?, ?>> functionInputData = new ArrayList();
/*     */   
/*     */   public FunctionRequest(GenericArray<GenericRecord> object, String id) {
/*  44 */     super(ARMASchema.FUNCTION_REQUEST, object, id);
/*  45 */     populateFunctionInputData();
/*     */   }
/*     */   
/*     */   private <I, O> void populateFunctionInputData()
/*     */   {
/*  50 */     for (GenericRecord object : (GenericArray)this.object) {
/*  51 */       Utf8 functionName = (Utf8)object.get("functionName");
/*  52 */       RegisteredFunction<I, O> registeredFunction = functionRegistry.getRegisteredFunction(functionName.toString());
/*  53 */       if (registeredFunction != null) {
/*     */         try {
/*  55 */           this.functionInputData.add(new FunctionRequestData(this.id, object, registeredFunction));
/*     */         } catch (IOException e) {
/*  57 */           new ARMAException(LOG, this.id, "function execution failed for function :" + functionName + " caused by :" + e.getMessage(), e);
/*     */         }
/*     */       } else {
/*  60 */         LOG.warn(this.id, "Function :" + functionName.toString() + " doesn't exist , so trigring local execution for all functions executing after this.");
/*     */         
/*  62 */         this.lastFunctionExecutionIndex = (this.functionInputData.size() - 1);
/*  63 */         break;
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*  68 */   private int firstLocalFunctionIndex = -1;
/*     */   
/*  70 */   private int lastFunctionExecutionIndex = -1;
/*     */   
/*     */   public FunctionRequest(String id)
/*     */   {
/*  74 */     super(ARMASchema.FUNCTION_REQUEST, id);
/*     */   }
/*     */   
/*     */ 
/*     */   public <I, O> void addFunction(String objName, String identifier, RegisteredFunction<I, O> registeredFunction, I input)
/*     */     throws ARMAFunctionExecutionException
/*     */   {
/*  81 */     GenericRecord functionInput = null;
/*     */     
/*  83 */     FunctionInputHandler<I> inputHandler = registeredFunction.getInputHandler();
/*  84 */     if (input != null) {
/*  85 */       if ((input instanceof GenericRecord)) {
/*  86 */         functionInput = (GenericRecord)input;
/*  87 */       } else if (inputHandler != null) {
/*     */         try {
/*  89 */           functionInput = inputHandler.handleInput(registeredFunction.getInputSchema(), input);
/*     */         } catch (Exception e) {
/*  91 */           throw new ARMAFunctionExecutionException(LOG, this.id, "Unknown exception in handling input caused by :" + e.getMessage(), e);
/*     */         }
/*     */       } else {
/*  94 */         throw new ARMAFunctionExecutionException(LOG, this.id, "Neither a valid input handler is defined, nor the function input is a instance of GenericRecord");
/*     */       }
/*     */     } else {
/*  97 */       Schema nullSchema = functionRegistry.getSchema("com/netapp/asup/data/function/schema/NullFunctionInput");
/*  98 */       functionInput = new GenericData.Record(nullSchema);
/*     */     }
/* 100 */     this.functionInputData.add(new FunctionRequestData(this.id, objName, identifier, registeredFunction, functionInput));
/* 101 */     if ((registeredFunction.isLocal()) && (this.firstLocalFunctionIndex == -1)) {
/* 102 */       this.firstLocalFunctionIndex = (this.functionInputData.size() - 1);
/*     */     }
/*     */   }
/*     */   
/*     */   public GenericArray<GenericRecord> getObject()
/*     */     throws IOException
/*     */   {
/* 109 */     int size = this.firstLocalFunctionIndex == -1 ? this.functionInputData.size() : this.firstLocalFunctionIndex;
/* 110 */     GenericArray<GenericRecord> object = new GenericData.Array(size, getSchema());
/* 111 */     for (int i = 0; i < size; i++) {
/* 112 */       object.add(((FunctionRequestData)this.functionInputData.get(i)).getObject());
/*     */     }
/* 114 */     return object;
/*     */   }
/*     */   
/*     */   public int getLastFunctionExecutionIndex() {
/* 118 */     return this.lastFunctionExecutionIndex;
/*     */   }
/*     */   
/*     */   public void execute(Identifier identifier, Response responseData)
/*     */     throws ARMAException
/*     */   {
/* 124 */     Functions functions = responseData.getFunctions();
/*     */     
/* 126 */     if (this.functionInputData.isEmpty())
/*     */     {
/* 128 */       return;
/*     */     }
/*     */     
/* 131 */     int totalInputFunctions = this.functionInputData.size();
/* 132 */     Integer executionIndex = Integer.valueOf(functions.getExecutionIndex());
/*     */     
/* 134 */     if (executionIndex.intValue() >= totalInputFunctions)
/*     */     {
/* 136 */       return;
/*     */     }
/*     */     
/* 139 */     ARMAObjectImpl graphRoot = getRoot(identifier, responseData);
/* 140 */     int index = executionIndex.intValue();
/*     */     
/*     */ 
/*     */ 
/*     */     try
/*     */     {
/* 146 */       ARMAObjectInstance instance = null;
/* 147 */       for (; index < totalInputFunctions; index++) {
/* 148 */         FunctionRequestData<?, ?> functionRequestData = (FunctionRequestData)this.functionInputData.get(index);
/* 149 */         FunctionData functionData = functions.initializeFunctionData(functionRequestData.getRegisteredFunction(), functionRequestData.getFunctionId());
/*     */         
/*     */ 
/* 152 */         ARMAObject node = graphRoot.jumpToNode(functionRequestData.getObjectName());
/*     */         
/* 154 */         ARMAObjectInstanceIterator instances = node.getInstances();
/* 155 */         while (instances.hasNext()) {
/* 156 */           instance = instances.next();
/* 157 */           functionData.addFunctionOutput(instance.getKey(), functionRequestData.execute(instance));
/*     */         }
/*     */         
/*     */       }
/*     */     }
/*     */     finally
/*     */     {
/* 164 */       this.lastFunctionExecutionIndex = index;
/* 165 */       functions.setExecutionIndex(String.valueOf(this.lastFunctionExecutionIndex));
/*     */     }
/*     */   }
/*     */   
/*     */   private ARMAObjectImpl getRoot(Identifier identifier, Response responseData) throws ARMAException
/*     */   {
/* 171 */     ARMAObjectFactoryImpl armaObjectFactory = new ARMAObjectFactoryImpl();
/* 172 */     ARMAObjectImpl graphRoot = (ARMAObjectImpl)armaObjectFactory.getGraphRoot(identifier, Boolean.valueOf(identifier.getClusterRoot()).booleanValue());
/* 173 */     ARMAClientSession session = graphRoot.getSession();
/* 174 */     session.populateResponse(responseData);
/* 175 */     return graphRoot;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\data\objects\FunctionRequest.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */