/*     */ package com.netapp.asup.data.objects;
/*     */ 
/*     */ import com.netapp.asup.common.AsupLogger;
/*     */ import com.netapp.asupdata.client.ARMA.utils.DateUtil;
/*     */ import java.io.IOException;
/*     */ import java.text.ParseException;
/*     */ import java.text.SimpleDateFormat;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Calendar;
/*     */ import java.util.HashMap;
/*     */ import java.util.Iterator;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.Map.Entry;
/*     */ import java.util.TimeZone;
/*     */ import java.util.TreeMap;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.Schema.Field;
/*     */ import org.apache.avro.generic.GenericArray;
/*     */ import org.apache.avro.generic.GenericData.Array;
/*     */ import org.apache.avro.generic.GenericRecord;
/*     */ import org.apache.avro.util.Utf8;
/*     */ 
/*     */ 
/*     */ public class PerfCounters
/*     */   extends AvroObject<Map<Utf8, Map<Utf8, GenericRecord>>>
/*     */ {
/*     */   private static final String DATA = "data";
/*     */   private static final String TIMESTAMPS = "timestamps";
/*     */   private static final long EPOCH_TIME_MAX_VALUE = 9999999999L;
/*     */   
/*     */   public PerfCounters(String id)
/*     */   {
/*  34 */     super(ARMASchema.PERFCOUNTERS, id);
/*  35 */     this.object = new HashMap();
/*     */   }
/*     */   
/*     */   public PerfCounters(ARMASchema schema, Map<Utf8, Map<Utf8, GenericRecord>> object, String id)
/*     */   {
/*  40 */     super(schema, object, id);
/*     */   }
/*     */   
/*     */   public void addPerfData(String requestId, Utf8 serialNo, Utf8 objectName, Schema readerSchema, Schema writerSchema, byte[] value)
/*     */     throws IOException
/*     */   {
/*  46 */     String objectNameLower = objectName.toString().toLowerCase();
/*  47 */     Utf8 objectNameLowerUtf8 = new Utf8(objectNameLower);
/*  48 */     PerfCounterData inputPerfCounterData = new PerfCounterData(value, readerSchema, writerSchema, objectNameLower, requestId);
/*  49 */     GenericRecord inputPerfData = inputPerfCounterData.getCounterData();
/*     */     
/*  51 */     logger.debug(requestId, "Perf Data found:" + inputPerfData.toString());
/*  52 */     int timeStampLen; if (logger.isDebugEnabled())
/*     */     {
/*  54 */       timeStampLen = ((List)inputPerfData.get("timestamps")).size();
/*  55 */       for (Schema.Field f : inputPerfData.getSchema().getFields())
/*     */       {
/*  57 */         if (!f.name().equals("timestamps"))
/*     */         {
/*     */ 
/*  60 */           map = (Map)inputPerfData.get(f.name());
/*  61 */           for (Utf8 key : map.keySet())
/*     */           {
/*  63 */             if (((GenericArray)map.get(key)).size() != timeStampLen)
/*     */             {
/*  65 */               logger.warn(requestId, "Mismatched counter data.  Instance key: " + key + ", length: " + ((GenericArray)map.get(key)).size() + "expected length: " + timeStampLen);
/*     */             }
/*     */             else
/*  68 */               logger.debug(requestId, "Counter data matches.  Instance key: " + key + ", length: " + ((GenericArray)map.get(key)).size() + "expected length: " + timeStampLen);
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */     Map<Utf8, GenericArray<GenericRecord>> map;
/*  74 */     Utf8 serNo = new Utf8(serialNo);
/*     */     
/*  76 */     Map<Utf8, GenericRecord> existingObjectMap = (Map)((Map)this.object).get(serNo);
/*     */     
/*  78 */     if (existingObjectMap == null)
/*     */     {
/*  80 */       Map<Utf8, GenericRecord> objectMap = new HashMap();
/*  81 */       objectMap.put(objectNameLowerUtf8, inputPerfData);
/*     */       
/*  83 */       ((Map)this.object).put(serNo, objectMap);
/*     */     } else {
/*  85 */       GenericRecord existingPerfData = (GenericRecord)existingObjectMap.get(objectNameLowerUtf8);
/*     */       
/*     */ 
/*  88 */       if (existingPerfData == null)
/*     */       {
/*  90 */         existingObjectMap.put(objectNameLowerUtf8, inputPerfData);
/*     */       }
/*     */       else
/*     */       {
/*  94 */         existingObjectMap.put(objectNameLowerUtf8, merge(existingPerfData, inputPerfData));
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private GenericRecord merge(GenericRecord existingPerfData, GenericRecord perfData)
/*     */   {
/* 107 */     if (existingPerfData != null) {
/* 108 */       GenericData.Array<Long> timestamps = (GenericData.Array)existingPerfData.get("timestamps");
/* 109 */       timestamps.addAll(getTimeStampForMerge(perfData));
/* 110 */       Map<Utf8, GenericData.Array<GenericRecord>> instanceCounterDataMap = (Map)existingPerfData.get("data");
/* 111 */       for (Map.Entry<Utf8, GenericData.Array<GenericRecord>> instanceEntry : instanceCounterDataMap.entrySet())
/*     */       {
/* 113 */         Utf8 instanceKey = (Utf8)instanceEntry.getKey();
/* 114 */         GenericData.Array<GenericRecord> counterRecords = (GenericData.Array)instanceEntry.getValue();
/* 115 */         counterRecords.addAll(getCounterDataForMerge(perfData, instanceKey));
/*     */       }
/*     */     }
/* 118 */     return existingPerfData;
/*     */   }
/*     */   
/*     */ 
/*     */   private List<Long> getTimeStampForMerge(GenericRecord data)
/*     */   {
/* 124 */     List<Long> timestampData = new ArrayList();
/*     */     
/* 126 */     if (data != null) {
/* 127 */       GenericData.Array<Long> timestamps = (GenericData.Array)data.get("timestamps");
/* 128 */       if (timestamps != null)
/*     */       {
/* 130 */         timestampData.addAll(timestamps);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/* 135 */     return timestampData;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   private List<GenericRecord> getCounterDataForMerge(GenericRecord objectdata, Utf8 instanceKey)
/*     */   {
/* 143 */     List<GenericRecord> records = new ArrayList();
/*     */     
/* 145 */     if (objectdata != null) {
/* 146 */       Map<Utf8, GenericData.Array<GenericRecord>> instanceCounterDataMap = (Map)objectdata.get("data");
/*     */       
/* 148 */       if (instanceCounterDataMap != null) {
/* 149 */         GenericData.Array<GenericRecord> counterDataArray = (GenericData.Array)instanceCounterDataMap.get(instanceKey);
/*     */         
/* 151 */         if (counterDataArray != null)
/*     */         {
/* 153 */           records.addAll(counterDataArray);
/*     */         }
/*     */       }
/*     */     }
/*     */     
/* 158 */     return records;
/*     */   }
/*     */   
/*     */   public Map<Utf8, Map<Utf8, GenericRecord>> getObject() throws IOException
/*     */   {
/* 163 */     return (Map)this.object;
/*     */   }
/*     */   
/*     */   public List<Long> getTimeStamp(Identifier id, Utf8 serialNo, String objectName, Integer intervalInDays) throws ParseException
/*     */   {
/* 168 */     List<Long> timestampData = new ArrayList();
/* 169 */     Map<Utf8, GenericRecord> data = (Map)((Map)this.object).get(serialNo);
/* 170 */     if (data != null) {
/* 171 */       GenericRecord objectdata = (GenericRecord)data.get(new Utf8(objectName.toLowerCase()));
/*     */       
/* 173 */       if (objectdata != null) {
/* 174 */         if (intervalInDays != null)
/*     */         {
/* 176 */           timestampData.addAll(filterTimeStamps(id, objectdata, intervalInDays.intValue()));
/*     */         }
/*     */         else {
/* 179 */           timestampData.addAll((GenericData.Array)objectdata.get("timestamps"));
/*     */         }
/*     */       }
/*     */     }
/* 183 */     return timestampData;
/*     */   }
/*     */   
/*     */   private List<Long> filterTimeStamps(Identifier id, GenericRecord counterRecord, int intervalInDays) throws ParseException
/*     */   {
/* 188 */     List<Long> returnTimeStamps = new ArrayList();
/* 189 */     List<Integer> timestampIndex = getTimeStampIndex(id, counterRecord, intervalInDays);
/* 190 */     GenericData.Array<Long> allTimeStamps = (GenericData.Array)counterRecord.get("timestamps");
/* 191 */     for (Iterator i$ = timestampIndex.iterator(); i$.hasNext();) { int index = ((Integer)i$.next()).intValue();
/*     */       
/* 193 */       returnTimeStamps.add(allTimeStamps.get(index));
/*     */     }
/* 195 */     return returnTimeStamps;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private List<Integer> getTimeStampIndex(Identifier id, GenericRecord counterRecord, int intervalInDays)
/*     */     throws ParseException
/*     */   {
/* 209 */     String latestRecordBizKey = ((Utf8)id.getBizKeys().get(0)).toString();
/* 210 */     String startKeyDate = getStartKeyDate(id);
/* 211 */     if (startKeyDate == null) {
/* 212 */       String[] split = latestRecordBizKey.split("\\|");
/* 213 */       if (split.length != 4) {
/* 214 */         throw new IllegalArgumentException("Invalid biz key: " + latestRecordBizKey);
/*     */       }
/*     */       
/* 217 */       startKeyDate = split[2];
/*     */     }
/*     */     
/* 220 */     Calendar latestDate = getDateFromBizKey(startKeyDate);
/* 221 */     long endTimeStamp = latestDate.getTimeInMillis() / 1000L;
/*     */     
/*     */ 
/* 224 */     latestDate.add(5, 0 - intervalInDays);
/*     */     
/* 226 */     long firstTimeStamp = latestDate.getTimeInMillis() / 1000L;
/*     */     
/* 228 */     List<Integer> indexList = new ArrayList();
/* 229 */     GenericData.Array<Long> allTimestamps = (GenericData.Array)counterRecord.get("timestamps");
/*     */     
/* 231 */     for (int i = 0; i < allTimestamps.size(); i++)
/*     */     {
/* 233 */       if ((((Long)allTimestamps.get(i)).longValue() >= firstTimeStamp) && (((Long)allTimestamps.get(i)).longValue() <= endTimeStamp))
/*     */       {
/* 235 */         indexList.add(Integer.valueOf(i));
/*     */       }
/*     */     }
/* 238 */     return indexList;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private List<Integer> getTimeStampIndex(Identifier id, GenericData.Array<Long> allTimestamps, int intervalInDays)
/*     */     throws ParseException
/*     */   {
/* 251 */     String latestRecordBizKey = ((Utf8)id.getBizKeys().get(0)).toString();
/* 252 */     String startKeyDate = getStartKeyDate(id);
/* 253 */     if (startKeyDate == null)
/*     */     {
/* 255 */       String[] split = latestRecordBizKey.split("\\|");
/* 256 */       if (split.length != 4) {
/* 257 */         throw new IllegalArgumentException("Invalid biz key: " + latestRecordBizKey);
/*     */       }
/*     */       
/* 260 */       startKeyDate = split[2];
/*     */       
/*     */ 
/*     */ 
/*     */ 
/* 265 */       long startTime = Long.valueOf(startKeyDate).longValue() - 50400L;
/* 266 */       startKeyDate = String.valueOf(startTime);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 271 */     Calendar latestDate = getDateFromBizKey(startKeyDate);
/* 272 */     long endTimeStamp = latestDate.getTimeInMillis() / 1000L;
/* 273 */     latestDate.add(5, 0 - intervalInDays);
/* 274 */     long firstTimeStamp = latestDate.getTimeInMillis() / 1000L;
/* 275 */     List<Integer> indexList = new ArrayList();
/* 276 */     for (int i = 0; i < allTimestamps.size(); i++)
/*     */     {
/* 278 */       if ((((Long)allTimestamps.get(i)).longValue() >= firstTimeStamp) && (((Long)allTimestamps.get(i)).longValue() <= endTimeStamp))
/*     */       {
/* 280 */         indexList.add(Integer.valueOf(i));
/*     */       }
/*     */     }
/* 283 */     return indexList;
/*     */   }
/*     */   
/*     */   private String getStartKeyDate(Identifier identifier) {
/* 287 */     Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
/*     */     
/* 289 */     SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
/*     */     
/* 291 */     dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));
/*     */     try {
/* 293 */       String timeStamp = identifier.getTimeStamp();
/* 294 */       if ((timeStamp != null) && (!timeStamp.isEmpty()))
/*     */       {
/* 296 */         calendar.setTime(dateFormat.parse(identifier.getTimeStamp()));
/*     */ 
/*     */       }
/*     */       else
/*     */       {
/* 301 */         return null;
/*     */       }
/*     */     } catch (ParseException e) {
/* 304 */       logger.debug(identifier.getId(), "Cannot parse date :" + identifier.getTimeStamp());
/* 305 */       return null;
/*     */     }
/* 307 */     return DateUtil.getAsupGenDateInInversedEpochTime(calendar);
/*     */   }
/*     */   
/* 310 */   private static Calendar getDateFromBizKey(String dateStringFromBizKey) { Long value = Long.valueOf(9999999999L - Long.valueOf(dateStringFromBizKey).longValue());
/* 311 */     Calendar calendar = Calendar.getInstance();
/* 312 */     calendar.setTimeInMillis(value.longValue() * 1000L);
/* 313 */     return calendar;
/*     */   }
/*     */   
/*     */   public Map<String, List<String>> getCounterData(Identifier id, Utf8 serialNo, String objectName, String counterName, Integer intervalInDays)
/*     */     throws ParseException
/*     */   {
/* 319 */     Map<String, List<String>> counterMap = new HashMap();
/*     */     
/* 321 */     Map<Utf8, GenericRecord> data = (Map)((Map)this.object).get(serialNo);
/*     */     
/*     */ 
/*     */     GenericRecord objectdata;
/*     */     
/*     */     Map<Utf8, GenericData.Array<GenericRecord>> instanceCounterDataMap;
/*     */     
/* 328 */     if (data != null) {
/* 329 */       objectdata = (GenericRecord)data.get(new Utf8(objectName.toLowerCase()));
/* 330 */       if (objectdata != null) {
/* 331 */         instanceCounterDataMap = (Map)objectdata.get("data");
/* 332 */         if (instanceCounterDataMap != null) {
/* 333 */           for (Utf8 counterDataName : instanceCounterDataMap.keySet()) {
/* 334 */             List<String> counters = new ArrayList();
/* 335 */             GenericData.Array<GenericRecord> counterDataArray = (GenericData.Array)instanceCounterDataMap.get(counterDataName);
/*     */             
/* 337 */             if (counterDataArray != null) {
/* 338 */               int counterDataArraySize = counterDataArray.size();
/* 339 */               Iterator i$; if (intervalInDays != null)
/*     */               {
/* 341 */                 List<Integer> timeStampIndex = getTimeStampIndex(id, objectdata, intervalInDays.intValue());
/*     */                 
/* 343 */                 for (i$ = timeStampIndex.iterator(); i$.hasNext();) { int index = ((Integer)i$.next()).intValue();
/*     */                   
/* 345 */                   if (index < counterDataArraySize)
/*     */                   {
/* 347 */                     GenericRecord counterData = (GenericRecord)counterDataArray.get(index);
/* 348 */                     if (counterData.get(counterName) != null)
/*     */                     {
/* 350 */                       counters.add(counterData.get(counterName).toString());
/*     */                     }
/*     */                     else {
/* 353 */                       logger.warn(id.getId(), "Mismatch between timestamps and values for following counter data instance: " + counterDataName);
/*     */                     }
/*     */                   }
/*     */                 }
/*     */               }
/*     */               else {
/* 359 */                 for (GenericRecord counterRecord : counterDataArray) {
/* 360 */                   if (counterRecord != null)
/*     */                   {
/* 362 */                     counters.add(counterRecord.get(counterName).toString());
/*     */                   }
/*     */                 }
/*     */               }
/*     */             }
/* 367 */             counterMap.put(counterDataName.toString(), counters);
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/* 372 */     return counterMap;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Map<Utf8, GenericData.Array<GenericRecord>> getCMPerfCounterData(Identifier id, Utf8 serialNo, String objectName, Integer intervalInDays)
/*     */     throws ParseException
/*     */   {
/* 387 */     Map<Utf8, GenericRecord> data = (Map)((Map)this.object).get(serialNo);
/*     */     
/* 389 */     Map<Utf8, GenericData.Array<GenericRecord>> instanceCounterDataMap = new HashMap();
/*     */     
/* 391 */     if (data != null) {
/* 392 */       GenericRecord objectdata = (GenericRecord)data.get(new Utf8(objectName.toLowerCase()));
/* 393 */       if (objectdata != null) {
/* 394 */         instanceCounterDataMap.putAll((Map)objectdata.get("data"));
/*     */       }
/*     */     }
/*     */     
/* 398 */     return instanceCounterDataMap;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public Map<String, Map<Long, Map<String, List<String>>>> getCounterDataGroupedByTimestamp(Identifier id, Utf8 serialNo, String objectName, List<String> counterNames, List<String> baseCounterNames, Integer intervalInDays)
/*     */     throws ParseException
/*     */   {
/* 407 */     Map<Utf8, GenericRecord> data = (Map)((Map)this.object).get(serialNo);
/*     */     
/*     */ 
/*     */ 
/* 411 */     Object baseCounterData = null;
/*     */     
/*     */ 
/* 414 */     Map<String, Map<Long, Map<String, List<String>>>> instanceMap = new HashMap();
/* 415 */     List<Integer> timeStampIndex; Map<Utf8, GenericData.Array<GenericRecord>> instanceCounterDataMap; GenericData.Array<Long> allTimeStamps; if (data != null) {
/* 416 */       GenericRecord objectdata = (GenericRecord)data.get(new Utf8(objectName.toLowerCase()));
/* 417 */       if (objectdata != null) {
/* 418 */         GenericData.Array<Long> allTimestamps = (GenericData.Array)objectdata.get("timestamps");
/* 419 */         timeStampIndex = getTimeStampIndex(id, allTimestamps, intervalInDays.intValue());
/* 420 */         instanceCounterDataMap = (Map)objectdata.get("data");
/* 421 */         allTimeStamps = (GenericData.Array)objectdata.get("timestamps");
/* 422 */         if (instanceCounterDataMap != null) {
/* 423 */           for (Utf8 counterDataName : instanceCounterDataMap.keySet()) {
/* 424 */             GenericData.Array<GenericRecord> counterDataArray = (GenericData.Array)instanceCounterDataMap.get(counterDataName);
/* 425 */             Map<Long, Map<String, List<String>>> timeStampMap = new TreeMap();
/* 426 */             Iterator i$; if ((counterDataArray != null) && 
/* 427 */               (intervalInDays != null))
/*     */             {
/*     */ 
/*     */ 
/* 431 */               for (i$ = timeStampIndex.iterator(); i$.hasNext();) { int index = ((Integer)i$.next()).intValue();
/*     */                 
/* 433 */                 GenericRecord counterRecord = null;
/* 434 */                 if (index < counterDataArray.size()) {
/* 435 */                   counterRecord = (GenericRecord)counterDataArray.get(index);
/*     */                   
/*     */ 
/* 438 */                   Long timestamp = (Long)allTimeStamps.get(index);
/*     */                   
/* 440 */                   if (counterRecord != null)
/*     */                   {
/*     */ 
/* 443 */                     Map<String, List<String>> counterMap = new HashMap();
/* 444 */                     int counterIndex = 0;
/* 445 */                     List<String> counterValueList = null;
/* 446 */                     for (String counterName : counterNames)
/*     */                     {
/* 448 */                       Object counterData = counterRecord.get(counterName);
/* 449 */                       if (baseCounterNames != null)
/*     */                       {
/* 451 */                         baseCounterData = counterRecord.get((String)baseCounterNames.get(counterIndex));
/*     */                       }
/* 453 */                       counterValueList = new ArrayList(2);
/* 454 */                       if (counterData != null)
/*     */                       {
/* 456 */                         counterValueList.add(counterData.toString());
/*     */                         
/* 458 */                         if (baseCounterData != null)
/*     */                         {
/* 460 */                           counterValueList.add(baseCounterData.toString());
/*     */                         }
/* 462 */                         counterMap.put(counterName, counterValueList);
/*     */                       }
/* 464 */                       counterIndex++;
/*     */                     }
/* 466 */                     timeStampMap.put(timestamp, counterMap);
/*     */                   }
/*     */                 }
/*     */               }
/*     */             }
/*     */             
/* 472 */             if (timeStampMap != null) {
/* 473 */               instanceMap.put(counterDataName.toString().toLowerCase(), timeStampMap);
/*     */             }
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/* 479 */     return instanceMap;
/*     */   }
/*     */   
/*     */   public boolean isEmpty()
/*     */   {
/* 484 */     return ((Map)this.object).isEmpty();
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\data\objects\PerfCounters.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */