/*     */ package com.netapp.asup.data.util;
/*     */ 
/*     */ import com.netapp.asup.common.AsupLogger;
/*     */ import com.netapp.asup.config.ARMAConfiguration;
/*     */ import com.netapp.asup.data.exception.ARMARemoteCallException;
/*     */ import com.netapp.asup.data.exception.ARMASchemaOutOfSyncException;
/*     */ import com.netapp.asup.data.objects.AvroTransmissionObject;
/*     */ import com.netapp.asup.exception.ARMAConfigurationException;
/*     */ import com.netapp.asup.exception.ARMAMetaModelException;
/*     */ import java.io.BufferedReader;
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.io.InputStreamReader;
/*     */ import java.io.OutputStream;
/*     */ import java.net.HttpURLConnection;
/*     */ import java.net.MalformedURLException;
/*     */ import java.net.URL;
/*     */ import java.util.Arrays;
/*     */ import java.util.List;
/*     */ import java.util.concurrent.Semaphore;
/*     */ import java.util.concurrent.TimeUnit;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class HttpConnectionManager
/*     */ {
/*     */   private final int readTimeOutInSec;
/*     */   private final int connectionTimeOutinSec;
/*     */   private final int maxConnections;
/*     */   private final URL baseUrl;
/*     */   private final String prefixUrl;
/*     */   private static final String ARMA_SERV_ENV = "arma.server.env";
/*  35 */   private static final List<String> ALL_ENV_STRINGS = Arrays.asList(new String[] { "dev", "qa", "utb", "stg", "prod", "hdc" });
/*     */   
/*     */ 
/*     */   private static final int CHUNK_LENGTH = 10240;
/*     */   
/*     */   private final Semaphore availableConnections;
/*     */   
/*  42 */   private static final AsupLogger logger = AsupLogger.getLogger(HttpConnectionManager.class);
/*     */   private static HttpConnectionManager instance;
/*     */   
/*     */   public static synchronized void initialize(ARMAConfiguration config)
/*     */     throws ARMAConfigurationException
/*     */   {
/*  48 */     String baseUrl = getPrefixURL(config);
/*  49 */     String prefixUrl = config.get("arma.rest.server.prefix.url");
/*     */     
/*  51 */     int maxConnections = new Integer(config.get("arma.rest.server.max.connection")).intValue();
/*  52 */     int readTimeOutInSec = new Integer(config.get("arma.rest.server.read.timeout.sec")).intValue();
/*  53 */     int connectionTimeOutinSec = new Integer(config.get("arma.rest.server.read.connect.sec")).intValue();
/*     */     
/*  55 */     if (instance == null) {
/*  56 */       instance = new HttpConnectionManager(baseUrl, prefixUrl, maxConnections, readTimeOutInSec, connectionTimeOutinSec);
/*     */     } else {
/*  58 */       throw new ARMAConfigurationException(logger, "The connection manager has already been initialized. It cannot be initialized again");
/*     */     }
/*     */   }
/*     */   
/*     */   private static String getPrefixURL(ARMAConfiguration config) throws ARMAConfigurationException
/*     */   {
/*  64 */     String property = System.getProperty("arma.server.env");
/*     */     
/*  66 */     if ((property == null) || (!ALL_ENV_STRINGS.contains(property.toLowerCase()))) {
/*  67 */       throw new ARMAConfigurationException(logger, "Expecting the following system property{arma.server.env}, to be set to any of the following environments:" + ALL_ENV_STRINGS + ".");
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*  73 */     return config.get(property.toLowerCase());
/*     */   }
/*     */   
/*     */   public static HttpConnectionManager getInstance() throws ARMAConfigurationException {
/*  77 */     return instance;
/*     */   }
/*     */   
/*     */   private HttpConnectionManager(String baseUrl, String prefixUrl, int maxConnections, int readTimeOutInSec, int connectionTimeOutinSec) throws ARMAConfigurationException {
/*  81 */     this.maxConnections = maxConnections;
/*  82 */     this.readTimeOutInSec = readTimeOutInSec;
/*  83 */     this.connectionTimeOutinSec = connectionTimeOutinSec;
/*  84 */     this.availableConnections = new Semaphore(maxConnections, true);
/*  85 */     this.prefixUrl = prefixUrl;
/*  86 */     System.setProperty("sun.net.http.errorstream.enableBuffering", "true");
/*  87 */     System.setProperty("http.maxConnections", String.valueOf(maxConnections));
/*     */     try {
/*  89 */       this.baseUrl = new URL(baseUrl);
/*     */     } catch (MalformedURLException e) {
/*  91 */       throw new ARMAConfigurationException(logger, "Invalid server URL caused by:" + e.getMessage(), e);
/*     */     }
/*     */   }
/*     */   
/*     */   public HttpConnection getConnection(String id, String relativeUrl, String requestMethod) throws ARMARemoteCallException
/*     */   {
/*     */     try
/*     */     {
/*  99 */       boolean connectionObtained = this.availableConnections.tryAcquire(this.connectionTimeOutinSec, TimeUnit.SECONDS);
/* 100 */       if (connectionObtained) {
/* 101 */         return new HttpConnection(id, relativeUrl, requestMethod, null);
/*     */       }
/* 103 */       throw new ARMARemoteCallException(logger, id, "Request timed out. Unable to take connection in :" + this.connectionTimeOutinSec + " secs");
/*     */     }
/*     */     catch (InterruptedException e) {
/* 106 */       throw new ARMARemoteCallException(logger, id, "Thread intrupted while acquiring connection caused by :" + e.getMessage());
/*     */     }
/*     */   }
/*     */   
/*     */   public int getMaxConnections()
/*     */   {
/* 112 */     return this.maxConnections;
/*     */   }
/*     */   
/*     */   public int getReadTimeOutInSec() {
/* 116 */     return this.readTimeOutInSec;
/*     */   }
/*     */   
/*     */   public int getConnectionTimeOutinSec() {
/* 120 */     return this.connectionTimeOutinSec;
/*     */   }
/*     */   
/*     */   public URL getBaseUrl() {
/* 124 */     return this.baseUrl;
/*     */   }
/*     */   
/*     */   public String getPrefixUrl() {
/* 128 */     return null != this.prefixUrl ? this.prefixUrl : "";
/*     */   }
/*     */   
/*     */   public class HttpConnection
/*     */   {
/*     */     private static final String NEW_LINE_CHAR = "\n";
/*     */     private static final String STATUS = "status";
/*     */     private static final String SUCCESS = "SUCCESS";
/*     */     private final HttpURLConnection connection;
/*     */     
/*     */     private HttpConnection(String id, String relativeUrl, String requestMethod)
/*     */       throws ARMARemoteCallException
/*     */     {
/*     */       try
/*     */       {
/* 143 */         url = new URL(HttpConnectionManager.this.getBaseUrl(), HttpConnectionManager.this.getPrefixUrl() + relativeUrl);
/* 144 */         HttpURLConnection connection = (HttpURLConnection)url.openConnection();
/* 145 */         connection.setDoInput(true);
/* 146 */         connection.setDoOutput(true);
/* 147 */         connection.setChunkedStreamingMode(10240);
/* 148 */         connection.setReadTimeout(HttpConnectionManager.this.getReadTimeOutInSec() * 1000);
/* 149 */         connection.setConnectTimeout(HttpConnectionManager.this.getConnectionTimeOutinSec() * 1000);
/* 150 */         connection.setRequestMethod(requestMethod);
/* 151 */         this.connection = connection;
/*     */       } catch (IOException e) { URL url;
/* 153 */         throw new ARMARemoteCallException(HttpConnectionManager.logger, id, "Unable to establish connection with URL :" + relativeUrl + " caused by :" + e.getMessage(), e);
/*     */       }
/*     */     }
/*     */     
/*     */     public <Y extends AvroTransmissionObject, X extends AvroTransmissionObject> Y postRequest(Y response, X request) throws ARMARemoteCallException {
/* 158 */       InputStream inputStream = null;
/* 159 */       InputStream errorStream = null;
/*     */       try {
/* 161 */         this.connection.setRequestProperty("requestId", request.getId());
/*     */         
/*     */ 
/*     */ 
/* 165 */         OutputStream outputStream = this.connection.getOutputStream();
/*     */         try {
/* 167 */           if (HttpConnectionManager.logger.isDebugEnabled()) {
/* 168 */             HttpConnectionManager.logger.debug(request.getId(), "Sending request :" + request.toString());
/*     */           }
/* 170 */           request.write(outputStream);
/*     */         }
/*     */         catch (Exception e) {
/* 173 */           throw new ARMARemoteCallException(HttpConnectionManager.logger, request.getId(), "Unable to write input into stream", e);
/*     */         }
/*     */         finally {
/* 176 */           outputStream.flush();
/* 177 */           outputStream.close();
/*     */         }
/*     */         
/* 180 */         inputStream = this.connection.getInputStream();
/* 181 */         String status = this.connection.getHeaderField("status");
/* 182 */         if ("SUCCESS".equals(status)) {
/* 183 */           response.read(inputStream);
/* 184 */           if (HttpConnectionManager.logger.isDebugEnabled()) {
/* 185 */             HttpConnectionManager.logger.debug(request.getId(), "Received response :" + response.toString());
/*     */           }
/* 187 */           return response;
/*     */         }
/*     */         
/* 190 */         HttpConnectionManager.logger.warn(request.getId(), "Server returned staus:" + status);
/*     */         
/*     */ 
/* 193 */         errorStream = this.connection.getErrorStream();
/*     */         
/* 195 */         if (null != errorStream)
/* 196 */           handleErrorStream(request.getId(), errorStream);
/* 197 */         if (null != inputStream)
/* 198 */           handleErrorStream(request.getId(), this.connection.getInputStream());
/* 199 */         throw new ARMARemoteCallException(HttpConnectionManager.logger, request.getId(), "Request to Remote server failed due to error :" + this.connection.getHeaderField("message"));
/*     */ 
/*     */       }
/*     */       catch (IOException e)
/*     */       {
/* 204 */         errorStream = this.connection.getErrorStream();
/* 205 */         String errorTrace = handleErrorStream(request.getId(), errorStream);
/*     */         
/* 207 */         if (errorTrace.contains("ARMASchemaOutOfSyncException")) {
/* 208 */           throw new ARMASchemaOutOfSyncException(HttpConnectionManager.logger, request.getId(), errorTrace);
/*     */         }
/* 210 */         throw new ARMARemoteCallException(HttpConnectionManager.logger, request.getId(), "Exception in post request to server. see debug logs for details caused by : " + e.getMessage(), e);
/*     */       }
/*     */       finally
/*     */       {
/* 214 */         closeStreams(request.getId(), inputStream, errorStream);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */     private synchronized void releaseConnection()
/*     */     {
/* 221 */       HttpConnectionManager.this.availableConnections.release();
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */     public void closeStreams(String id, InputStream inputStream, InputStream errorStream)
/*     */     {
/*     */       try
/*     */       {
/* 230 */         if (errorStream != null) {
/*     */           try {
/* 232 */             errorStream.close();
/*     */             
/*     */ 
/*     */ 
/* 236 */             if (inputStream != null) {
/*     */               try {
/* 238 */                 inputStream.close();
/*     */               } catch (IOException e) {
/* 240 */                 new ARMARemoteCallException(HttpConnectionManager.logger, id, "Unable to close request stream caused by :" + e.getMessage(), e);
/*     */               }
/*     */             }
/*     */           }
/*     */           catch (IOException e)
/*     */           {
/* 234 */             new ARMARemoteCallException(HttpConnectionManager.logger, id, "Unable to close request stream caused by :" + e.getMessage(), e);
/*     */           } finally {
/* 236 */             if (inputStream != null) {
/*     */               try {
/* 238 */                 inputStream.close();
/*     */               } catch (IOException e) {
/* 240 */                 new ARMARemoteCallException(HttpConnectionManager.logger, id, "Unable to close request stream caused by :" + e.getMessage(), e);
/*     */               }
/*     */             }
/*     */           }
/*     */         }
/*     */       } finally {
/* 246 */         releaseConnection();
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */     private String handleErrorStream(String id, InputStream errorStream)
/*     */       throws ARMARemoteCallException
/*     */     {
/* 254 */       StringBuilder sb = new StringBuilder();
/* 255 */       BufferedReader reader = new BufferedReader(new InputStreamReader(errorStream));
/* 256 */       String trace = null;
/*     */       try {
/* 258 */         while ((trace = reader.readLine()) != null) {
/* 259 */           sb.append(trace);
/* 260 */           sb.append("\n");
/*     */         }
/*     */       } catch (IOException e1) {
/* 263 */         throw new ARMARemoteCallException(HttpConnectionManager.logger, id, "Unable to post read error stream caused by :" + e1.getMessage(), e1);
/*     */       }
/* 265 */       HttpConnectionManager.logger.debug(id, "Uncaught server exception with following trace:" + sb.toString());
/* 266 */       return sb.toString();
/*     */     }
/*     */     
/*     */     public InputStream getRequest(String metamodelVersion) throws ARMAMetaModelException, ARMARemoteCallException
/*     */     {
/* 271 */       InputStream inputStream = null;
/* 272 */       InputStream errorStream = null;
/*     */       try {
/* 274 */         this.connection.setRequestProperty("metamodelVersion", metamodelVersion);
/* 275 */         inputStream = this.connection.getInputStream();
/* 276 */         String status = this.connection.getHeaderField("status");
/* 277 */         if ("SUCCESS".equals(status)) {
/* 278 */           if (HttpConnectionManager.logger.isDebugEnabled()) {
/* 279 */             HttpConnectionManager.logger.debug(null, "Received response :");
/*     */           }
/* 281 */           return inputStream;
/*     */         }
/*     */         
/* 284 */         throw new ARMAMetaModelException(HttpConnectionManager.logger, "Request to Remote server failed due to error :" + this.connection.getHeaderField("message") + ",for version " + metamodelVersion);
/*     */ 
/*     */       }
/*     */       catch (IOException e)
/*     */       {
/* 289 */         errorStream = this.connection.getErrorStream();
/* 290 */         handleErrorStream(null, errorStream);
/* 291 */         closeStreams(null, inputStream, errorStream);
/* 292 */         throw new ARMARemoteCallException(HttpConnectionManager.logger, null, "Exception in fetching the response from the server :" + e.getMessage(), e);
/*     */       }
/*     */       finally
/*     */       {
/* 296 */         releaseConnection();
/*     */       }
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\data\util\HttpConnectionManager.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */