/*     */ package com.netapp.asup.filter;
/*     */ 
/*     */ import com.netapp.asup.client.arma.data.ARMAObjectInstance;
/*     */ import com.netapp.asup.client.arma.data.ARMAObjectInstanceIterator;
/*     */ import com.netapp.asup.common.AsupLogger;
/*     */ import com.netapp.asup.exception.ARMAException;
/*     */ import com.netapp.asup.exception.ARMAIllegalArgumentException;
/*     */ import com.netapp.asupdata.client.ARMA.ARMAObjectInstanceImpl;
/*     */ import java.util.Arrays;
/*     */ import java.util.HashMap;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.Stack;
/*     */ import org.apache.avro.util.Utf8;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class ARMAPathFilterIterator
/*     */ {
/*     */   private static final char HASH_DELIMITER = '_';
/*  23 */   private static AsupLogger logger = AsupLogger.getLogger(ARMAPathFilterIterator.class);
/*     */   
/*     */ 
/*  26 */   private ARMAObjectInstance currentInstance = null;
/*     */   
/*     */   private final Stack<StackObject> workingStack;
/*     */   private final List<String> objects;
/*     */   private final String rootObject;
/*     */   private int currentObjectIndex;
/*     */   private final int maxObjectIndex;
/*  33 */   private int instanceCounter = 0;
/*     */   
/*     */   private static final int MAX_INSTANCE_COUNT = 50000;
/*     */   private Map<Utf8, Map<Utf8, Utf8>> filters;
/*     */   private FilterMetaData metaData;
/*  38 */   private Map<Integer, ARMAObjectInstance> currInstancesChain = new HashMap();
/*  39 */   private Map<String, ParentInstance> parentInstancesChain = new HashMap();
/*     */   
/*     */   private class ParentInstance
/*     */   {
/*  43 */     private int childCount = 0;
/*  44 */     private int filteredCount = 0;
/*     */     private ARMAObjectInstance parentInstance;
/*     */     private String hashKey;
/*     */     private String preParentHashKey;
/*     */     
/*     */     private ParentInstance(int childCount, String hashkey, ARMAObjectInstance instance, String preParentHashKey) {
/*  50 */       this.childCount = childCount;
/*  51 */       this.parentInstance = instance;
/*  52 */       this.hashKey = hashkey;
/*  53 */       this.preParentHashKey = preParentHashKey;
/*     */     }
/*     */     
/*     */     public void decrement() {
/*  57 */       this.filteredCount += 1;
/*  58 */       if (this.childCount == this.filteredCount)
/*     */       {
/*  60 */         ARMAObjectInstanceImpl instance = (ARMAObjectInstanceImpl)this.parentInstance;
/*  61 */         ARMAPathFilterIterator.this.metaData.removeParentInstances(ARMAPathFilterIterator.this.rootObject + ',' + ARMAPathFilterIterator.this.objects, instance);
/*     */         
/*  63 */         if (null != this.preParentHashKey)
/*     */         {
/*  65 */           if (ARMAPathFilterIterator.logger.isDebugEnabled()) {
/*  66 */             ARMAPathFilterIterator.logger.debug("-9999", "Decrementing the parent hash:" + this.preParentHashKey + ",for the child instance key:" + instance.getInstanceKey());
/*     */           }
/*  68 */           ParentInstance prevParent = (ParentInstance)ARMAPathFilterIterator.this.parentInstancesChain.get(this.preParentHashKey);
/*  69 */           prevParent.decrement();
/*     */         }
/*  71 */         else if (this.parentInstance.getObjectName() != ARMAPathFilterIterator.this.rootObject) {
/*  72 */           ARMAPathFilterIterator.logger.warn("-9999", "Null parent hash,Potential problem:" + this.parentInstance.getKey());
/*     */         }
/*     */       }
/*  75 */       if (this.filteredCount > this.childCount) {
/*  76 */         ARMAPathFilterIterator.logger.warn("-9999", "Filtered count cannot be more than child count for instance:" + this.parentInstance.getKey().toString());
/*     */       }
/*     */     }
/*     */     
/*     */     public String getHashKey() {
/*  81 */       return this.hashKey;
/*     */     }
/*     */     
/*     */     public String toString()
/*     */     {
/*  86 */       StringBuilder msg = new StringBuilder("{");
/*  87 */       msg.append("Child count :" + this.childCount);
/*  88 */       msg.append(",filtered count :" + this.filteredCount);
/*  89 */       msg.append(".Current Hash:" + this.hashKey);
/*  90 */       msg.append(",Parent Hash:" + this.preParentHashKey);
/*  91 */       msg.append(",Parent InstanceKey:" + this.parentInstance.getKey().toString());
/*  92 */       msg.append("}");
/*  93 */       return msg.toString();
/*     */     }
/*     */   }
/*     */   
/*     */   private static class StackObject
/*     */   {
/*     */     private final ARMAObjectInstanceIterator iterator;
/*     */     private final ARMAObjectInstance instance;
/*     */     private final boolean hasMultipleInstances;
/*     */     private final String objectName;
/*     */     
/*     */     private StackObject(String objectName, ARMAObjectInstanceIterator iterator) {
/* 105 */       this.objectName = objectName;
/* 106 */       this.iterator = iterator;
/* 107 */       this.instance = null;
/* 108 */       this.hasMultipleInstances = true;
/*     */     }
/*     */     
/*     */     private StackObject(String objectName, ARMAObjectInstance instance) {
/* 112 */       this.objectName = objectName;
/* 113 */       this.iterator = null;
/* 114 */       this.instance = instance;
/* 115 */       this.hasMultipleInstances = false;
/*     */     }
/*     */     
/*     */     public String toString()
/*     */     {
/* 120 */       return this.objectName;
/*     */     }
/*     */   }
/*     */   
/*     */   public ARMAPathFilterIterator(FilterMetaData metaData, Map<Utf8, Map<Utf8, Utf8>> filters, ARMAObjectInstance rootInstance, String[] objects)
/*     */     throws ARMAException
/*     */   {
/* 127 */     this.rootObject = rootInstance.getObjectName();
/* 128 */     this.objects = Arrays.asList(objects);
/* 129 */     this.workingStack = new Stack();
/* 130 */     this.maxObjectIndex = this.objects.size();
/* 131 */     this.currentObjectIndex = 0;
/* 132 */     this.filters = filters;
/* 133 */     this.metaData = metaData;
/* 134 */     this.currInstancesChain.put(Integer.valueOf(this.currentObjectIndex), rootInstance);
/* 135 */     String objectName = null;
/* 136 */     if (this.objects.size() > this.currentObjectIndex) {
/* 137 */       objectName = (String)this.objects.get(this.currentObjectIndex);
/*     */     }
/* 139 */     this.currentInstance = getFilteredInstance(rootInstance, objectName);
/*     */   }
/*     */   
/*     */   private ParentInstance getParentInstance(ARMAObjectInstance rootInstance, int childCount) {
/* 143 */     if (this.currentObjectIndex == 0) {
/* 144 */       return new ParentInstance(childCount, getHash(rootInstance, this.currentObjectIndex), rootInstance, null, null);
/*     */     }
/* 146 */     int previousIndex = this.currentObjectIndex - 1;
/* 147 */     ARMAObjectInstance parentObjectInstance = (ARMAObjectInstance)this.currInstancesChain.get(Integer.valueOf(previousIndex));
/* 148 */     return new ParentInstance(childCount, getHash(rootInstance, this.currentObjectIndex), rootInstance, getHash(parentObjectInstance, previousIndex), null);
/*     */   }
/*     */   
/*     */   private ARMAObjectInstance getNextInstance(String objectName, ARMAObjectInstance instance)
/*     */     throws ARMAException
/*     */   {
/* 154 */     if (instance.isParent(objectName)) {
/* 155 */       ARMAObjectInstance parentInstance = instance.getParentInstance(objectName);
/* 156 */       ParentInstance parentIns = getParentInstance(instance, 1);
/* 157 */       String key = parentIns.getHashKey();
/* 158 */       if (!this.parentInstancesChain.containsKey(key)) {
/* 159 */         this.parentInstancesChain.put(key, parentIns);
/*     */       }
/*     */       else {
/* 162 */         logger.warn("-9999", "Same parent hash,Potential problem. Existing hash:" + this.parentInstancesChain.get(key) + ",Incomming hash:" + parentIns);
/*     */       }
/* 164 */       if (parentInstance != null) {
/* 165 */         StackObject object = new StackObject(objectName, parentInstance, null);
/* 166 */         this.workingStack.push(object);
/* 167 */         this.currentObjectIndex += 1;
/* 168 */         return getNextInstance();
/*     */       }
/* 170 */       handleNoChildInstances(instance);
/* 171 */       updateStackAndCurrentIndex();
/* 172 */       return null;
/*     */     }
/* 174 */     if (instance.isChild(objectName)) {
/* 175 */       ARMAObjectInstanceIterator iterator = instance.getChildInstances(objectName);
/* 176 */       ParentInstance parentIns = getParentInstance(instance, iterator.size());
/* 177 */       String key = parentIns.getHashKey();
/* 178 */       if (!this.parentInstancesChain.containsKey(key)) {
/* 179 */         this.parentInstancesChain.put(key, parentIns);
/*     */       }
/*     */       else {
/* 182 */         logger.warn("-9999", "Same parent hash,Potential problem. Existing hash:" + this.parentInstancesChain.get(key) + ",Incomming hash:" + parentIns);
/*     */       }
/* 184 */       if (0 == iterator.size()) {
/* 185 */         handleNoChildInstances(instance);
/*     */       }
/* 187 */       StackObject object = new StackObject(objectName, iterator, null);
/* 188 */       this.workingStack.push(object);
/* 189 */       this.currentObjectIndex += 1;
/* 190 */       return getNextInstance();
/*     */     }
/* 192 */     throw new ARMAException(logger, objectName + ":" + instance.getKey().toString(), "Invalid path, The object :" + objectName + " is neither parent nor child to object :" + instance.getObjectName());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private void handleNoChildInstances(ARMAObjectInstance instance)
/*     */   {
/* 202 */     for (int i = this.currentObjectIndex; i < this.objects.size(); i++) {
/* 203 */       if (this.filters.containsKey(new Utf8(((String)this.objects.get(i)).toUpperCase()))) {
/* 204 */         removePreviousInchain(this.currentObjectIndex);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   private ARMAObjectInstance getNextInstance() throws ARMAException
/*     */   {
/* 211 */     if (50000 == this.instanceCounter) {
/* 212 */       throw new ARMAIllegalArgumentException(logger, "The path iteration reached the maximum limit:50000 instances.Please chek the path:{" + this.rootObject + ',' + this.objects + '}');
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 217 */     if (this.workingStack.isEmpty()) {
/* 218 */       return null;
/*     */     }
/*     */     
/* 221 */     if (this.currentObjectIndex == this.maxObjectIndex) {
/* 222 */       StackObject stackObject = (StackObject)this.workingStack.peek();
/* 223 */       if (stackObject.hasMultipleInstances) {
/* 224 */         if (stackObject.iterator.hasNext()) {
/* 225 */           this.instanceCounter += 1;
/* 226 */           return getFilteredInstance(stackObject.iterator.next(), null);
/*     */         }
/*     */         
/* 229 */         this.workingStack.pop();
/* 230 */         this.currentObjectIndex -= 1;
/* 231 */         updateStackAndCurrentIndex();
/* 232 */         return null;
/*     */       }
/*     */       
/* 235 */       this.workingStack.pop();
/* 236 */       this.currentObjectIndex -= 1;
/* 237 */       updateStackAndCurrentIndex();
/* 238 */       this.instanceCounter += 1;
/* 239 */       return getFilteredInstance(stackObject.instance, null);
/*     */     }
/*     */     
/* 242 */     StackObject stackObject = (StackObject)this.workingStack.peek();
/* 243 */     String objectName = (String)this.objects.get(this.currentObjectIndex);
/* 244 */     if (stackObject.hasMultipleInstances) {
/* 245 */       if (stackObject.iterator.hasNext()) {
/* 246 */         this.instanceCounter += 1;
/* 247 */         return getFilteredIteratorInstance(stackObject, objectName);
/*     */       }
/* 249 */       this.workingStack.pop();
/* 250 */       this.currentObjectIndex -= 1;
/* 251 */       updateStackAndCurrentIndex();
/* 252 */       return null;
/*     */     }
/*     */     
/* 255 */     ARMAObjectInstance instance = stackObject.instance;
/* 256 */     this.instanceCounter += 1;
/* 257 */     return getFilteredInstance(instance, objectName);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   private ARMAObjectInstance getFilteredInstance(ARMAObjectInstance instance, String objectName)
/*     */     throws ARMAException
/*     */   {
/* 265 */     this.currInstancesChain.put(Integer.valueOf(this.currentObjectIndex), instance);
/* 266 */     if (logger.isDebugEnabled()) {
/* 267 */       logger.debug("-9999", "ObjectName:" + instance.getObjectName() + ",InstanceId:" + instance.getKey());
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 272 */     if (FilterUtil.isMatching(instance, this.filters)) {
/* 273 */       this.metaData.addInstance((ARMAObjectInstanceImpl)instance);
/* 274 */       if (null != objectName) {
/* 275 */         return getNextInstance(objectName, instance);
/*     */       }
/* 277 */       return instance;
/*     */     }
/* 279 */     removePreviousInchain(this.currentObjectIndex);
/* 280 */     return getNextInstance();
/*     */   }
/*     */   
/*     */ 
/*     */   private void removePreviousInchain(int currentObjIndex)
/*     */   {
/* 286 */     int index = currentObjIndex - 1;
/* 287 */     ARMAObjectInstanceImpl armaObjectInstance = (ARMAObjectInstanceImpl)this.currInstancesChain.get(Integer.valueOf(currentObjIndex));
/*     */     
/*     */ 
/* 290 */     this.metaData.removeInstance(armaObjectInstance);
/*     */     
/*     */ 
/* 293 */     if (index >= 0) {
/* 294 */       ARMAObjectInstanceImpl parentObjectInstance = (ARMAObjectInstanceImpl)this.currInstancesChain.get(Integer.valueOf(index));
/* 295 */       String hashKey = getHash(parentObjectInstance, index);
/* 296 */       ParentInstance parentInstance = (ParentInstance)this.parentInstancesChain.get(hashKey);
/* 297 */       parentInstance.decrement();
/*     */     }
/*     */   }
/*     */   
/*     */   String getHash(ARMAObjectInstance armaObjectInstance, int i)
/*     */   {
/* 303 */     StringBuilder hash = new StringBuilder();
/* 304 */     hash.append(i);
/* 305 */     hash.append('_');
/* 306 */     hash.append(armaObjectInstance.getObjectName());
/* 307 */     hash.append('_');
/* 308 */     hash.append(armaObjectInstance.getKey());
/* 309 */     return hash.toString();
/*     */   }
/*     */   
/*     */   private ARMAObjectInstance getFilteredIteratorInstance(StackObject stackObject, String objectName) throws ARMAException
/*     */   {
/* 314 */     ARMAObjectInstance instance = stackObject.iterator.next();
/*     */     
/* 316 */     this.currInstancesChain.put(Integer.valueOf(this.currentObjectIndex), instance);
/*     */     
/* 318 */     if (logger.isDebugEnabled()) {
/* 319 */       logger.debug("-9999", "ObjectName:" + instance.getObjectName() + ",InstanceId:" + instance.getKey());
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 324 */     if (FilterUtil.isMatching(instance, this.filters)) {
/* 325 */       this.metaData.addInstance((ARMAObjectInstanceImpl)instance);
/* 326 */       return getNextInstance(objectName, instance);
/*     */     }
/* 328 */     removePreviousInchain(this.currentObjectIndex);
/* 329 */     return getNextInstance();
/*     */   }
/*     */   
/*     */   private void updateStackAndCurrentIndex()
/*     */   {
/* 334 */     while (!this.workingStack.isEmpty()) {
/* 335 */       StackObject peek = (StackObject)this.workingStack.peek();
/* 336 */       if (peek.hasMultipleInstances) {
/*     */         break;
/*     */       }
/* 339 */       this.workingStack.pop();
/* 340 */       this.currentObjectIndex -= 1;
/*     */     }
/*     */   }
/*     */   
/*     */   public boolean hasNext() throws ARMAException
/*     */   {
/* 346 */     while ((this.currentInstance == null) && (!this.workingStack.isEmpty())) {
/* 347 */       this.currentInstance = getNextInstance();
/*     */     }
/* 349 */     if (this.currentInstance == null) {
/* 350 */       return false;
/*     */     }
/* 352 */     return true;
/*     */   }
/*     */   
/*     */   public ARMAObjectInstance next() throws ARMAException
/*     */   {
/* 357 */     while ((this.currentInstance == null) && (!this.workingStack.isEmpty())) {
/* 358 */       this.currentInstance = getNextInstance();
/*     */     }
/* 360 */     ARMAObjectInstance currentInstance = this.currentInstance;
/* 361 */     this.currentInstance = null;
/* 362 */     return currentInstance;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\filter\ARMAPathFilterIterator.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */