/*     */ package com.netapp.asup.filter;
/*     */ 
/*     */ import com.netapp.asup.client.arma.ARMAObject;
/*     */ import com.netapp.asup.client.arma.data.ARMAObjectInstanceIterator;
/*     */ import com.netapp.asup.common.AsupLogger;
/*     */ import com.netapp.asup.data.objects.ARMASchema;
/*     */ import com.netapp.asup.data.objects.Identifier;
/*     */ import com.netapp.asup.data.objects.ObjectData;
/*     */ import com.netapp.asup.data.objects.Objects;
/*     */ import com.netapp.asup.data.objects.ObjectsRequest;
/*     */ import com.netapp.asup.data.objects.Request;
/*     */ import com.netapp.asup.data.objects.Response;
/*     */ import com.netapp.asup.exception.ARMADataStoreException;
/*     */ import com.netapp.asup.exception.ARMAException;
/*     */ import com.netapp.asupdata.client.ARMA.ARMAClientSession;
/*     */ import com.netapp.asupdata.client.ARMA.ARMAObjectFactoryImpl;
/*     */ import com.netapp.asupdata.client.ARMA.ARMAObjectImpl;
/*     */ import com.netapp.asupdata.client.ARMA.ARMAObjectInstanceImpl;
/*     */ import java.io.IOException;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Arrays;
/*     */ import java.util.HashSet;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.Set;
/*     */ import org.apache.avro.generic.GenericData.Array;
/*     */ import org.apache.avro.util.Utf8;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class DataFilterProcessor
/*     */ {
/*  34 */   private static final AsupLogger LOGGER = AsupLogger.getLogger(DataFilterProcessor.class);
/*     */   
/*     */ 
/*     */   public void addDependentObjects(Request request)
/*     */   {
/*  39 */     Identifier identifier = request.getIdentifier();
/*  40 */     Map<Utf8, Map<Utf8, Utf8>> filters = identifier.getFilter();
/*     */     
/*  42 */     if ((!Boolean.valueOf(identifier.isIdentifierProcessed()).booleanValue()) && (!filters.isEmpty()))
/*     */     {
/*  44 */       GenericData.Array<Utf8> paths = identifier.getPath();
/*     */       
/*  46 */       Set<String> objectsRequested = FilterUtil.getRequestedObjects(request.getRequestObject().getObjects());
/*     */       
/*     */ 
/*  49 */       Set<String> objects = FilterUtil.addFilterObjects(request.getRequestObject().getObjects(), filters);
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  55 */       Set<Utf8> dependentObjects = new HashSet();
/*  56 */       dependentObjects.addAll(filters.keySet());
/*  57 */       Set<Utf8> trimmedPaths = new HashSet();
/*     */       
/*     */ 
/*  60 */       for (Utf8 connectedPath : paths) {
/*  61 */         StringBuilder filteredPaths = FilterUtil.getFilteredPaths(connectedPath, objects, objectsRequested);
/*     */         
/*  63 */         if (0 != filteredPaths.length()) {
/*  64 */           Utf8 trimmedPath = new Utf8(filteredPaths.toString());
/*  65 */           trimmedPaths.add(trimmedPath);
/*  66 */           dependentObjects.addAll(FilterUtil.getDependentObjectsFromPath(trimmedPath, objects, objectsRequested));
/*     */         }
/*     */       }
/*     */       
/*     */ 
/*  71 */       addDependentObjsAndTrimmedPathsToReq(request, dependentObjects, trimmedPaths);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public void applyFilters(Request request, Response responseData)
/*     */     throws ARMADataStoreException
/*     */   {
/*  79 */     Map<Utf8, Map<Utf8, Utf8>> filters = request.getIdentifier().getFilter();
/*     */     
/*     */ 
/*  82 */     if ((!Boolean.valueOf(responseData.isFiltersApplied()).booleanValue()) && (!filters.isEmpty()))
/*     */     {
/*     */ 
/*  85 */       long startTime = System.currentTimeMillis();
/*     */       try
/*     */       {
/*  88 */         responseData.setFiltersApplied(String.valueOf(Boolean.TRUE));
/*  89 */         ARMAObject graphRoot = getGraphRoot(request, responseData);
/*     */         
/*  91 */         Set<Utf8> reqObjsPaths = new HashSet(request.getIdentifier().getPath());
/*     */         
/*  93 */         FilterMetaData filterMetaData = applyFiltersForReqObjects(request, graphRoot, reqObjsPaths, filters);
/*     */         
/*     */ 
/*  96 */         applyOnResponse(request, graphRoot, responseData, filterMetaData);
/*     */       }
/*     */       catch (Exception e) {
/*  99 */         throw new ARMADataStoreException(LOGGER, request.getId(), "Error occured while applying filters", e);
/*     */       }
/*     */       
/* 102 */       long endTime = System.currentTimeMillis();
/*     */       
/* 104 */       LOGGER.info(request.getId(), "Total time taken for applying filters in (ms):" + (endTime - startTime));
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   private void applyOnResponse(Request request, ARMAObject graphRoot, Response responseData, FilterMetaData filterMetaData)
/*     */     throws IOException, ARMAException
/*     */   {
/* 113 */     Map<String, Set<InstanceKey>> filteredResult = filterMetaData.consolidate();
/*     */     
/*     */ 
/*     */ 
/* 117 */     Objects objects = responseData.getObjects();
/* 118 */     for (String objName : filteredResult.keySet()) {
/* 119 */       ObjectData objectData = objects.getObjectData(new Utf8(objName.toUpperCase()));
/*     */       
/* 121 */       ObjectData filteredData = objectData.retainInstances((Set)filteredResult.get(objName));
/*     */       
/*     */ 
/* 124 */       objects.putObjectData(filteredData);
/*     */     }
/*     */     
/*     */ 
/* 128 */     List<Utf8> reqObject = request.getRequestObject().getObjects();
/* 129 */     Set<Utf8> responseObjs = objects.getObjectSet();
/* 130 */     for (Utf8 responseObj : responseObjs) {
/* 131 */       if (!reqObject.contains(responseObj)) {
/* 132 */         objects.removeObjectData(responseObj);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   private FilterMetaData applyFiltersForReqObjects(Request request, ARMAObject graphRoot, Set<Utf8> reqObjsPaths, Map<Utf8, Map<Utf8, Utf8>> filters)
/*     */     throws Exception
/*     */   {
/* 140 */     List<Utf8> reqObjs = request.getRequestObject().getObjects();
/* 141 */     FilterMetaData allPathFilterMetaData = new FilterMetaData();
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 146 */     Utf8[] reqObjsPathArray = (Utf8[])reqObjsPaths.toArray(new Utf8[reqObjsPaths.size()]);
/*     */     
/* 148 */     for (int i = 0; i < reqObjsPathArray.length; i++) {
/* 149 */       Utf8 reqObjsPath = reqObjsPathArray[i];
/* 150 */       String[] curPathObjects = FilterUtil.getObjects(reqObjsPath);
/* 151 */       if (curPathObjects.length != 0) {
/* 152 */         ARMAObject pathRootNode = graphRoot.jumpToNode(curPathObjects[0]);
/* 153 */         if (0 == i) {
/* 154 */           allPathFilterMetaData = applyFilterOnPath(reqObjs, pathRootNode, filters, curPathObjects);
/*     */         }
/*     */         else {
/* 157 */           allPathFilterMetaData = allPathFilterMetaData.intersect(reqObjs, applyFilterOnPath(reqObjs, pathRootNode, filters, curPathObjects));
/*     */         }
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 165 */     return allPathFilterMetaData;
/*     */   }
/*     */   
/*     */ 
/*     */   private FilterMetaData applyFilterOnPath(List<Utf8> reqObjs, ARMAObject pathRootNode, Map<Utf8, Map<Utf8, Utf8>> filters, String[] curPathObjects)
/*     */     throws ARMAException
/*     */   {
/* 172 */     FilterMetaData metaData = new FilterMetaData(getRequestObjectInPath(reqObjs, curPathObjects));
/*     */     
/* 174 */     String[] pathObjects = new String[0];
/* 175 */     if (curPathObjects.length > 1) {
/* 176 */       pathObjects = (String[])Arrays.copyOfRange(curPathObjects, 1, curPathObjects.length);
/*     */     }
/*     */     
/* 179 */     ARMAObjectInstanceIterator instances = pathRootNode.getInstances();
/*     */     
/*     */ 
/* 182 */     while (instances.hasNext()) {
/* 183 */       ARMAObjectInstanceImpl instance = (ARMAObjectInstanceImpl)instances.next();
/*     */       
/* 185 */       if (LOGGER.isDebugEnabled()) {
/* 186 */         LOGGER.debug("-9999", "ObjectName:" + instance.getObjectName() + ",InstanceId:" + instance.getKey());
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 191 */       ARMAPathFilterIterator armaPathFilterIterator = new ARMAPathFilterIterator(metaData, filters, instance, pathObjects);
/*     */       
/* 193 */       while (armaPathFilterIterator.hasNext()) {
/* 194 */         armaPathFilterIterator.next();
/*     */       }
/*     */     }
/*     */     
/* 198 */     return metaData;
/*     */   }
/*     */   
/*     */ 
/*     */   private List<Utf8> getRequestObjectInPath(List<Utf8> reqObjs, String[] curPathObjects)
/*     */   {
/* 204 */     List<Utf8> reqObjsInPath = new ArrayList();
/* 205 */     List<String> curPathObjs = Arrays.asList(curPathObjects);
/*     */     
/* 207 */     for (Utf8 reqObj : reqObjs) {
/* 208 */       if (curPathObjs.contains(reqObj.toString().toLowerCase())) {
/* 209 */         reqObjsInPath.add(reqObj);
/*     */       }
/*     */     }
/*     */     
/* 213 */     return reqObjsInPath;
/*     */   }
/*     */   
/*     */   private ARMAObject getGraphRoot(Request request, Response responseData) throws ARMAException
/*     */   {
/* 218 */     Identifier identifier = request.getIdentifier();
/* 219 */     ARMAObjectFactoryImpl armaObjectFactory = new ARMAObjectFactoryImpl();
/* 220 */     ARMAObjectImpl graphRoot = (ARMAObjectImpl)armaObjectFactory.getGraphRoot(identifier, Boolean.valueOf(identifier.getClusterRoot()).booleanValue());
/*     */     
/*     */ 
/* 223 */     ARMAClientSession session = graphRoot.getSession();
/* 224 */     session.populateResponse(responseData);
/* 225 */     return graphRoot;
/*     */   }
/*     */   
/*     */ 
/*     */   private void addDependentObjsAndTrimmedPathsToReq(Request request, Set<Utf8> dependentObjects, Set<Utf8> trimmedPaths)
/*     */   {
/* 231 */     GenericData.Array<Utf8> depObjs = new GenericData.Array(dependentObjects.size(), ARMASchema.DEPENDENT_OBJECTS.getSchema());
/*     */     
/*     */ 
/* 234 */     depObjs.addAll(dependentObjects);
/* 235 */     request.setDependentObjects(depObjs);
/*     */     
/* 237 */     if (LOGGER.isDebugEnabled()) {
/* 238 */       LOGGER.debug(request.getId(), "Dependenet Objects:" + depObjs);
/*     */     }
/* 240 */     GenericData.Array<Utf8> avroPaths = new GenericData.Array(trimmedPaths.size(), ARMASchema.PATH.getSchema());
/*     */     
/* 242 */     avroPaths.addAll(trimmedPaths);
/*     */     
/* 244 */     Identifier identifier = request.getIdentifier();
/* 245 */     identifier.setPath(avroPaths);
/*     */     
/* 247 */     if (LOGGER.isDebugEnabled()) {
/* 248 */       LOGGER.debug(request.getId(), "Trimmed Paths:" + avroPaths);
/*     */     }
/* 250 */     identifier.setIdentifierProcessed(String.valueOf(Boolean.TRUE));
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\filter\DataFilterProcessor.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */