/*     */ package com.netapp.asupdata.client.ARMA;
/*     */ 
/*     */ import com.netapp.asup.avro.AvroReader;
/*     */ import com.netapp.asup.client.arma.ARMAFilter;
/*     */ import com.netapp.asup.client.arma.ARMAObject;
/*     */ import com.netapp.asup.client.arma.ARMAViewOnlyObject;
/*     */ import com.netapp.asup.client.arma.data.ARMAObjectInstance;
/*     */ import com.netapp.asup.client.arma.data.ARMAObjectInstanceIterator;
/*     */ import com.netapp.asup.client.helper.ObjectDataXmlHelper;
/*     */ import com.netapp.asup.common.AsupLogger;
/*     */ import com.netapp.asup.data.function.FunctionOutputHandler;
/*     */ import com.netapp.asup.data.function.FunctionRegistry;
/*     */ import com.netapp.asup.data.function.RegisteredFunction;
/*     */ import com.netapp.asup.data.objects.Filter;
/*     */ import com.netapp.asup.data.objects.FunctionData;
/*     */ import com.netapp.asup.data.objects.FunctionRequest;
/*     */ import com.netapp.asup.data.objects.Identifier;
/*     */ import com.netapp.asup.data.objects.ObjectData;
/*     */ import com.netapp.asup.data.objects.Request;
/*     */ import com.netapp.asup.data.schema.SchemaHelper;
/*     */ import com.netapp.asup.data.store.BaseContainer;
/*     */ import com.netapp.asup.data.store.StoreContainer;
/*     */ import com.netapp.asup.exception.ARMADataStoreException;
/*     */ import com.netapp.asup.exception.ARMAException;
/*     */ import com.netapp.asup.exception.ARMAFunctionExecutionException;
/*     */ import com.netapp.asup.exception.ARMAIllegalArgumentException;
/*     */ import com.netapp.asup.metamodel.NodeObject;
/*     */ import com.netapp.asup.util.StringUtils;
/*     */ import com.netapp.asupdata.client.ARMA.parsers.GraphParser;
/*     */ import java.io.IOException;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Arrays;
/*     */ import java.util.Collection;
/*     */ import java.util.HashMap;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.generic.GenericRecord;
/*     */ import org.apache.avro.util.Utf8;
/*     */ 
/*     */ public class ARMAObjectImpl
/*     */   implements ARMAObject
/*     */ {
/*     */   private final ARMAClientSession session;
/*  45 */   private static final FunctionRegistry functionRegistory = BaseContainer.getInstance().getFunctionRegistry();
/*     */   
/*     */   private final NodeObject nodeObject;
/*     */   
/*     */   private ARMAPath path;
/*     */   
/*  51 */   private static final AsupLogger LOG = AsupLogger.getLogger(ARMAObjectImpl.class);
/*     */   
/*     */ 
/*  54 */   private GraphParser parser = GraphParser.getInstance();
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   private final String objectName;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public ARMAObjectImpl(ARMAClientSession session, NodeObject nodeObject)
/*     */   {
/*  66 */     this.nodeObject = nodeObject;
/*  67 */     this.session = session;
/*  68 */     this.objectName = nodeObject.getName();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public ARMAObjectImpl(ARMAClientSession session, ARMAObject myPrevious, NodeObject object, NodeType nodeType)
/*     */   {
/*  82 */     this(session, object);
/*  83 */     session.traceMyPath(this, myPrevious, nodeType);
/*     */   }
/*     */   
/*     */   public void setPath(ARMAPath path) {
/*  87 */     this.path = path;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public ARMAObject getRoot()
/*     */   {
/*  96 */     return this.session.getRoot();
/*     */   }
/*     */   
/*     */ 
/*     */   public Collection<ARMAObject> getParents()
/*     */   {
/* 102 */     return this.parser.getAllParents(this.session, this, this.nodeObject, false);
/*     */   }
/*     */   
/*     */ 
/*     */   public ARMAObject getParent(String objectName)
/*     */   {
/* 108 */     if (isInvalidObjectName(objectName))
/* 109 */       return null;
/* 110 */     return this.parser.getParentNodeByName(this.session, this, this.nodeObject, objectName, false);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public Collection<ARMAObject> getChilds()
/*     */   {
/* 117 */     return this.parser.getAllChilds(this.session, this, this.nodeObject, false);
/*     */   }
/*     */   
/*     */ 
/*     */   public ARMAObject getChild(String objectName)
/*     */   {
/* 123 */     if (isInvalidObjectName(objectName))
/* 124 */       return null;
/* 125 */     return this.parser.getChildNodeByName(this.session, this, this.nodeObject, objectName, false);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public Collection<ARMAObject> getAll()
/*     */   {
/* 132 */     return this.parser.getAllNodes(this.session, this, this.nodeObject, false);
/*     */   }
/*     */   
/*     */ 
/*     */   public ARMAObject getAll(String objectName)
/*     */   {
/* 138 */     if (isInvalidObjectName(objectName))
/* 139 */       return null;
/* 140 */     return this.parser.getFromAllByName(this.session, this, this.nodeObject, objectName, false);
/*     */   }
/*     */   
/*     */ 
/*     */   public ARMAFilter getFilter()
/*     */   {
/* 146 */     return this.session.getFilter().getFilter(this);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public String getKeyAttribute()
/*     */   {
/* 154 */     return this.nodeObject.getKeyAttribute();
/*     */   }
/*     */   
/*     */   public ARMAObject jumpToNode(String objectName) throws ARMAException {
/* 158 */     if (isNotValidString(objectName)) {
/* 159 */       throw new ARMAIllegalArgumentException(LOG, "object name is null or empty:" + objectName);
/*     */     }
/*     */     
/* 162 */     return this.parser.jumpToNode(getSession(), objectName);
/*     */   }
/*     */   
/*     */   public ARMAObject jumpToPath(String... connectedPath) throws ARMAException
/*     */   {
/* 167 */     if (null == connectedPath) {
/* 168 */       throw new ARMAIllegalArgumentException(LOG, "Path Cannot be null:" + connectedPath);
/*     */     }
/*     */     
/* 171 */     if (connectedPath.length == 0) {
/* 172 */       throw new ARMAIllegalArgumentException(LOG, "Path Cannot be empty" + Arrays.asList(connectedPath));
/*     */     }
/*     */     
/*     */ 
/* 176 */     return this.parser.getLastNodeInPath(this, connectedPath);
/*     */   }
/*     */   
/*     */   public String getObjectName()
/*     */   {
/* 181 */     return this.objectName;
/*     */   }
/*     */   
/*     */   public String getVersion()
/*     */   {
/* 186 */     throw new UnsupportedOperationException("Version operation is not supported as of this arma version");
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int hashCode()
/*     */   {
/* 195 */     int prime = 31;
/* 196 */     int result = 1;
/* 197 */     int columnCount = getColumnCount();
/* 198 */     List<String> attributeNames = getAttributeNames();
/*     */     
/* 200 */     int columnHash = 1;
/* 201 */     for (int i = 0; i < columnCount; i++) {
/* 202 */       int columnValueHash = 0;
/* 203 */       if (null != get(getColumnName(i))) {
/* 204 */         columnValueHash = get(getColumnName(i)).hashCode();
/*     */       }
/* 206 */       columnHash += columnValueHash;
/*     */     }
/*     */     
/* 209 */     result = 31 * result + ((attributeNames == null) || (attributeNames.size() == 0) ? 0 : attributeNames.hashCode() + columnHash);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 214 */     return result;
/*     */   }
/*     */   
/*     */   private List<String> getAttributeNames() {
/* 218 */     return this.nodeObject.getAttributeNames();
/*     */   }
/*     */   
/*     */   public int getColumnCount() {
/* 222 */     return this.nodeObject.getColumnCount();
/*     */   }
/*     */   
/*     */   public String getColumnName(int index) {
/* 226 */     return this.nodeObject.getColumnName(index);
/*     */   }
/*     */   
/*     */   public Object get(String columnName) {
/* 230 */     return this.nodeObject.get(columnName);
/*     */   }
/*     */   
/*     */   public Object getValueAt(int index)
/*     */   {
/* 235 */     return this.nodeObject.getValueAt(index);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public boolean equals(Object obj)
/*     */   {
/* 242 */     if (this == obj)
/* 243 */       return true;
/* 244 */     if (obj == null)
/* 245 */       return false;
/* 246 */     if (getClass() != obj.getClass())
/*     */     {
/*     */ 
/* 249 */       if (!getClass().isInstance(obj)) {
/* 250 */         return false;
/*     */       }
/*     */     }
/* 253 */     ARMAObjectImpl other = (ARMAObjectImpl)obj;
/* 254 */     List<String> attributeNames = getAttributeNames();
/* 255 */     if (attributeNames == null) {
/* 256 */       if (other.getAttributeNames() != null)
/* 257 */         return false;
/* 258 */     } else { if (!attributeNames.equals(other.getAttributeNames())) {
/* 259 */         return false;
/*     */       }
/* 261 */       int columnCount = getColumnCount();
/*     */       
/* 263 */       for (int i = 0; i < columnCount; i++) {
/* 264 */         if (false == getValueAt(i).equals(other.getValueAt(i))) {
/* 265 */           return false;
/*     */         }
/*     */       }
/*     */     }
/*     */     
/* 270 */     return true;
/*     */   }
/*     */   
/*     */ 
/*     */   public String toString()
/*     */   {
/* 276 */     int columnCount = getColumnCount();
/* 277 */     StringBuilder dataRep = new StringBuilder();
/* 278 */     dataRep.append('{');
/* 279 */     for (int i = 0; i < columnCount; i++)
/*     */     {
/* 281 */       dataRep.append(get(getColumnName(i)));
/* 282 */       if (i + 1 != columnCount)
/* 283 */         dataRep.append(',');
/*     */     }
/* 285 */     dataRep.append('}');
/* 286 */     return dataRep.toString();
/*     */   }
/*     */   
/*     */ 
/*     */   public ARMAObject getObjectByName(String objectName)
/*     */   {
/* 292 */     return this.session.getObjectStore().getFromStore(objectName);
/*     */   }
/*     */   
/*     */ 
/*     */   public ARMAViewOnlyObject getViewOnlyARMAObject()
/*     */   {
/* 298 */     return this;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public ARMAObject getParentFromStore(String objectName)
/*     */   {
/* 307 */     return this.parser.getParentNodeByNameFromStore(this.session, this, objectName);
/*     */   }
/*     */   
/*     */   public Collection<ARMAObject> getAllParentsFromStore()
/*     */   {
/* 312 */     return this.parser.getAllParentsFromStore(this.session, this);
/*     */   }
/*     */   
/*     */   public ARMAObject getChildFromStore(String objectName)
/*     */   {
/* 317 */     return this.parser.getChildNodeByNameFromStore(this.session, this, objectName);
/*     */   }
/*     */   
/*     */   public Collection<ARMAObject> getAllChildsFromStore()
/*     */   {
/* 322 */     return this.parser.getAllChildsFromStore(this.session, this);
/*     */   }
/*     */   
/*     */   public ARMAPath getPathTraversed() {
/* 326 */     return this.path;
/*     */   }
/*     */   
/*     */   public ARMAClientSession getSession() {
/* 330 */     return this.session;
/*     */   }
/*     */   
/*     */   public String getRequestId() {
/* 334 */     return this.session.getRequestId();
/*     */   }
/*     */   
/*     */   public void updateMyPath(ARMAObject myPrevious, NodeType nodeType) {
/* 338 */     this.session.traceMyPath(this, myPrevious, nodeType);
/*     */   }
/*     */   
/*     */   public void getData(String... objectNames)
/*     */     throws ARMAException
/*     */   {
/* 344 */     this.session.getData(objectNames);
/*     */   }
/*     */   
/*     */   public Collection<String> getDataAttributes() throws ARMAException {
/* 348 */     StoreContainer storeContainer = BaseContainer.getInstance().getStoreContainer();
/*     */     
/* 350 */     SchemaHelper schemaHelper = storeContainer.getSchemaHelper();
/* 351 */     Schema objectSchema = schemaHelper.getLatestSchema(getObjectName().toUpperCase());
/*     */     
/* 353 */     return AvroReader.getSchemaAttributes(objectSchema);
/*     */   }
/*     */   
/*     */   public ARMAObjectInstanceIterator getInstances() throws ARMAException
/*     */   {
/* 358 */     ObjectData objectData = null;
/*     */     try
/*     */     {
/* 361 */       objectData = getObjectData();
/*     */     } catch (ARMADataStoreException ex) {
/* 363 */       throw new ARMAException(LOG, null, "Problem occured while connecting to data store to fetch the object data:" + ex.getMessage(), ex);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 368 */     Map<Utf8, Map<Utf8, GenericRecord>> systemToGenericRecord = objectData.getSystemToGenericRecord();
/*     */     
/* 370 */     ARMAObjectInstanceIteratorImpl instanceIterator = new ARMAObjectInstanceIteratorImpl(this, systemToGenericRecord, objectData.getKeyIndex());
/*     */     
/* 372 */     return instanceIterator;
/*     */   }
/*     */   
/*     */ 
/*     */   public ARMAObjectInstance getInstance(String systemId, String instanceKey)
/*     */     throws ARMAException
/*     */   {
/* 379 */     if (isNotValidString(instanceKey)) {
/* 380 */       throw new ARMAIllegalArgumentException(LOG, null, "Validation failed: input instanceKey can't be null or empty");
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 385 */     Utf8 encodedInstanceKey = new Utf8(instanceKey);
/* 386 */     GenericRecord genericRecord = null;
/* 387 */     Map<Utf8, GenericRecord> instanceMap = null;
/* 388 */     Map<Utf8, Map<Utf8, GenericRecord>> systemToGenericRecord = getObjectData().getSystemToGenericRecord();
/*     */     
/* 390 */     if (isNotValidString(systemId))
/*     */     {
/* 392 */       for (Utf8 sysId : systemToGenericRecord.keySet()) {
/* 393 */         instanceMap = (Map)systemToGenericRecord.get(sysId);
/* 394 */         genericRecord = (GenericRecord)instanceMap.get(encodedInstanceKey);
/*     */         
/* 396 */         if (null != genericRecord) {
/* 397 */           ARMAObjectInstance instance = new ARMAObjectInstanceImpl(this, sysId, encodedInstanceKey, genericRecord);
/*     */           
/* 399 */           return instance;
/*     */         }
/*     */       }
/* 402 */       LOG.warn(null, "No instance found.For the instance Key :" + instanceKey);
/*     */       
/* 404 */       return null;
/*     */     }
/*     */     
/* 407 */     Utf8 encodedSysId = new Utf8(systemId);
/* 408 */     Map<Utf8, GenericRecord> instances = (Map)systemToGenericRecord.get(encodedSysId);
/* 409 */     if (instances != null) {
/* 410 */       genericRecord = (GenericRecord)instances.get(encodedInstanceKey);
/* 411 */       if (genericRecord != null) {
/* 412 */         ARMAObjectInstance instance = new ARMAObjectInstanceImpl(this, encodedSysId, encodedInstanceKey, genericRecord);
/*     */         
/* 414 */         return instance;
/*     */       }
/* 416 */       LOG.warn(null, "No instance found for input instanceKey : " + instanceKey);
/*     */       
/* 418 */       return null;
/*     */     }
/*     */     
/* 421 */     LOG.warn(null, "No instance found for input systemId : " + systemId);
/* 422 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Collection<ARMAViewOnlyObject> getViewOnlyParents()
/*     */   {
/* 433 */     return this.parser.getAllParents(this.session, this, this.nodeObject, true);
/*     */   }
/*     */   
/*     */ 
/*     */   public ARMAViewOnlyObject getViewOnlyParent(String objectName)
/*     */   {
/* 439 */     if (isInvalidObjectName(objectName))
/* 440 */       return null;
/* 441 */     return this.parser.getParentNodeByName(this.session, this, this.nodeObject, objectName, true);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public Collection<ARMAViewOnlyObject> getViewOnlyChilds()
/*     */   {
/* 448 */     return this.parser.getAllChilds(this.session, this, this.nodeObject, true);
/*     */   }
/*     */   
/*     */ 
/*     */   public ARMAViewOnlyObject getViewOnlyChild(String objectName)
/*     */   {
/* 454 */     if (isInvalidObjectName(objectName))
/* 455 */       return null;
/* 456 */     return this.parser.getChildNodeByName(this.session, this, this.nodeObject, objectName, true);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public Collection<ARMAViewOnlyObject> getViewOnlyAll()
/*     */   {
/* 463 */     Collection<ARMAViewOnlyObject> allViewOnlyARMANodes = new ArrayList();
/* 464 */     allViewOnlyARMANodes.addAll(getViewOnlyParents());
/* 465 */     allViewOnlyARMANodes.addAll(getViewOnlyChilds());
/*     */     
/* 467 */     return allViewOnlyARMANodes;
/*     */   }
/*     */   
/*     */ 
/*     */   public ARMAViewOnlyObject getViewOnlyAll(String objectName)
/*     */   {
/* 473 */     if (isInvalidObjectName(objectName)) {
/* 474 */       return null;
/*     */     }
/* 476 */     ARMAViewOnlyObject viewOnlyObject = getViewOnlyParent(objectName);
/* 477 */     if (null == viewOnlyObject) {
/* 478 */       viewOnlyObject = getViewOnlyChild(objectName);
/*     */     }
/* 480 */     return viewOnlyObject;
/*     */   }
/*     */   
/*     */   public boolean isOptionsAvailable()
/*     */   {
/* 485 */     return this.nodeObject.isOptionsSupported();
/*     */   }
/*     */   
/*     */   public boolean fetchOptions()
/*     */   {
/* 490 */     if (isOptionsAvailable()) {
/* 491 */       this.session.addOptions(getObjectName());
/* 492 */       return true;
/*     */     }
/* 494 */     return false;
/*     */   }
/*     */   
/*     */ 
/*     */   public void addObjectToRequest()
/*     */   {
/* 500 */     this.session.addObjectToRequest(this.objectName);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public <I, O> Map<Utf8, O> getFunctionData(String identifier, RegisteredFunction<I, O> registeredFunction)
/*     */     throws ARMAException
/*     */   {
/* 508 */     if (isNotValidString(identifier)) {
/* 509 */       throw new ARMAIllegalArgumentException(LOG, "Function identifier is null or empty:" + identifier);
/*     */     }
/*     */     
/* 512 */     if (null == registeredFunction) {
/* 513 */       throw new ARMAIllegalArgumentException(LOG, "Registered Function cannot be null :" + registeredFunction);
/*     */     }
/*     */     
/*     */ 
/* 517 */     Map<Utf8, O> functionResults = new HashMap();
/* 518 */     FunctionOutputHandler<O> outputHandler = registeredFunction.getOutputHandler();
/*     */     
/*     */     Map<Utf8, GenericRecord> instanceKeyVsFunctionData;
/*     */     try
/*     */     {
/* 523 */       instanceKeyVsFunctionData = this.session.getFunctionData(getObjectName(), identifier, registeredFunction).getFunctionOutput();
/*     */     }
/*     */     catch (IOException e)
/*     */     {
/* 527 */       throw new ARMAException(LOG, registeredFunction.toString(), "Error in getting function results data caused by :" + e.getMessage(), e);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 532 */     for (Utf8 key : instanceKeyVsFunctionData.keySet()) {
/* 533 */       if (null != outputHandler) {
/* 534 */         functionResults.put(key, outputHandler.handleOutput((GenericRecord)instanceKeyVsFunctionData.get(key)));
/*     */       } else {
/* 536 */         functionResults.put(key, instanceKeyVsFunctionData.get(key));
/*     */       }
/*     */     }
/*     */     
/* 540 */     return functionResults;
/*     */   }
/*     */   
/*     */ 
/*     */   public <I, O> void addFunction(String identifier, RegisteredFunction<I, O> registeredFunction, I input)
/*     */     throws ARMAException
/*     */   {
/* 547 */     if (isNotValidString(identifier)) {
/* 548 */       throw new ARMAIllegalArgumentException(LOG, "Function identifier is null or empty:" + identifier);
/*     */     }
/* 550 */     if (null == registeredFunction) {
/* 551 */       throw new ARMAIllegalArgumentException(LOG, "Registered Function cannot be null :" + registeredFunction);
/*     */     }
/*     */     
/* 554 */     this.session.getRequest().getFunctionRequest().addFunction(getObjectName(), identifier, registeredFunction, input);
/*     */   }
/*     */   
/*     */ 
/*     */   public void addGenericFunction(String identifier, RegisteredFunction<GenericRecord, GenericRecord> registeredFunction, GenericRecord input)
/*     */     throws ARMAException
/*     */   {
/* 561 */     if (isNotValidString(identifier)) {
/* 562 */       throw new ARMAIllegalArgumentException(LOG, "Function identifier is null or empty:" + identifier);
/*     */     }
/* 564 */     if (null == registeredFunction) {
/* 565 */       throw new ARMAIllegalArgumentException(LOG, "Registered Function cannot be null :" + registeredFunction);
/*     */     }
/*     */     
/* 568 */     this.session.getRequest().getFunctionRequest().addFunction(getObjectName(), identifier, registeredFunction, input);
/*     */   }
/*     */   
/*     */   public <I, O> Map<Utf8, O> executeFunction(String identifier, RegisteredFunction<I, O> registeredFunction, I input)
/*     */     throws ARMAException
/*     */   {
/* 574 */     if (isNotValidString(identifier)) {
/* 575 */       throw new ARMAIllegalArgumentException(LOG, "Function identifier is null or empty:" + identifier);
/*     */     }
/* 577 */     if (null == registeredFunction) {
/* 578 */       throw new ARMAIllegalArgumentException(LOG, "Registered Function cannot be null :" + registeredFunction);
/*     */     }
/*     */     
/* 581 */     this.session.getRequest().getFunctionRequest().addFunction(getObjectName(), identifier, registeredFunction, input);
/*     */     
/* 583 */     return getFunctionData(identifier, registeredFunction);
/*     */   }
/*     */   
/*     */   public <I, O> RegisteredFunction<I, O> getRegisteredFunction(String functionName)
/*     */     throws ARMAException
/*     */   {
/* 589 */     if (isNotValidString(functionName)) {
/* 590 */       throw new ARMAIllegalArgumentException(LOG, "Function name is null or empty:" + functionName);
/*     */     }
/*     */     
/* 593 */     RegisteredFunction<I, O> registeredFunction = functionRegistory.getRegisteredFunction(functionName);
/* 594 */     if (registeredFunction != null) {
/* 595 */       return registeredFunction;
/*     */     }
/* 597 */     throw new ARMAFunctionExecutionException(LOG, this.session.getRequestId(), "No function registered by name :" + functionName);
/*     */   }
/*     */   
/*     */ 
/*     */   public RegisteredFunction<GenericRecord, GenericRecord> getGenericIORegisteredFunction(String functionName)
/*     */     throws ARMAException
/*     */   {
/* 604 */     if (isNotValidString(functionName)) {
/* 605 */       throw new ARMAIllegalArgumentException(LOG, "Function name is null or empty:" + functionName);
/*     */     }
/*     */     
/* 608 */     RegisteredFunction<GenericRecord, GenericRecord> registeredFunction = functionRegistory.getRegisteredFunction(functionName);
/* 609 */     if (registeredFunction != null) {
/* 610 */       return registeredFunction;
/*     */     }
/* 612 */     throw new ARMAFunctionExecutionException(LOG, this.session.getRequestId(), "No function registered by name :" + functionName);
/*     */   }
/*     */   
/*     */ 
/*     */   public String getSessionId()
/*     */   {
/* 618 */     return getSession().getRequestId();
/*     */   }
/*     */   
/*     */   public boolean isNodeAvailable(String nodeName) {
/* 622 */     NodeObject nodeObject = this.nodeObject.getNode(nodeName);
/* 623 */     if (nodeObject != null) {
/* 624 */       return true;
/*     */     }
/* 626 */     return false;
/*     */   }
/*     */   
/*     */   public NodeObject getNodeObject() {
/* 630 */     return this.nodeObject;
/*     */   }
/*     */   
/*     */   public ObjectData getObjectData() throws ARMAException {
/* 634 */     return this.session.getObject(getObjectName().toUpperCase());
/*     */   }
/*     */   
/*     */   private boolean isNotValidString(String identifier) {
/* 638 */     return StringUtils.isEmpty(identifier);
/*     */   }
/*     */   
/*     */   private boolean isInvalidObjectName(String objectName) {
/* 642 */     if (isNotValidString(objectName)) {
/* 643 */       LOG.warn(getRequestId(), "Invalid object name is provided:" + objectName);
/*     */       
/* 645 */       return true;
/*     */     }
/* 647 */     return false;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void retrieveCounterData(int intervalInDays, String... counterObjectNames)
/*     */   {
/* 661 */     if ((intervalInDays > 0) && (counterObjectNames != null)) {
/* 662 */       this.session.addPerfCounters(getObjectName(), intervalInDays, counterObjectNames);
/*     */     } else {
/* 664 */       LOG.warn(getRequestId(), "Invalid interval / perfCounter combination specified, interval: " + intervalInDays + ", counterObjectNames: " + counterObjectNames);
/*     */     }
/*     */   }
/*     */   
/*     */   public void retrieveEventData(int intervalInDays, String... eventNames) {
/* 669 */     retrieveEventData(intervalInDays, false, eventNames);
/*     */   }
/*     */   
/*     */   public void retrieveEventData(int intervalInDays, boolean addBizKeyToTags, String... eventNames)
/*     */   {
/* 674 */     if ((intervalInDays > 0) && (eventNames != null)) {
/* 675 */       this.session.addLogEvents(getObjectName(), intervalInDays, addBizKeyToTags, eventNames);
/*     */     } else {
/* 677 */       LOG.warn(getRequestId(), "Invalid interval / eventName combination specified, interval: " + intervalInDays + ", evnetNames: " + eventNames);
/*     */     }
/*     */   }
/*     */   
/*     */   public String getObjectXml() throws ARMAException
/*     */   {
/*     */     try {
/* 684 */       return ObjectDataXmlHelper.serializeObject(this);
/*     */     } catch (Exception e) {
/* 686 */       throw new ARMAException(LOG, "-9999", "unsupported encoding exception occured " + e.getMessage());
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public Collection<String> getChildNames()
/*     */   {
/* 693 */     return this.nodeObject.getChilds();
/*     */   }
/*     */   
/*     */ 
/*     */   public Collection<String> getParentNames()
/*     */   {
/* 699 */     return this.nodeObject.getParents();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public void retrieveEventData(String viewAsof, int intervalInDays, String... eventNames)
/*     */   {
/* 707 */     Identifier identifier = getSession().getIdentifier();
/* 708 */     identifier.setTimeStamp(viewAsof);
/*     */     
/* 710 */     retrieveEventData(intervalInDays, false, eventNames);
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asupdata\client\ARMA\ARMAObjectImpl.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */