/*     */ package net.rubyeye.xmemcached.auth;
/*     */ 
/*     */ import java.net.InetSocketAddress;
/*     */ import java.util.concurrent.CountDownLatch;
/*     */ import java.util.concurrent.atomic.AtomicBoolean;
/*     */ import javax.security.sasl.Sasl;
/*     */ import javax.security.sasl.SaslClient;
/*     */ import javax.security.sasl.SaslException;
/*     */ import net.rubyeye.xmemcached.CommandFactory;
/*     */ import net.rubyeye.xmemcached.command.Command;
/*     */ import net.rubyeye.xmemcached.command.binary.BaseBinaryCommand;
/*     */ import net.rubyeye.xmemcached.command.binary.ResponseStatus;
/*     */ import net.rubyeye.xmemcached.impl.MemcachedTCPSession;
/*     */ import net.rubyeye.xmemcached.utils.ByteUtils;
/*     */ import org.slf4j.Logger;
/*     */ import org.slf4j.LoggerFactory;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class AuthTask
/*     */   extends Thread
/*     */ {
/*     */   private final AuthInfo authInfo;
/*     */   private final CommandFactory commandFactory;
/*     */   private MemcachedTCPSession memcachedTCPSession;
/*  33 */   public static final byte[] EMPTY_BYTES = new byte[0];
/*  34 */   static final Logger log = LoggerFactory.getLogger(AuthTask.class);
/*     */   
/*     */ 
/*     */   public AuthTask(AuthInfo authInfo, CommandFactory commandFactory, MemcachedTCPSession memcachedTCPSession)
/*     */   {
/*  39 */     this.authInfo = authInfo;
/*  40 */     this.commandFactory = commandFactory;
/*  41 */     this.memcachedTCPSession = memcachedTCPSession;
/*     */   }
/*     */   
/*     */   public void run() {
/*  45 */     if (this.authInfo.isValid()) {
/*  46 */       doAuth();
/*  47 */       this.authInfo.increaseAttempts();
/*     */     }
/*     */   }
/*     */   
/*     */   private void doAuth()
/*     */   {
/*  53 */     SaslClient saslClient = null;
/*     */     try {
/*  55 */       saslClient = Sasl.createSaslClient(this.authInfo.getMechanisms(), null, "memcached", this.memcachedTCPSession.getRemoteSocketAddress().toString(), null, this.authInfo.getCallbackHandler());
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*  60 */       AtomicBoolean done = new AtomicBoolean(false);
/*  61 */       byte[] response = saslClient.hasInitialResponse() ? saslClient.evaluateChallenge(EMPTY_BYTES) : EMPTY_BYTES;
/*     */       
/*  63 */       CountDownLatch latch = new CountDownLatch(1);
/*  64 */       Command command = this.commandFactory.createAuthStartCommand(saslClient.getMechanismName(), latch, response);
/*     */       
/*  66 */       if (!this.memcachedTCPSession.isClosed()) {
/*  67 */         this.memcachedTCPSession.write(command);
/*     */       } else {
/*  69 */         log.error("Authentication fail,because the connection has been closed");
/*     */         
/*  71 */         throw new RuntimeException("Authentication fai,connection has been close");
/*     */       }
/*     */       
/*     */ 
/*  75 */       while (!done.get()) {
/*     */         try {
/*  77 */           latch.await();
/*     */         } catch (InterruptedException e) {
/*  79 */           Thread.currentThread().interrupt();
/*  80 */           done.set(true);
/*     */         }
/*  82 */         ResponseStatus responseStatus = ((BaseBinaryCommand)command).getResponseStatus();
/*     */         
/*  84 */         switch (responseStatus) {
/*     */         case NO_ERROR: 
/*  86 */           done.set(true);
/*  87 */           log.info("Authentication to " + this.memcachedTCPSession.getRemoteSocketAddress() + " successfully");
/*     */           
/*     */ 
/*  90 */           break;
/*     */         case AUTH_REQUIRED: 
/*  92 */           log.error("Authentication failed to " + this.memcachedTCPSession.getRemoteSocketAddress());
/*     */           
/*     */ 
/*     */ 
/*  96 */           log.warn("Reopen connection to " + this.memcachedTCPSession.getRemoteSocketAddress() + ",beacause auth fail");
/*     */           
/*     */ 
/*  99 */           this.memcachedTCPSession.setAuthFailed(true);
/*     */           
/*     */ 
/* 102 */           if (!this.authInfo.isFirstTime()) {
/* 103 */             Thread.sleep(1000L);
/*     */           }
/* 105 */           this.memcachedTCPSession.close();
/* 106 */           done.set(true);
/* 107 */           break;
/*     */         case FUTHER_AUTH_REQUIRED: 
/* 109 */           String result = (String)command.getResult();
/* 110 */           response = saslClient.evaluateChallenge(ByteUtils.getBytes(result));
/*     */           
/* 112 */           latch = new CountDownLatch(1);
/* 113 */           command = this.commandFactory.createAuthStepCommand(saslClient.getMechanismName(), latch, response);
/*     */           
/* 115 */           if (!this.memcachedTCPSession.isClosed()) {
/* 116 */             this.memcachedTCPSession.write(command);
/*     */           } else {
/* 118 */             log.error("Authentication fail,because the connection has been closed");
/*     */             
/* 120 */             throw new RuntimeException("Authentication fai,connection has been close");
/*     */           }
/*     */           
/*     */ 
/*     */           break;
/*     */         default: 
/* 126 */           done.set(true);
/* 127 */           log.error("Authentication failed to " + this.memcachedTCPSession.getRemoteSocketAddress() + ",response status=" + responseStatus);
/*     */         }
/*     */         
/*     */       }
/*     */       
/*     */       return;
/*     */     }
/*     */     catch (Exception e)
/*     */     {
/* 136 */       log.error("Create saslClient error", e);
/*     */     } finally {
/* 138 */       if (saslClient != null) {
/*     */         try {
/* 140 */           saslClient.dispose();
/*     */         } catch (SaslException e) {
/* 142 */           log.error("Dispose saslClient error", e);
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\net\rubyeye\xmemcached\auth\AuthTask.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */