/*     */ package net.rubyeye.xmemcached.command.text;
/*     */ 
/*     */ import com.google.code.yanf4j.buffer.IoBuffer;
/*     */ import java.io.UnsupportedEncodingException;
/*     */ import java.nio.ByteBuffer;
/*     */ import java.util.HashMap;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.concurrent.CountDownLatch;
/*     */ import net.rubyeye.xmemcached.command.AssocCommandAware;
/*     */ import net.rubyeye.xmemcached.command.Command;
/*     */ import net.rubyeye.xmemcached.command.CommandType;
/*     */ import net.rubyeye.xmemcached.command.MapReturnValueAware;
/*     */ import net.rubyeye.xmemcached.command.MergeCommandsAware;
/*     */ import net.rubyeye.xmemcached.impl.MemcachedTCPSession;
/*     */ import net.rubyeye.xmemcached.monitor.Constants;
/*     */ import net.rubyeye.xmemcached.transcoders.CachedData;
/*     */ import net.rubyeye.xmemcached.utils.ByteUtils;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class TextGetCommand
/*     */   extends Command
/*     */   implements MergeCommandsAware, AssocCommandAware, MapReturnValueAware
/*     */ {
/*     */   protected Map<String, CachedData> returnValues;
/*     */   private String currentReturnKey;
/*     */   private int offset;
/*     */   private List<Command> assocCommands;
/*     */   private Map<Object, Command> mergeCommands;
/*     */   
/*     */   public final Map<Object, Command> getMergeCommands()
/*     */   {
/*  65 */     return this.mergeCommands;
/*     */   }
/*     */   
/*     */   public final void setMergeCommands(Map<Object, Command> mergeCommands) {
/*  69 */     this.mergeCommands = mergeCommands;
/*     */   }
/*     */   
/*     */   public final List<Command> getAssocCommands() {
/*  73 */     return this.assocCommands;
/*     */   }
/*     */   
/*     */   public final void setAssocCommands(List<Command> assocCommands) {
/*  77 */     this.assocCommands = assocCommands;
/*     */   }
/*     */   
/*     */   public TextGetCommand(String key, byte[] keyBytes, CommandType cmdType, CountDownLatch latch)
/*     */   {
/*  82 */     super(key, keyBytes, cmdType, latch);
/*  83 */     this.returnValues = new HashMap(32);
/*     */   }
/*     */   
/*  86 */   private ParseStatus parseStatus = ParseStatus.NULL;
/*     */   
/*     */   public static enum ParseStatus {
/*  89 */     NULL,  VALUE,  KEY,  FLAG,  DATA_LEN,  DATA_LEN_DONE,  CAS,  CAS_DONE,  DATA,  END;
/*     */     
/*     */     private ParseStatus() {} }
/*  92 */   protected boolean wasFirst = true;
/*     */   
/*     */   public ParseStatus getParseStatus() {
/*  95 */     return this.parseStatus;
/*     */   }
/*     */   
/*     */   public void setParseStatus(ParseStatus parseStatus) {
/*  99 */     this.parseStatus = parseStatus;
/*     */   }
/*     */   
/*     */   public final boolean decode(MemcachedTCPSession session, ByteBuffer buffer)
/*     */   {
/*     */     for (;;) {
/* 105 */       if ((buffer == null) || (!buffer.hasRemaining()))
/* 106 */         return false;
/*     */       byte first;
/* 108 */       String item; switch (this.parseStatus)
/*     */       {
/*     */       case NULL: 
/* 111 */         if (buffer.remaining() < 2)
/* 112 */           return false;
/* 113 */         first = buffer.get(buffer.position());
/* 114 */         byte second = buffer.get(buffer.position() + 1);
/* 115 */         if ((first == 69) && (second == 78)) {
/* 116 */           this.parseStatus = ParseStatus.END;
/*     */           
/* 118 */           dispatch();
/* 119 */           this.currentReturnKey = null;
/*     */         }
/* 121 */         else if (first == 86) {
/* 122 */           this.parseStatus = ParseStatus.VALUE;
/* 123 */           this.wasFirst = false;
/*     */         }
/*     */         else {
/* 126 */           return decodeError(session, buffer);
/*     */         }
/*     */         break;
/*     */       case END: 
/* 130 */         return ByteUtils.stepBuffer(buffer, 5);
/*     */       
/*     */       case VALUE: 
/* 133 */         if (ByteUtils.stepBuffer(buffer, 6)) {
/* 134 */           this.parseStatus = ParseStatus.KEY;
/*     */         }
/*     */         else
/* 137 */           return false;
/*     */         break;
/*     */       case KEY: 
/* 140 */         item = getItem(buffer, ' ', new char[0]);
/* 141 */         if (item == null) {
/* 142 */           return false;
/*     */         }
/* 144 */         this.currentReturnKey = item;
/* 145 */         this.returnValues.put(this.currentReturnKey, new CachedData());
/*     */         
/* 147 */         this.parseStatus = ParseStatus.FLAG;
/* 148 */         break;
/*     */       
/*     */       case FLAG: 
/* 151 */         item = getItem(buffer, ' ', new char[0]);
/* 152 */         if (item == null) {
/* 153 */           return false;
/*     */         }
/* 155 */         CachedData cachedData = (CachedData)this.returnValues.get(this.currentReturnKey);
/*     */         
/* 157 */         cachedData.setFlag(Integer.parseInt(item));
/* 158 */         this.parseStatus = ParseStatus.DATA_LEN;
/* 159 */         break;
/*     */       
/*     */       case DATA_LEN: 
/* 162 */         item = getItem(buffer, '\r', new char[] { ' ' });
/* 163 */         if (item == null) {
/* 164 */           return false;
/*     */         }
/* 166 */         CachedData cachedData = (CachedData)this.returnValues.get(this.currentReturnKey);
/*     */         
/* 168 */         cachedData.setCapacity(Integer.parseInt(item));
/* 169 */         assert (cachedData.getCapacity() >= 0);
/* 170 */         cachedData.setData(new byte[cachedData.getCapacity()]);
/* 171 */         this.parseStatus = ParseStatus.DATA_LEN_DONE;
/* 172 */         break;
/*     */       
/*     */       case DATA_LEN_DONE: 
/* 175 */         if (buffer.remaining() < 1) {
/* 176 */           return false;
/*     */         }
/* 178 */         first = buffer.get(buffer.position());
/*     */         
/* 180 */         if (first == 10)
/*     */         {
/* 182 */           buffer.position(buffer.position() + 1);
/* 183 */           this.parseStatus = ParseStatus.DATA;
/*     */         }
/*     */         else {
/* 186 */           this.parseStatus = ParseStatus.CAS; }
/* 187 */         break;
/*     */       
/*     */ 
/*     */ 
/*     */       case CAS: 
/* 192 */         item = getItem(buffer, '\r', new char[0]);
/* 193 */         if (item == null) {
/* 194 */           return false;
/*     */         }
/* 196 */         CachedData cachedData = (CachedData)this.returnValues.get(this.currentReturnKey);
/*     */         
/* 198 */         cachedData.setCas(Long.parseLong(item));
/* 199 */         this.parseStatus = ParseStatus.CAS_DONE;
/* 200 */         break;
/*     */       
/*     */       case CAS_DONE: 
/* 203 */         if (buffer.remaining() < 1) {
/* 204 */           return false;
/*     */         }
/* 206 */         this.parseStatus = ParseStatus.DATA;
/*     */         
/* 208 */         buffer.position(buffer.position() + 1);
/* 209 */         break;
/*     */       
/*     */       case DATA: 
/* 212 */         CachedData value = (CachedData)this.returnValues.get(this.currentReturnKey);
/*     */         
/* 214 */         int remaining = buffer.remaining();
/* 215 */         int remainingCapacity = value.remainingCapacity();
/* 216 */         assert (remainingCapacity >= 0);
/*     */         
/* 218 */         if (remaining < remainingCapacity + 2) {
/* 219 */           int length = remaining > remainingCapacity ? remainingCapacity : remaining;
/*     */           
/* 221 */           value.fillData(buffer, length);
/* 222 */           return false; }
/* 223 */         if (remainingCapacity > 0) {
/* 224 */           value.fillData(buffer, remainingCapacity);
/*     */         }
/* 226 */         assert (value.remainingCapacity() == 0);
/* 227 */         buffer.position(buffer.position() + ByteUtils.SPLIT.remaining());
/*     */         
/*     */ 
/*     */ 
/* 231 */         Map<Object, Command> mergetCommands = getMergeCommands();
/* 232 */         if (mergetCommands != null) {
/* 233 */           TextGetCommand command = (TextGetCommand)mergetCommands.remove(this.currentReturnKey);
/*     */           
/* 235 */           if (command != null) {
/* 236 */             command.setResult(value);
/* 237 */             command.countDownLatch();
/* 238 */             this.mergeCount -= 1;
/* 239 */             if (command.getAssocCommands() != null) {
/* 240 */               for (Command assocCommand : command.getAssocCommands())
/*     */               {
/* 242 */                 assocCommand.setResult(value);
/* 243 */                 assocCommand.countDownLatch();
/* 244 */                 this.mergeCount -= 1;
/*     */               }
/*     */             }
/*     */           }
/*     */         }
/*     */         
/* 250 */         this.currentReturnKey = null;
/* 251 */         this.parseStatus = ParseStatus.NULL;
/*     */       }
/*     */     }
/* 254 */     return decodeError(session, buffer);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   private String getItem(ByteBuffer buffer, char token, char... others)
/*     */   {
/* 261 */     int pos = buffer.position() + this.offset;
/* 262 */     int limit = buffer.limit();
/* 263 */     for (; pos < limit; pos++) {
/* 264 */       byte b = buffer.get(pos);
/* 265 */       if ((b == token) || (isIn(b, others))) {
/* 266 */         byte[] keyBytes = new byte[pos - buffer.position()];
/* 267 */         buffer.get(keyBytes);
/* 268 */         this.offset = 0;
/* 269 */         assert (pos == buffer.position());
/*     */         
/* 271 */         buffer.position(pos + 1);
/* 272 */         return getString(keyBytes);
/*     */       }
/*     */     }
/* 275 */     this.offset = (pos - buffer.position());
/* 276 */     return null;
/*     */   }
/*     */   
/*     */   private boolean isIn(byte b, char[] others) {
/* 280 */     for (int i = 0; i < others.length; i++) {
/* 281 */       if (b == others[i]) {
/* 282 */         return true;
/*     */       }
/*     */     }
/* 285 */     return false;
/*     */   }
/*     */   
/*     */   private String getString(byte[] keyBytes) {
/*     */     try {
/* 290 */       return new String(keyBytes, "utf-8");
/*     */     } catch (UnsupportedEncodingException e) {
/* 292 */       throw new RuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final Map<String, CachedData> getReturnValues()
/*     */   {
/* 303 */     return this.returnValues;
/*     */   }
/*     */   
/*     */   public final void setReturnValues(Map<String, CachedData> returnValues) {
/* 307 */     this.returnValues = returnValues;
/*     */   }
/*     */   
/*     */   public abstract void dispatch();
/*     */   
/*     */   public void encode()
/*     */   {
/* 314 */     byte[] cmdBytes = (this.commandType == CommandType.GET_ONE) || (this.commandType == CommandType.GET_MANY) ? Constants.GET : Constants.GETS;
/*     */     
/*     */ 
/* 317 */     this.ioBuffer = IoBuffer.allocate(cmdBytes.length + Constants.CRLF.length + 1 + this.keyBytes.length);
/*     */     
/* 319 */     ByteUtils.setArguments(this.ioBuffer, new Object[] { cmdBytes, this.keyBytes });
/* 320 */     this.ioBuffer.flip();
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\net\rubyeye\xmemcached\command\text\TextGetCommand.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */