/*     */ package org.apache.avro.file;
/*     */ 
/*     */ import java.io.ByteArrayOutputStream;
/*     */ import java.io.IOException;
/*     */ import java.io.OutputStream;
/*     */ import java.nio.ByteBuffer;
/*     */ import java.util.zip.Deflater;
/*     */ import java.util.zip.DeflaterOutputStream;
/*     */ import java.util.zip.Inflater;
/*     */ import java.util.zip.InflaterOutputStream;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ class DeflateCodec
/*     */   extends Codec
/*     */ {
/*     */   private ByteArrayOutputStream outputBuffer;
/*     */   private Deflater deflater;
/*     */   private Inflater inflater;
/*     */   
/*     */   static class Option
/*     */     extends CodecFactory
/*     */   {
/*     */     private int compressionLevel;
/*     */     
/*     */     Option(int compressionLevel)
/*     */     {
/*  45 */       this.compressionLevel = compressionLevel;
/*     */     }
/*     */     
/*     */     protected Codec createInstance()
/*     */     {
/*  50 */       return new DeflateCodec(this.compressionLevel);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  58 */   private boolean nowrap = true;
/*     */   private int compressionLevel;
/*     */   
/*     */   public DeflateCodec(int compressionLevel) {
/*  62 */     this.compressionLevel = compressionLevel;
/*     */   }
/*     */   
/*     */   String getName()
/*     */   {
/*  67 */     return "deflate";
/*     */   }
/*     */   
/*     */   ByteBuffer compress(ByteBuffer data) throws IOException
/*     */   {
/*  72 */     ByteArrayOutputStream baos = getOutputBuffer(data.remaining());
/*  73 */     DeflaterOutputStream ios = new DeflaterOutputStream(baos, getDeflater());
/*  74 */     writeAndClose(data, ios);
/*  75 */     ByteBuffer result = ByteBuffer.wrap(baos.toByteArray());
/*  76 */     return result;
/*     */   }
/*     */   
/*     */   ByteBuffer decompress(ByteBuffer data) throws IOException
/*     */   {
/*  81 */     ByteArrayOutputStream baos = getOutputBuffer(data.remaining());
/*  82 */     InflaterOutputStream ios = new InflaterOutputStream(baos, getInflater());
/*  83 */     writeAndClose(data, ios);
/*  84 */     ByteBuffer result = ByteBuffer.wrap(baos.toByteArray());
/*  85 */     return result;
/*     */   }
/*     */   
/*     */   private void writeAndClose(ByteBuffer data, OutputStream to) throws IOException {
/*  89 */     byte[] input = data.array();
/*  90 */     int offset = data.arrayOffset() + data.position();
/*  91 */     int length = data.remaining();
/*     */     try {
/*  93 */       to.write(input, offset, length);
/*     */     } finally {
/*  95 */       to.close();
/*     */     }
/*     */   }
/*     */   
/*     */   private Inflater getInflater()
/*     */   {
/* 101 */     if (null == this.inflater) {
/* 102 */       this.inflater = new Inflater(this.nowrap);
/*     */     }
/* 104 */     this.inflater.reset();
/* 105 */     return this.inflater;
/*     */   }
/*     */   
/*     */   private Deflater getDeflater()
/*     */   {
/* 110 */     if (null == this.deflater) {
/* 111 */       this.deflater = new Deflater(this.compressionLevel, this.nowrap);
/*     */     }
/* 113 */     this.deflater.reset();
/* 114 */     return this.deflater;
/*     */   }
/*     */   
/*     */   private ByteArrayOutputStream getOutputBuffer(int suggestedLength)
/*     */   {
/* 119 */     if (null == this.outputBuffer) {
/* 120 */       this.outputBuffer = new ByteArrayOutputStream(suggestedLength);
/*     */     }
/* 122 */     this.outputBuffer.reset();
/* 123 */     return this.outputBuffer;
/*     */   }
/*     */   
/*     */   public int hashCode()
/*     */   {
/* 128 */     return this.nowrap ? 0 : 1;
/*     */   }
/*     */   
/*     */   public boolean equals(Object obj)
/*     */   {
/* 133 */     if (this == obj)
/* 134 */       return true;
/* 135 */     if (getClass() != obj.getClass())
/* 136 */       return false;
/* 137 */     DeflateCodec other = (DeflateCodec)obj;
/* 138 */     return this.nowrap == other.nowrap;
/*     */   }
/*     */   
/*     */   public String toString()
/*     */   {
/* 143 */     return getName() + "-" + this.compressionLevel;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\file\DeflateCodec.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */