/*     */ package org.apache.avro.io;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.io.OutputStream;
/*     */ import java.nio.ByteBuffer;
/*     */ import java.util.BitSet;
/*     */ import org.apache.avro.AvroTypeException;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.io.parsing.JsonGrammarGenerator;
/*     */ import org.apache.avro.io.parsing.Parser;
/*     */ import org.apache.avro.io.parsing.Parser.ActionHandler;
/*     */ import org.apache.avro.io.parsing.Symbol;
/*     */ import org.apache.avro.io.parsing.Symbol.Alternative;
/*     */ import org.apache.avro.io.parsing.Symbol.EnumLabelsAction;
/*     */ import org.apache.avro.io.parsing.Symbol.FieldAdjustAction;
/*     */ import org.apache.avro.io.parsing.Symbol.IntCheckAction;
/*     */ import org.apache.avro.util.Utf8;
/*     */ import org.codehaus.jackson.JsonEncoding;
/*     */ import org.codehaus.jackson.JsonFactory;
/*     */ import org.codehaus.jackson.JsonGenerator;
/*     */ import org.codehaus.jackson.util.MinimalPrettyPrinter;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class JsonEncoder
/*     */   extends ParsingEncoder
/*     */   implements Parser.ActionHandler
/*     */ {
/*     */   final Parser parser;
/*     */   private JsonGenerator out;
/*  51 */   protected BitSet isEmpty = new BitSet();
/*     */   
/*     */   JsonEncoder(Schema sc, OutputStream out) throws IOException {
/*  54 */     this(sc, getJsonGenerator(out));
/*     */   }
/*     */   
/*     */   JsonEncoder(Schema sc, JsonGenerator out) throws IOException {
/*  58 */     configure(out);
/*  59 */     this.parser = new Parser(new JsonGrammarGenerator().generate(sc), this);
/*     */   }
/*     */   
/*     */   public void flush()
/*     */     throws IOException
/*     */   {
/*  65 */     this.parser.processImplicitActions();
/*  66 */     if (this.out != null) {
/*  67 */       this.out.flush();
/*     */     }
/*     */   }
/*     */   
/*     */   private static JsonGenerator getJsonGenerator(OutputStream out)
/*     */     throws IOException
/*     */   {
/*  74 */     if (null == out)
/*  75 */       throw new NullPointerException("OutputStream cannot be null");
/*  76 */     JsonGenerator g = new JsonFactory().createJsonGenerator(out, JsonEncoding.UTF8);
/*     */     
/*  78 */     MinimalPrettyPrinter pp = new MinimalPrettyPrinter();
/*  79 */     pp.setRootValueSeparator(System.getProperty("line.separator"));
/*  80 */     g.setPrettyPrinter(pp);
/*  81 */     return g;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonEncoder configure(OutputStream out)
/*     */     throws IOException
/*     */   {
/*  99 */     configure(getJsonGenerator(out));
/* 100 */     return this;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonEncoder configure(JsonGenerator generator)
/*     */     throws IOException
/*     */   {
/* 117 */     if (null == generator)
/* 118 */       throw new NullPointerException("JsonGenerator cannot be null");
/* 119 */     if (null != this.parser) {
/* 120 */       flush();
/*     */     }
/* 122 */     this.out = generator;
/* 123 */     return this;
/*     */   }
/*     */   
/*     */   public void writeNull() throws IOException
/*     */   {
/* 128 */     this.parser.advance(Symbol.NULL);
/* 129 */     this.out.writeNull();
/*     */   }
/*     */   
/*     */   public void writeBoolean(boolean b) throws IOException
/*     */   {
/* 134 */     this.parser.advance(Symbol.BOOLEAN);
/* 135 */     this.out.writeBoolean(b);
/*     */   }
/*     */   
/*     */   public void writeInt(int n) throws IOException
/*     */   {
/* 140 */     this.parser.advance(Symbol.INT);
/* 141 */     this.out.writeNumber(n);
/*     */   }
/*     */   
/*     */   public void writeLong(long n) throws IOException
/*     */   {
/* 146 */     this.parser.advance(Symbol.LONG);
/* 147 */     this.out.writeNumber(n);
/*     */   }
/*     */   
/*     */   public void writeFloat(float f) throws IOException
/*     */   {
/* 152 */     this.parser.advance(Symbol.FLOAT);
/* 153 */     this.out.writeNumber(f);
/*     */   }
/*     */   
/*     */   public void writeDouble(double d) throws IOException
/*     */   {
/* 158 */     this.parser.advance(Symbol.DOUBLE);
/* 159 */     this.out.writeNumber(d);
/*     */   }
/*     */   
/*     */   public void writeString(Utf8 utf8) throws IOException
/*     */   {
/* 164 */     writeString(utf8.toString());
/*     */   }
/*     */   
/*     */   public void writeString(String str) throws IOException
/*     */   {
/* 169 */     this.parser.advance(Symbol.STRING);
/* 170 */     if (this.parser.topSymbol() == Symbol.MAP_KEY_MARKER) {
/* 171 */       this.parser.advance(Symbol.MAP_KEY_MARKER);
/* 172 */       this.out.writeFieldName(str);
/*     */     } else {
/* 174 */       this.out.writeString(str);
/*     */     }
/*     */   }
/*     */   
/*     */   public void writeBytes(ByteBuffer bytes) throws IOException
/*     */   {
/* 180 */     if (bytes.hasArray()) {
/* 181 */       writeBytes(bytes.array(), bytes.position(), bytes.remaining());
/*     */     } else {
/* 183 */       byte[] b = new byte[bytes.remaining()];
/* 184 */       for (int i = 0; i < b.length; i++) {
/* 185 */         b[i] = bytes.get();
/*     */       }
/* 187 */       writeBytes(b);
/*     */     }
/*     */   }
/*     */   
/*     */   public void writeBytes(byte[] bytes, int start, int len) throws IOException
/*     */   {
/* 193 */     this.parser.advance(Symbol.BYTES);
/* 194 */     writeByteArray(bytes, start, len);
/*     */   }
/*     */   
/*     */   private void writeByteArray(byte[] bytes, int start, int len) throws IOException
/*     */   {
/* 199 */     this.out.writeString(new String(bytes, start, len, "ISO-8859-1"));
/*     */   }
/*     */   
/*     */   public void writeFixed(byte[] bytes, int start, int len)
/*     */     throws IOException
/*     */   {
/* 205 */     this.parser.advance(Symbol.FIXED);
/* 206 */     Symbol.IntCheckAction top = (Symbol.IntCheckAction)this.parser.popSymbol();
/* 207 */     if (len != top.size) {
/* 208 */       throw new AvroTypeException("Incorrect length for fixed binary: expected " + top.size + " but received " + len + " bytes.");
/*     */     }
/*     */     
/*     */ 
/* 212 */     writeByteArray(bytes, start, len);
/*     */   }
/*     */   
/*     */   public void writeEnum(int e) throws IOException
/*     */   {
/* 217 */     this.parser.advance(Symbol.ENUM);
/* 218 */     Symbol.EnumLabelsAction top = (Symbol.EnumLabelsAction)this.parser.popSymbol();
/* 219 */     if ((e < 0) || (e >= top.size)) {
/* 220 */       throw new AvroTypeException("Enumeration out of range: max is " + top.size + " but received " + e);
/*     */     }
/*     */     
/*     */ 
/* 224 */     this.out.writeString(top.getLabel(e));
/*     */   }
/*     */   
/*     */   public void writeArrayStart() throws IOException
/*     */   {
/* 229 */     this.parser.advance(Symbol.ARRAY_START);
/* 230 */     this.out.writeStartArray();
/* 231 */     push();
/* 232 */     this.isEmpty.set(depth());
/*     */   }
/*     */   
/*     */   public void writeArrayEnd() throws IOException
/*     */   {
/* 237 */     if (!this.isEmpty.get(this.pos)) {
/* 238 */       this.parser.advance(Symbol.ITEM_END);
/*     */     }
/* 240 */     pop();
/* 241 */     this.parser.advance(Symbol.ARRAY_END);
/* 242 */     this.out.writeEndArray();
/*     */   }
/*     */   
/*     */   public void writeMapStart() throws IOException
/*     */   {
/* 247 */     push();
/* 248 */     this.isEmpty.set(depth());
/*     */     
/* 250 */     this.parser.advance(Symbol.MAP_START);
/* 251 */     this.out.writeStartObject();
/*     */   }
/*     */   
/*     */   public void writeMapEnd() throws IOException
/*     */   {
/* 256 */     if (!this.isEmpty.get(this.pos)) {
/* 257 */       this.parser.advance(Symbol.ITEM_END);
/*     */     }
/* 259 */     pop();
/*     */     
/* 261 */     this.parser.advance(Symbol.MAP_END);
/* 262 */     this.out.writeEndObject();
/*     */   }
/*     */   
/*     */   public void startItem() throws IOException
/*     */   {
/* 267 */     if (!this.isEmpty.get(this.pos)) {
/* 268 */       this.parser.advance(Symbol.ITEM_END);
/*     */     }
/* 270 */     super.startItem();
/* 271 */     this.isEmpty.clear(depth());
/*     */   }
/*     */   
/*     */   public void writeIndex(int unionIndex) throws IOException
/*     */   {
/* 276 */     this.parser.advance(Symbol.UNION);
/* 277 */     Symbol.Alternative top = (Symbol.Alternative)this.parser.popSymbol();
/* 278 */     Symbol symbol = top.getSymbol(unionIndex);
/* 279 */     if (symbol != Symbol.NULL) {
/* 280 */       this.out.writeStartObject();
/* 281 */       this.out.writeFieldName(top.getLabel(unionIndex));
/* 282 */       this.parser.pushSymbol(Symbol.UNION_END);
/*     */     }
/* 284 */     this.parser.pushSymbol(symbol);
/*     */   }
/*     */   
/*     */   public Symbol doAction(Symbol input, Symbol top) throws IOException
/*     */   {
/* 289 */     if ((top instanceof Symbol.FieldAdjustAction)) {
/* 290 */       Symbol.FieldAdjustAction fa = (Symbol.FieldAdjustAction)top;
/* 291 */       this.out.writeFieldName(fa.fname);
/* 292 */     } else if (top == Symbol.RECORD_START) {
/* 293 */       this.out.writeStartObject();
/* 294 */     } else if ((top == Symbol.RECORD_END) || (top == Symbol.UNION_END)) {
/* 295 */       this.out.writeEndObject();
/* 296 */     } else if (top != Symbol.FIELD_END) {
/* 297 */       throw new AvroTypeException("Unknown action symbol " + top);
/*     */     }
/* 299 */     return null;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\io\JsonEncoder.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */