/*     */ package org.apache.avro.reflect;
/*     */ 
/*     */ import com.thoughtworks.paranamer.CachingParanamer;
/*     */ import com.thoughtworks.paranamer.Paranamer;
/*     */ import java.lang.annotation.Annotation;
/*     */ import java.lang.reflect.Array;
/*     */ import java.lang.reflect.Field;
/*     */ import java.lang.reflect.GenericArrayType;
/*     */ import java.lang.reflect.Method;
/*     */ import java.lang.reflect.ParameterizedType;
/*     */ import java.lang.reflect.Type;
/*     */ import java.nio.ByteBuffer;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Arrays;
/*     */ import java.util.Collection;
/*     */ import java.util.Collections;
/*     */ import java.util.LinkedHashMap;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.concurrent.ConcurrentHashMap;
/*     */ import org.apache.avro.AvroRemoteException;
/*     */ import org.apache.avro.AvroRuntimeException;
/*     */ import org.apache.avro.AvroTypeException;
/*     */ import org.apache.avro.Protocol;
/*     */ import org.apache.avro.Protocol.Message;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.Schema.Field;
/*     */ import org.apache.avro.Schema.Type;
/*     */ import org.apache.avro.generic.GenericContainer;
/*     */ import org.apache.avro.generic.GenericFixed;
/*     */ import org.apache.avro.generic.IndexedRecord;
/*     */ import org.apache.avro.io.BinaryData;
/*     */ import org.apache.avro.io.DatumReader;
/*     */ import org.apache.avro.specific.FixedSize;
/*     */ import org.apache.avro.specific.SpecificData;
/*     */ import org.codehaus.jackson.JsonNode;
/*     */ import org.codehaus.jackson.node.NullNode;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class ReflectData
/*     */   extends SpecificData
/*     */ {
/*     */   public static class AllowNull
/*     */     extends ReflectData
/*     */   {
/*  64 */     private static final AllowNull INSTANCE = new AllowNull();
/*     */     
/*     */ 
/*  67 */     public static AllowNull get() { return INSTANCE; }
/*     */     
/*     */     protected Schema createFieldSchema(Field field, Map<String, Schema> names) {
/*  70 */       Schema schema = super.createFieldSchema(field, names);
/*  71 */       return makeNullable(schema);
/*     */     }
/*     */   }
/*     */   
/*  75 */   private static final ReflectData INSTANCE = new ReflectData();
/*     */   
/*     */ 
/*     */   public static ReflectData get()
/*     */   {
/*  80 */     return INSTANCE;
/*     */   }
/*     */   
/*     */   public DatumReader createDatumReader(Schema schema) {
/*  84 */     return new ReflectDatumReader(schema, schema, this);
/*     */   }
/*     */   
/*     */   public void setField(Object record, String name, int position, Object o)
/*     */   {
/*  89 */     if ((record instanceof IndexedRecord)) {
/*  90 */       super.setField(record, name, position, o);
/*  91 */       return;
/*     */     }
/*     */     try {
/*  94 */       getField(record.getClass(), name).set(record, o);
/*     */     } catch (IllegalAccessException e) {
/*  96 */       throw new AvroRuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   public Object getField(Object record, String name, int position)
/*     */   {
/* 102 */     if ((record instanceof IndexedRecord))
/* 103 */       return super.getField(record, name, position);
/*     */     try {
/* 105 */       return getField(record.getClass(), name).get(record);
/*     */     } catch (IllegalAccessException e) {
/* 107 */       throw new AvroRuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   protected boolean isRecord(Object datum)
/*     */   {
/* 113 */     if (datum == null) return false;
/* 114 */     if (super.isRecord(datum)) return true;
/* 115 */     if ((datum instanceof Collection)) return false;
/* 116 */     if ((datum instanceof Map)) return false;
/* 117 */     return getSchema(datum.getClass()).getType() == Schema.Type.RECORD;
/*     */   }
/*     */   
/*     */   protected boolean isArray(Object datum)
/*     */   {
/* 122 */     if (datum == null) return false;
/* 123 */     return ((datum instanceof Collection)) || (datum.getClass().isArray());
/*     */   }
/*     */   
/*     */   protected boolean isBytes(Object datum)
/*     */   {
/* 128 */     if (datum == null) return false;
/* 129 */     if (super.isBytes(datum)) return true;
/* 130 */     Class c = datum.getClass();
/* 131 */     return (c.isArray()) && (c.getComponentType() == Byte.TYPE);
/*     */   }
/*     */   
/*     */   protected Schema getRecordSchema(Object record)
/*     */   {
/* 136 */     if ((record instanceof GenericContainer))
/* 137 */       return super.getRecordSchema(record);
/* 138 */     return getSchema(record.getClass());
/*     */   }
/*     */   
/*     */   public boolean validate(Schema schema, Object datum)
/*     */   {
/* 143 */     switch (schema.getType()) {
/*     */     case ARRAY: 
/* 145 */       if (!datum.getClass().isArray())
/* 146 */         return super.validate(schema, datum);
/* 147 */       int length = Array.getLength(datum);
/* 148 */       for (int i = 0; i < length; i++)
/* 149 */         if (!validate(schema.getElementType(), Array.get(datum, i)))
/*     */         {
/* 151 */           return false; }
/* 152 */       return true;
/*     */     }
/* 154 */     return super.validate(schema, datum);
/*     */   }
/*     */   
/*     */ 
/* 158 */   private static final Map<Class, Map<String, Field>> FIELD_CACHE = new ConcurrentHashMap();
/*     */   static final String CLASS_PROP = "java-class";
/*     */   static final String ELEMENT_PROP = "java-element-class";
/*     */   
/*     */   private static Field getField(Class c, String name)
/*     */   {
/* 164 */     Map<String, Field> fields = (Map)FIELD_CACHE.get(c);
/* 165 */     if (fields == null) {
/* 166 */       fields = new ConcurrentHashMap();
/* 167 */       FIELD_CACHE.put(c, fields);
/*     */     }
/* 169 */     Field f = (Field)fields.get(name);
/* 170 */     if (f == null) {
/* 171 */       f = findField(c, name);
/* 172 */       fields.put(name, f);
/*     */     }
/* 174 */     return f;
/*     */   }
/*     */   
/*     */   private static Field findField(Class original, String name) {
/* 178 */     Class c = original;
/*     */     do {
/*     */       try {
/* 181 */         Field f = c.getDeclaredField(name);
/* 182 */         f.setAccessible(true);
/* 183 */         return f;
/*     */       } catch (NoSuchFieldException e) {
/* 185 */         c = c.getSuperclass();
/* 186 */       } } while (c != null);
/* 187 */     throw new AvroRuntimeException("No field named " + name + " in: " + original);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   static Class getClassProp(Schema schema, String prop)
/*     */   {
/* 194 */     String name = schema.getProp(prop);
/* 195 */     if (name == null) return null;
/*     */     try {
/* 197 */       return Class.forName(name);
/*     */     } catch (ClassNotFoundException e) {
/* 199 */       throw new AvroRuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/* 203 */   private static final Class BYTES_CLASS = new byte[0].getClass();
/*     */   
/*     */   public Class getClass(Schema schema)
/*     */   {
/* 207 */     switch (schema.getType()) {
/*     */     case ARRAY: 
/* 209 */       Class collectionClass = getClassProp(schema, "java-class");
/* 210 */       if (collectionClass != null)
/* 211 */         return collectionClass;
/* 212 */       return Array.newInstance(getClass(schema.getElementType()), 0).getClass();
/* 213 */     case STRING:  return String.class;
/* 214 */     case BYTES:  return BYTES_CLASS;
/*     */     case INT: 
/* 216 */       String intClass = schema.getProp("java-class");
/* 217 */       if (Byte.class.getName().equals(intClass)) return Byte.TYPE;
/* 218 */       if (Short.class.getName().equals(intClass)) return Short.TYPE;
/*     */       break; }
/* 220 */     return super.getClass(schema);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected Schema createSchema(Type type, Map<String, Schema> names)
/*     */   {
/* 227 */     if ((type instanceof GenericArrayType)) {
/* 228 */       Type component = ((GenericArrayType)type).getGenericComponentType();
/* 229 */       if (component == Byte.TYPE)
/* 230 */         return Schema.create(Schema.Type.BYTES);
/* 231 */       Schema result = Schema.createArray(createSchema(component, names));
/* 232 */       setElement(result, component);
/* 233 */       return result; }
/* 234 */     if ((type instanceof ParameterizedType)) {
/* 235 */       ParameterizedType ptype = (ParameterizedType)type;
/* 236 */       Class raw = (Class)ptype.getRawType();
/* 237 */       Type[] params = ptype.getActualTypeArguments();
/* 238 */       if (Map.class.isAssignableFrom(raw)) {
/* 239 */         Type key = params[0];
/* 240 */         Type value = params[1];
/* 241 */         if (key != String.class)
/* 242 */           throw new AvroTypeException("Map key class not String: " + key);
/* 243 */         return Schema.createMap(createSchema(value, names)); }
/* 244 */       if (Collection.class.isAssignableFrom(raw)) {
/* 245 */         if (params.length != 1)
/* 246 */           throw new AvroTypeException("No array type specified.");
/* 247 */         Schema schema = Schema.createArray(createSchema(params[0], names));
/* 248 */         schema.addProp("java-class", raw.getName());
/* 249 */         return schema;
/*     */       }
/* 251 */     } else { if ((type == Byte.class) || (type == Byte.TYPE)) {
/* 252 */         Schema result = Schema.create(Schema.Type.INT);
/* 253 */         result.addProp("java-class", Byte.class.getName());
/* 254 */         return result; }
/* 255 */       if ((type == Short.class) || (type == Short.TYPE)) {
/* 256 */         Schema result = Schema.create(Schema.Type.INT);
/* 257 */         result.addProp("java-class", Short.class.getName());
/* 258 */         return result; }
/* 259 */       if ((type instanceof Class)) {
/* 260 */         Class<?> c = (Class)type;
/* 261 */         if ((c.isPrimitive()) || (Number.class.isAssignableFrom(c)) || (c == Void.class) || (c == Boolean.class))
/*     */         {
/* 263 */           return super.createSchema(type, names); }
/* 264 */         if (c.isArray()) {
/* 265 */           Class component = c.getComponentType();
/* 266 */           if (component == Byte.TYPE) {
/* 267 */             Schema result = Schema.create(Schema.Type.BYTES);
/* 268 */             result.addProp("java-class", c.getName());
/* 269 */             return result;
/*     */           }
/* 271 */           Schema result = Schema.createArray(createSchema(component, names));
/* 272 */           result.addProp("java-class", c.getName());
/* 273 */           setElement(result, component);
/* 274 */           return result;
/*     */         }
/* 276 */         if (CharSequence.class.isAssignableFrom(c))
/* 277 */           return Schema.create(Schema.Type.STRING);
/* 278 */         if (ByteBuffer.class.isAssignableFrom(c))
/* 279 */           return Schema.create(Schema.Type.BYTES);
/* 280 */         if (Collection.class.isAssignableFrom(c))
/* 281 */           throw new AvroRuntimeException("Can't find element type of Collection");
/* 282 */         String fullName = c.getName();
/* 283 */         Schema schema = (Schema)names.get(fullName);
/* 284 */         if (schema == null) {
/* 285 */           String name = c.getSimpleName();
/* 286 */           String space = c.getPackage() == null ? "" : c.getPackage().getName();
/* 287 */           if (c.getEnclosingClass() != null)
/* 288 */             space = c.getEnclosingClass().getName() + "$";
/* 289 */           Union union = (Union)c.getAnnotation(Union.class);
/* 290 */           if (union != null)
/* 291 */             return getAnnotatedUnion(union, names);
/* 292 */           if (c.isAnnotationPresent(Stringable.class)) {
/* 293 */             Schema result = Schema.create(Schema.Type.STRING);
/* 294 */             result.addProp("java-class", c.getName());
/* 295 */             return result; }
/* 296 */           if (c.isEnum()) {
/* 297 */             List<String> symbols = new ArrayList();
/* 298 */             Enum[] constants = (Enum[])c.getEnumConstants();
/* 299 */             for (int i = 0; i < constants.length; i++)
/* 300 */               symbols.add(constants[i].name());
/* 301 */             schema = Schema.createEnum(name, null, space, symbols);
/* 302 */           } else if (GenericFixed.class.isAssignableFrom(c)) {
/* 303 */             int size = ((FixedSize)c.getAnnotation(FixedSize.class)).value();
/* 304 */             schema = Schema.createFixed(name, null, space, size);
/* 305 */           } else { if (IndexedRecord.class.isAssignableFrom(c)) {
/* 306 */               return super.createSchema(type, names);
/*     */             }
/* 308 */             List<Schema.Field> fields = new ArrayList();
/* 309 */             boolean error = Throwable.class.isAssignableFrom(c);
/* 310 */             schema = Schema.createRecord(name, null, space, error);
/* 311 */             names.put(c.getName(), schema);
/* 312 */             for (Field field : getFields(c)) {
/* 313 */               if ((field.getModifiers() & 0x88) == 0) {
/* 314 */                 Schema fieldSchema = createFieldSchema(field, names);
/* 315 */                 JsonNode defaultValue = null;
/* 316 */                 if (fieldSchema.getType() == Schema.Type.UNION) {
/* 317 */                   Schema defaultType = (Schema)fieldSchema.getTypes().get(0);
/* 318 */                   if (defaultType.getType() == Schema.Type.NULL) {
/* 319 */                     defaultValue = NullNode.getInstance();
/*     */                   }
/*     */                 }
/* 322 */                 fields.add(new Schema.Field(field.getName(), fieldSchema, null, defaultValue));
/*     */               }
/*     */             }
/* 325 */             if (error) {
/* 326 */               fields.add(new Schema.Field("detailMessage", THROWABLE_MESSAGE, null, null));
/*     */             }
/* 328 */             schema.setFields(fields);
/*     */           }
/* 330 */           names.put(fullName, schema);
/*     */         }
/* 332 */         return schema;
/*     */       } }
/* 334 */     return super.createSchema(type, names);
/*     */   }
/*     */   
/* 337 */   private static final Schema THROWABLE_MESSAGE = makeNullable(Schema.create(Schema.Type.STRING));
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   private void setElement(Schema schema, Type element)
/*     */   {
/* 344 */     if (!(element instanceof Class)) return;
/* 345 */     Class<?> c = (Class)element;
/* 346 */     Union union = (Union)c.getAnnotation(Union.class);
/* 347 */     if (union != null) {
/* 348 */       schema.addProp("java-element-class", c.getName());
/*     */     }
/*     */   }
/*     */   
/*     */   private Schema getAnnotatedUnion(Union union, Map<String, Schema> names) {
/* 353 */     List<Schema> branches = new ArrayList();
/* 354 */     for (Class branch : union.value())
/* 355 */       branches.add(createSchema(branch, names));
/* 356 */     return Schema.createUnion(branches);
/*     */   }
/*     */   
/*     */   public static Schema makeNullable(Schema schema)
/*     */   {
/* 361 */     return Schema.createUnion(Arrays.asList(new Schema[] { Schema.create(Schema.Type.NULL), schema }));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   private Collection<Field> getFields(Class recordClass)
/*     */   {
/* 368 */     Map<String, Field> fields = new LinkedHashMap();
/* 369 */     Class c = recordClass;
/*     */     do {
/* 371 */       if ((c.getPackage() != null) && (c.getPackage().getName().startsWith("java."))) {
/*     */         break;
/*     */       }
/* 374 */       for (Field field : c.getDeclaredFields())
/* 375 */         if (((field.getModifiers() & 0x88) == 0) && 
/* 376 */           (fields.put(field.getName(), field) != null))
/* 377 */           throw new AvroTypeException(c + " contains two fields named: " + field);
/* 378 */       c = c.getSuperclass();
/* 379 */     } while (c != null);
/* 380 */     return fields.values();
/*     */   }
/*     */   
/*     */   protected Schema createFieldSchema(Field field, Map<String, Schema> names)
/*     */   {
/* 385 */     Schema schema = createSchema(field.getGenericType(), names);
/* 386 */     if (field.isAnnotationPresent(Nullable.class))
/* 387 */       schema = makeNullable(schema);
/* 388 */     return schema;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Protocol getProtocol(Class iface)
/*     */   {
/* 398 */     Protocol protocol = new Protocol(iface.getSimpleName(), iface.getPackage() == null ? "" : iface.getPackage().getName());
/*     */     
/*     */ 
/* 401 */     Map<String, Schema> names = new LinkedHashMap();
/* 402 */     Map<String, Protocol.Message> messages = protocol.getMessages();
/* 403 */     for (Method method : iface.getMethods()) {
/* 404 */       if ((method.getModifiers() & 0x8) == 0) {
/* 405 */         String name = method.getName();
/* 406 */         if (messages.containsKey(name))
/* 407 */           throw new AvroTypeException("Two methods with same name: " + name);
/* 408 */         messages.put(name, getMessage(method, protocol, names));
/*     */       }
/*     */     }
/*     */     
/* 412 */     List<Schema> types = new ArrayList();
/* 413 */     types.addAll(names.values());
/* 414 */     Collections.reverse(types);
/* 415 */     protocol.setTypes(types);
/*     */     
/* 417 */     return protocol;
/*     */   }
/*     */   
/* 420 */   private final Paranamer paranamer = new CachingParanamer();
/*     */   
/*     */   private Protocol.Message getMessage(Method method, Protocol protocol, Map<String, Schema> names)
/*     */   {
/* 424 */     List<Schema.Field> fields = new ArrayList();
/* 425 */     String[] paramNames = this.paranamer.lookupParameterNames(method);
/* 426 */     Type[] paramTypes = method.getGenericParameterTypes();
/* 427 */     Annotation[][] annotations = method.getParameterAnnotations();
/* 428 */     for (int i = 0; i < paramTypes.length; i++) {
/* 429 */       Schema paramSchema = getSchema(paramTypes[i], names);
/* 430 */       for (int j = 0; j < annotations[i].length; j++)
/* 431 */         if ((annotations[i][j] instanceof Union)) {
/* 432 */           paramSchema = getAnnotatedUnion((Union)annotations[i][j], names);
/* 433 */         } else if ((annotations[i][j] instanceof Nullable))
/* 434 */           paramSchema = makeNullable(paramSchema);
/* 435 */       String paramName = paramSchema.getName() + i;
/*     */       
/*     */ 
/* 438 */       fields.add(new Schema.Field(paramName, paramSchema, null, null));
/*     */     }
/*     */     
/* 441 */     Schema request = Schema.createRecord(fields);
/*     */     
/* 443 */     Union union = (Union)method.getAnnotation(Union.class);
/* 444 */     Schema response = union == null ? getSchema(method.getGenericReturnType(), names) : getAnnotatedUnion(union, names);
/*     */     
/*     */ 
/* 447 */     if (method.isAnnotationPresent(Nullable.class)) {
/* 448 */       response = makeNullable(response);
/*     */     }
/* 450 */     List<Schema> errs = new ArrayList();
/* 451 */     errs.add(Protocol.SYSTEM_ERROR);
/* 452 */     for (Type err : method.getGenericExceptionTypes())
/* 453 */       if (err != AvroRemoteException.class)
/* 454 */         errs.add(getSchema(err, names));
/* 455 */     Schema errors = Schema.createUnion(errs);
/*     */     
/* 457 */     return protocol.createMessage(method.getName(), null, request, response, errors);
/*     */   }
/*     */   
/*     */   private Schema getSchema(Type type, Map<String, Schema> names) {
/*     */     try {
/* 462 */       return createSchema(type, names);
/*     */     } catch (AvroTypeException e) {
/* 464 */       throw new AvroTypeException("Error getting schema for " + type + ": " + e.getMessage(), e);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   protected int compare(Object o1, Object o2, Schema s, boolean equals)
/*     */   {
/* 471 */     switch (s.getType()) {
/*     */     case ARRAY: 
/* 473 */       if (o1.getClass().isArray())
/*     */       {
/* 475 */         Schema elementType = s.getElementType();
/* 476 */         int l1 = Array.getLength(o1);
/* 477 */         int l2 = Array.getLength(o2);
/* 478 */         int l = Math.min(l1, l2);
/* 479 */         for (int i = 0; i < l; i++) {
/* 480 */           int compare = compare(Array.get(o1, i), Array.get(o2, i), elementType, equals);
/*     */           
/*     */ 
/* 483 */           if (compare != 0) return compare;
/*     */         }
/* 485 */         return l1 - l2;
/*     */       }
/*     */       break; case BYTES:  if (o1.getClass().isArray())
/*     */       {
/* 489 */         byte[] b1 = (byte[])o1;
/* 490 */         byte[] b2 = (byte[])o2;
/* 491 */         return BinaryData.compareBytes(b1, 0, b1.length, b2, 0, b2.length); }
/*     */       break; }
/* 493 */     return super.compare(o1, o2, s, equals);
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\reflect\ReflectData.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */