/*     */ package org.apache.avro.reflect;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.lang.reflect.Array;
/*     */ import java.lang.reflect.Constructor;
/*     */ import java.lang.reflect.InvocationTargetException;
/*     */ import java.nio.ByteBuffer;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Collection;
/*     */ import org.apache.avro.AvroRuntimeException;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.io.Decoder;
/*     */ import org.apache.avro.specific.SpecificDatumReader;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class ReflectDatumReader<T>
/*     */   extends SpecificDatumReader<T>
/*     */ {
/*     */   public ReflectDatumReader()
/*     */   {
/*  38 */     this(null, null, ReflectData.get());
/*     */   }
/*     */   
/*     */   public ReflectDatumReader(Class<T> c) {
/*  42 */     this(ReflectData.get().getSchema(c));
/*     */   }
/*     */   
/*     */   public ReflectDatumReader(Schema root)
/*     */   {
/*  47 */     this(root, root, ReflectData.get());
/*     */   }
/*     */   
/*     */   public ReflectDatumReader(Schema writer, Schema reader)
/*     */   {
/*  52 */     this(writer, reader, ReflectData.get());
/*     */   }
/*     */   
/*     */   protected ReflectDatumReader(Schema writer, Schema reader, ReflectData data) {
/*  56 */     super(writer, reader, data);
/*     */   }
/*     */   
/*     */ 
/*     */   protected Object newArray(Object old, int size, Schema schema)
/*     */   {
/*  62 */     Class collectionClass = ReflectData.getClassProp(schema, "java-class");
/*     */     
/*  64 */     Class elementClass = ReflectData.getClassProp(schema, "java-element-class");
/*     */     
/*     */ 
/*  67 */     if ((collectionClass == null) && (elementClass == null)) {
/*  68 */       return super.newArray(old, size, schema);
/*     */     }
/*  70 */     ReflectData data = (ReflectData)getData();
/*  71 */     if ((collectionClass != null) && (!collectionClass.isArray())) {
/*  72 */       if ((old instanceof Collection)) {
/*  73 */         ((Collection)old).clear();
/*  74 */         return old;
/*     */       }
/*  76 */       if (collectionClass.isAssignableFrom(ArrayList.class))
/*  77 */         return new ArrayList();
/*  78 */       return ReflectData.newInstance(collectionClass, schema);
/*     */     }
/*     */     
/*  81 */     if (elementClass == null)
/*  82 */       elementClass = data.getClass(schema.getElementType());
/*  83 */     return Array.newInstance(elementClass, size);
/*     */   }
/*     */   
/*     */   protected Object peekArray(Object array)
/*     */   {
/*  88 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */   protected void addToArray(Object array, long pos, Object e)
/*     */   {
/*  94 */     if ((array instanceof Collection)) {
/*  95 */       ((Collection)array).add(e);
/*     */     } else {
/*  97 */       Array.set(array, (int)pos, e);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   protected Object readString(Object old, Schema s, Decoder in)
/*     */     throws IOException
/*     */   {
/* 105 */     String value = (String)readString(null, in);
/* 106 */     Class c = ReflectData.getClassProp(s, "java-class");
/* 107 */     if (c != null)
/*     */       try {
/* 109 */         return c.getConstructor(new Class[] { String.class }).newInstance(new Object[] { value });
/*     */       } catch (NoSuchMethodException e) {
/* 111 */         throw new AvroRuntimeException(e);
/*     */       } catch (InstantiationException e) {
/* 113 */         throw new AvroRuntimeException(e);
/*     */       } catch (IllegalAccessException e) {
/* 115 */         throw new AvroRuntimeException(e);
/*     */       } catch (InvocationTargetException e) {
/* 117 */         throw new AvroRuntimeException(e);
/*     */       }
/* 119 */     return value;
/*     */   }
/*     */   
/*     */   protected Object readString(Object old, Decoder in) throws IOException
/*     */   {
/* 124 */     return super.readString(null, in).toString();
/*     */   }
/*     */   
/*     */   protected Object createString(String value) {
/* 128 */     return value;
/*     */   }
/*     */   
/*     */   protected Object readBytes(Object old, Schema s, Decoder in) throws IOException
/*     */   {
/* 133 */     ByteBuffer bytes = in.readBytes(null);
/* 134 */     Class c = ReflectData.getClassProp(s, "java-class");
/* 135 */     if ((c != null) && (c.isArray())) {
/* 136 */       byte[] result = new byte[bytes.remaining()];
/* 137 */       bytes.get(result);
/* 138 */       return result;
/*     */     }
/* 140 */     return bytes;
/*     */   }
/*     */   
/*     */ 
/*     */   protected Object readInt(Object old, Schema expected, Decoder in)
/*     */     throws IOException
/*     */   {
/* 147 */     Object value = Integer.valueOf(in.readInt());
/* 148 */     String intClass = expected.getProp("java-class");
/* 149 */     if (Byte.class.getName().equals(intClass)) {
/* 150 */       value = Byte.valueOf(((Integer)value).byteValue());
/* 151 */     } else if (Short.class.getName().equals(intClass))
/* 152 */       value = Short.valueOf(((Integer)value).shortValue());
/* 153 */     return value;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\reflect\ReflectDatumReader.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */