/*     */ package org.codehaus.jackson.map.deser;
/*     */ 
/*     */ import java.lang.reflect.Constructor;
/*     */ import java.util.HashMap;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedConstructor;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedMethod;
/*     */ import org.codehaus.jackson.map.introspect.BasicBeanDescription;
/*     */ import org.codehaus.jackson.map.util.ClassUtil;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class CreatorContainer
/*     */ {
/*     */   final BasicBeanDescription _beanDesc;
/*     */   final boolean _canFixAccess;
/*     */   protected Constructor<?> _defaultConstructor;
/*     */   AnnotatedMethod _strFactory;
/*     */   AnnotatedMethod _intFactory;
/*     */   AnnotatedMethod _longFactory;
/*     */   AnnotatedMethod _delegatingFactory;
/*     */   AnnotatedMethod _propertyBasedFactory;
/*  27 */   SettableBeanProperty[] _propertyBasedFactoryProperties = null;
/*     */   AnnotatedConstructor _strConstructor;
/*     */   AnnotatedConstructor _intConstructor;
/*     */   AnnotatedConstructor _longConstructor;
/*     */   AnnotatedConstructor _delegatingConstructor;
/*  32 */   AnnotatedConstructor _propertyBasedConstructor; SettableBeanProperty[] _propertyBasedConstructorProperties = null;
/*     */   
/*     */   public CreatorContainer(BasicBeanDescription beanDesc, boolean canFixAccess)
/*     */   {
/*  36 */     this._beanDesc = beanDesc;
/*  37 */     this._canFixAccess = canFixAccess;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void setDefaultConstructor(Constructor<?> ctor)
/*     */   {
/*  48 */     this._defaultConstructor = ctor;
/*     */   }
/*     */   
/*     */   public void addStringConstructor(AnnotatedConstructor ctor) {
/*  52 */     this._strConstructor = verifyNonDup(ctor, this._strConstructor, "String");
/*     */   }
/*     */   
/*  55 */   public void addIntConstructor(AnnotatedConstructor ctor) { this._intConstructor = verifyNonDup(ctor, this._intConstructor, "int"); }
/*     */   
/*     */   public void addLongConstructor(AnnotatedConstructor ctor) {
/*  58 */     this._longConstructor = verifyNonDup(ctor, this._longConstructor, "long");
/*     */   }
/*     */   
/*     */   public void addDelegatingConstructor(AnnotatedConstructor ctor) {
/*  62 */     this._delegatingConstructor = verifyNonDup(ctor, this._delegatingConstructor, "long");
/*     */   }
/*     */   
/*     */   public void addPropertyConstructor(AnnotatedConstructor ctor, SettableBeanProperty[] properties)
/*     */   {
/*  67 */     this._propertyBasedConstructor = verifyNonDup(ctor, this._propertyBasedConstructor, "property-based");
/*     */     
/*  69 */     if (properties.length > 1) {
/*  70 */       HashMap<String, Integer> names = new HashMap();
/*  71 */       int i = 0; for (int len = properties.length; i < len; i++) {
/*  72 */         String name = properties[i].getName();
/*  73 */         Integer old = (Integer)names.put(name, Integer.valueOf(i));
/*  74 */         if (old != null) {
/*  75 */           throw new IllegalArgumentException("Duplicate creator property \"" + name + "\" (index " + old + " vs " + i + ")");
/*     */         }
/*     */       }
/*     */     }
/*  79 */     this._propertyBasedConstructorProperties = properties;
/*     */   }
/*     */   
/*     */   public void addStringFactory(AnnotatedMethod factory) {
/*  83 */     this._strFactory = verifyNonDup(factory, this._strFactory, "String");
/*     */   }
/*     */   
/*  86 */   public void addIntFactory(AnnotatedMethod factory) { this._intFactory = verifyNonDup(factory, this._intFactory, "int"); }
/*     */   
/*     */   public void addLongFactory(AnnotatedMethod factory) {
/*  89 */     this._longFactory = verifyNonDup(factory, this._longFactory, "long");
/*     */   }
/*     */   
/*     */   public void addDelegatingFactory(AnnotatedMethod factory) {
/*  93 */     this._delegatingFactory = verifyNonDup(factory, this._delegatingFactory, "long");
/*     */   }
/*     */   
/*     */   public void addPropertyFactory(AnnotatedMethod factory, SettableBeanProperty[] properties)
/*     */   {
/*  98 */     this._propertyBasedFactory = verifyNonDup(factory, this._propertyBasedFactory, "property-based");
/*  99 */     this._propertyBasedFactoryProperties = properties;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Constructor<?> getDefaultConstructor()
/*     */   {
/* 108 */     return this._defaultConstructor;
/*     */   }
/*     */   
/*     */   public Creator.StringBased stringCreator() {
/* 112 */     if ((this._strConstructor == null) && (this._strFactory == null)) {
/* 113 */       return null;
/*     */     }
/* 115 */     return new Creator.StringBased(this._beanDesc.getBeanClass(), this._strConstructor, this._strFactory);
/*     */   }
/*     */   
/*     */   public Creator.NumberBased numberCreator()
/*     */   {
/* 120 */     if ((this._intConstructor == null) && (this._intFactory == null) && (this._longConstructor == null) && (this._longFactory == null))
/*     */     {
/* 122 */       return null;
/*     */     }
/* 124 */     return new Creator.NumberBased(this._beanDesc.getBeanClass(), this._intConstructor, this._intFactory, this._longConstructor, this._longFactory);
/*     */   }
/*     */   
/*     */ 
/*     */   public Creator.Delegating delegatingCreator()
/*     */   {
/* 130 */     if ((this._delegatingConstructor == null) && (this._delegatingFactory == null)) {
/* 131 */       return null;
/*     */     }
/* 133 */     return new Creator.Delegating(this._beanDesc, this._delegatingConstructor, this._delegatingFactory);
/*     */   }
/*     */   
/*     */   public Creator.PropertyBased propertyBasedCreator()
/*     */   {
/* 138 */     if ((this._propertyBasedConstructor == null) && (this._propertyBasedFactory == null)) {
/* 139 */       return null;
/*     */     }
/* 141 */     return new Creator.PropertyBased(this._propertyBasedConstructor, this._propertyBasedConstructorProperties, this._propertyBasedFactory, this._propertyBasedFactoryProperties);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected AnnotatedConstructor verifyNonDup(AnnotatedConstructor newOne, AnnotatedConstructor oldOne, String type)
/*     */   {
/* 154 */     if (oldOne != null) {
/* 155 */       throw new IllegalArgumentException("Conflicting " + type + " constructors: already had " + oldOne + ", encountered " + newOne);
/*     */     }
/* 157 */     if (this._canFixAccess) {
/* 158 */       ClassUtil.checkAndFixAccess(newOne.getAnnotated());
/*     */     }
/* 160 */     return newOne;
/*     */   }
/*     */   
/*     */ 
/*     */   protected AnnotatedMethod verifyNonDup(AnnotatedMethod newOne, AnnotatedMethod oldOne, String type)
/*     */   {
/* 166 */     if (oldOne != null) {
/* 167 */       throw new IllegalArgumentException("Conflicting " + type + " factory methods: already had " + oldOne + ", encountered " + newOne);
/*     */     }
/* 169 */     if (this._canFixAccess) {
/* 170 */       ClassUtil.checkAndFixAccess(newOne.getAnnotated());
/*     */     }
/* 172 */     return newOne;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\deser\CreatorContainer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */