/*      */ package org.codehaus.jackson.map.deser;
/*      */ 
/*      */ import java.io.IOException;
/*      */ import java.math.BigDecimal;
/*      */ import java.math.BigInteger;
/*      */ import java.util.Calendar;
/*      */ import java.util.concurrent.atomic.AtomicBoolean;
/*      */ import java.util.concurrent.atomic.AtomicReference;
/*      */ import org.codehaus.jackson.Base64Variant;
/*      */ import org.codehaus.jackson.Base64Variants;
/*      */ import org.codehaus.jackson.JsonParser;
/*      */ import org.codehaus.jackson.JsonProcessingException;
/*      */ import org.codehaus.jackson.JsonToken;
/*      */ import org.codehaus.jackson.io.NumberInput;
/*      */ import org.codehaus.jackson.map.BeanProperty;
/*      */ import org.codehaus.jackson.map.DeserializationConfig;
/*      */ import org.codehaus.jackson.map.DeserializationConfig.Feature;
/*      */ import org.codehaus.jackson.map.DeserializationContext;
/*      */ import org.codehaus.jackson.map.DeserializerProvider;
/*      */ import org.codehaus.jackson.map.JsonDeserializer;
/*      */ import org.codehaus.jackson.map.JsonMappingException;
/*      */ import org.codehaus.jackson.map.ResolvableDeserializer;
/*      */ import org.codehaus.jackson.map.TypeDeserializer;
/*      */ import org.codehaus.jackson.map.annotate.JacksonStdImpl;
/*      */ import org.codehaus.jackson.type.JavaType;
/*      */ import org.codehaus.jackson.util.TokenBuffer;
/*      */ 
/*      */ 
/*      */ 
/*      */ public abstract class StdDeserializer<T>
/*      */   extends JsonDeserializer<T>
/*      */ {
/*      */   protected final Class<?> _valueClass;
/*      */   
/*      */   protected StdDeserializer(Class<?> vc)
/*      */   {
/*   37 */     this._valueClass = vc;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   protected StdDeserializer(JavaType valueType)
/*      */   {
/*   44 */     this._valueClass = (valueType == null ? null : valueType.getRawClass());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Class<?> getValueClass()
/*      */   {
/*   53 */     return this._valueClass;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public JavaType getValueType()
/*      */   {
/*   60 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected boolean isDefaultSerializer(JsonDeserializer<?> deserializer)
/*      */   {
/*   72 */     return (deserializer != null) && (deserializer.getClass().getAnnotation(JacksonStdImpl.class) != null);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Object deserializeWithType(JsonParser jp, DeserializationContext ctxt, TypeDeserializer typeDeserializer)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/*   91 */     return typeDeserializer.deserializeTypedFromAny(jp, ctxt);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected final boolean _parseBooleanPrimitive(JsonParser jp, DeserializationContext ctxt)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/*  103 */     JsonToken t = jp.getCurrentToken();
/*  104 */     if (t == JsonToken.VALUE_TRUE) {
/*  105 */       return true;
/*      */     }
/*  107 */     if (t == JsonToken.VALUE_FALSE) {
/*  108 */       return false;
/*      */     }
/*  110 */     if (t == JsonToken.VALUE_NULL) {
/*  111 */       return false;
/*      */     }
/*      */     
/*  114 */     if (t == JsonToken.VALUE_NUMBER_INT) {
/*  115 */       return jp.getIntValue() != 0;
/*      */     }
/*      */     
/*  118 */     if (t == JsonToken.VALUE_STRING) {
/*  119 */       String text = jp.getText().trim();
/*  120 */       if ("true".equals(text)) {
/*  121 */         return true;
/*      */       }
/*  123 */       if (("false".equals(text)) || (text.length() == 0)) {
/*  124 */         return Boolean.FALSE.booleanValue();
/*      */       }
/*  126 */       throw ctxt.weirdStringException(this._valueClass, "only \"true\" or \"false\" recognized");
/*      */     }
/*      */     
/*  129 */     throw ctxt.mappingException(this._valueClass);
/*      */   }
/*      */   
/*      */   protected final Boolean _parseBoolean(JsonParser jp, DeserializationContext ctxt)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/*  135 */     JsonToken t = jp.getCurrentToken();
/*  136 */     if (t == JsonToken.VALUE_TRUE) {
/*  137 */       return Boolean.TRUE;
/*      */     }
/*  139 */     if (t == JsonToken.VALUE_FALSE) {
/*  140 */       return Boolean.FALSE;
/*      */     }
/*  142 */     if (t == JsonToken.VALUE_NULL) {
/*  143 */       return null;
/*      */     }
/*      */     
/*  146 */     if (t == JsonToken.VALUE_NUMBER_INT) {
/*  147 */       return jp.getIntValue() == 0 ? Boolean.FALSE : Boolean.TRUE;
/*      */     }
/*      */     
/*  150 */     if (t == JsonToken.VALUE_STRING) {
/*  151 */       String text = jp.getText().trim();
/*  152 */       if ("true".equals(text)) {
/*  153 */         return Boolean.TRUE;
/*      */       }
/*  155 */       if (("false".equals(text)) || (text.length() == 0)) {
/*  156 */         return Boolean.FALSE;
/*      */       }
/*  158 */       throw ctxt.weirdStringException(this._valueClass, "only \"true\" or \"false\" recognized");
/*      */     }
/*      */     
/*  161 */     throw ctxt.mappingException(this._valueClass);
/*      */   }
/*      */   
/*      */   protected final Short _parseShort(JsonParser jp, DeserializationContext ctxt)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/*  167 */     JsonToken t = jp.getCurrentToken();
/*  168 */     if (t == JsonToken.VALUE_NULL) {
/*  169 */       return null;
/*      */     }
/*  171 */     if ((t == JsonToken.VALUE_NUMBER_INT) || (t == JsonToken.VALUE_NUMBER_FLOAT)) {
/*  172 */       return Short.valueOf(jp.getShortValue());
/*      */     }
/*  174 */     int value = _parseIntPrimitive(jp, ctxt);
/*      */     
/*  176 */     if ((value < 32768) || (value > 32767)) {
/*  177 */       throw ctxt.weirdStringException(this._valueClass, "overflow, value can not be represented as 16-bit value");
/*      */     }
/*  179 */     return Short.valueOf((short)value);
/*      */   }
/*      */   
/*      */   protected final short _parseShortPrimitive(JsonParser jp, DeserializationContext ctxt)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/*  185 */     int value = _parseIntPrimitive(jp, ctxt);
/*      */     
/*  187 */     if ((value < 32768) || (value > 32767)) {
/*  188 */       throw ctxt.weirdStringException(this._valueClass, "overflow, value can not be represented as 16-bit value");
/*      */     }
/*  190 */     return (short)value;
/*      */   }
/*      */   
/*      */   protected final int _parseIntPrimitive(JsonParser jp, DeserializationContext ctxt)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/*  196 */     JsonToken t = jp.getCurrentToken();
/*      */     
/*      */ 
/*  199 */     if ((t == JsonToken.VALUE_NUMBER_INT) || (t == JsonToken.VALUE_NUMBER_FLOAT)) {
/*  200 */       return jp.getIntValue();
/*      */     }
/*  202 */     if (t == JsonToken.VALUE_STRING)
/*      */     {
/*      */ 
/*      */ 
/*  206 */       String text = jp.getText().trim();
/*      */       try {
/*  208 */         int len = text.length();
/*  209 */         if (len > 9) {
/*  210 */           long l = Long.parseLong(text);
/*  211 */           if ((l < -2147483648L) || (l > 2147483647L)) {
/*  212 */             throw ctxt.weirdStringException(this._valueClass, "Overflow: numeric value (" + text + ") out of range of int (" + Integer.MIN_VALUE + " - " + Integer.MAX_VALUE + ")");
/*      */           }
/*      */           
/*  215 */           return (int)l;
/*      */         }
/*  217 */         if (len == 0) {
/*  218 */           return 0;
/*      */         }
/*  220 */         return NumberInput.parseInt(text);
/*      */       } catch (IllegalArgumentException iae) {
/*  222 */         throw ctxt.weirdStringException(this._valueClass, "not a valid int value");
/*      */       }
/*      */     }
/*  225 */     if (t == JsonToken.VALUE_NULL) {
/*  226 */       return 0;
/*      */     }
/*      */     
/*  229 */     throw ctxt.mappingException(this._valueClass);
/*      */   }
/*      */   
/*      */   protected final Integer _parseInteger(JsonParser jp, DeserializationContext ctxt)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/*  235 */     JsonToken t = jp.getCurrentToken();
/*  236 */     if ((t == JsonToken.VALUE_NUMBER_INT) || (t == JsonToken.VALUE_NUMBER_FLOAT)) {
/*  237 */       return Integer.valueOf(jp.getIntValue());
/*      */     }
/*  239 */     if (t == JsonToken.VALUE_STRING) {
/*  240 */       String text = jp.getText().trim();
/*      */       try {
/*  242 */         int len = text.length();
/*  243 */         if (len > 9) {
/*  244 */           long l = Long.parseLong(text);
/*  245 */           if ((l < -2147483648L) || (l > 2147483647L)) {
/*  246 */             throw ctxt.weirdStringException(this._valueClass, "Overflow: numeric value (" + text + ") out of range of Integer (" + Integer.MIN_VALUE + " - " + Integer.MAX_VALUE + ")");
/*      */           }
/*      */           
/*  249 */           return Integer.valueOf((int)l);
/*      */         }
/*  251 */         if (len == 0) {
/*  252 */           return null;
/*      */         }
/*  254 */         return Integer.valueOf(NumberInput.parseInt(text));
/*      */       } catch (IllegalArgumentException iae) {
/*  256 */         throw ctxt.weirdStringException(this._valueClass, "not a valid Integer value");
/*      */       }
/*      */     }
/*  259 */     if (t == JsonToken.VALUE_NULL) {
/*  260 */       return null;
/*      */     }
/*      */     
/*  263 */     throw ctxt.mappingException(this._valueClass);
/*      */   }
/*      */   
/*      */   protected final Long _parseLong(JsonParser jp, DeserializationContext ctxt)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/*  269 */     JsonToken t = jp.getCurrentToken();
/*      */     
/*      */ 
/*  272 */     if ((t == JsonToken.VALUE_NUMBER_INT) || (t == JsonToken.VALUE_NUMBER_FLOAT)) {
/*  273 */       return Long.valueOf(jp.getLongValue());
/*      */     }
/*      */     
/*  276 */     if (t == JsonToken.VALUE_STRING)
/*      */     {
/*  278 */       String text = jp.getText().trim();
/*  279 */       if (text.length() == 0) {
/*  280 */         return null;
/*      */       }
/*      */       try {
/*  283 */         return Long.valueOf(NumberInput.parseLong(text));
/*      */       } catch (IllegalArgumentException iae) {
/*  285 */         throw ctxt.weirdStringException(this._valueClass, "not a valid Long value");
/*      */       } }
/*  287 */     if (t == JsonToken.VALUE_NULL) {
/*  288 */       return null;
/*      */     }
/*      */     
/*  291 */     throw ctxt.mappingException(this._valueClass);
/*      */   }
/*      */   
/*      */   protected final long _parseLongPrimitive(JsonParser jp, DeserializationContext ctxt)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/*  297 */     JsonToken t = jp.getCurrentToken();
/*  298 */     if ((t == JsonToken.VALUE_NUMBER_INT) || (t == JsonToken.VALUE_NUMBER_FLOAT)) {
/*  299 */       return jp.getLongValue();
/*      */     }
/*  301 */     if (t == JsonToken.VALUE_STRING) {
/*  302 */       String text = jp.getText().trim();
/*  303 */       if (text.length() == 0) {
/*  304 */         return 0L;
/*      */       }
/*      */       try {
/*  307 */         return NumberInput.parseLong(text);
/*      */       } catch (IllegalArgumentException iae) {
/*  309 */         throw ctxt.weirdStringException(this._valueClass, "not a valid long value");
/*      */       } }
/*  311 */     if (t == JsonToken.VALUE_NULL) {
/*  312 */       return 0L;
/*      */     }
/*  314 */     throw ctxt.mappingException(this._valueClass);
/*      */   }
/*      */   
/*      */ 
/*      */   protected final Float _parseFloat(JsonParser jp, DeserializationContext ctxt)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/*  321 */     JsonToken t = jp.getCurrentToken();
/*      */     
/*  323 */     if ((t == JsonToken.VALUE_NUMBER_INT) || (t == JsonToken.VALUE_NUMBER_FLOAT)) {
/*  324 */       return Float.valueOf(jp.getFloatValue());
/*      */     }
/*      */     
/*  327 */     if (t == JsonToken.VALUE_STRING) {
/*  328 */       String text = jp.getText().trim();
/*  329 */       if (text.length() == 0) {
/*  330 */         return null;
/*      */       }
/*  332 */       switch (text.charAt(0)) {
/*      */       case 'I': 
/*  334 */         if (("Infinity".equals(text)) || ("INF".equals(text))) {
/*  335 */           return Float.valueOf(Float.POSITIVE_INFINITY);
/*      */         }
/*      */         break;
/*      */       case 'N': 
/*  339 */         if ("NaN".equals(text)) {
/*  340 */           return Float.valueOf(NaN.0F);
/*      */         }
/*      */         break;
/*      */       case '-': 
/*  344 */         if (("-Infinity".equals(text)) || ("-INF".equals(text))) {
/*  345 */           return Float.valueOf(Float.NEGATIVE_INFINITY);
/*      */         }
/*      */         break;
/*      */       }
/*      */       try {
/*  350 */         return Float.valueOf(Float.parseFloat(text));
/*      */       } catch (IllegalArgumentException iae) {
/*  352 */         throw ctxt.weirdStringException(this._valueClass, "not a valid Float value");
/*      */       } }
/*  354 */     if (t == JsonToken.VALUE_NULL) {
/*  355 */       return null;
/*      */     }
/*      */     
/*  358 */     throw ctxt.mappingException(this._valueClass);
/*      */   }
/*      */   
/*      */   protected final float _parseFloatPrimitive(JsonParser jp, DeserializationContext ctxt)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/*  364 */     JsonToken t = jp.getCurrentToken();
/*      */     
/*  366 */     if ((t == JsonToken.VALUE_NUMBER_INT) || (t == JsonToken.VALUE_NUMBER_FLOAT)) {
/*  367 */       return jp.getFloatValue();
/*      */     }
/*  369 */     if (t == JsonToken.VALUE_STRING) {
/*  370 */       String text = jp.getText().trim();
/*  371 */       if (text.length() == 0) {
/*  372 */         return 0.0F;
/*      */       }
/*  374 */       switch (text.charAt(0)) {
/*      */       case 'I': 
/*  376 */         if (("Infinity".equals(text)) || ("INF".equals(text))) {
/*  377 */           return Float.POSITIVE_INFINITY;
/*      */         }
/*      */         break;
/*      */       case 'N': 
/*  381 */         if ("NaN".equals(text)) {
/*  382 */           return NaN.0F;
/*      */         }
/*      */         break;
/*      */       case '-': 
/*  386 */         if (("-Infinity".equals(text)) || ("-INF".equals(text))) {
/*  387 */           return Float.NEGATIVE_INFINITY;
/*      */         }
/*      */         break;
/*      */       }
/*      */       try {
/*  392 */         return Float.parseFloat(text);
/*      */       } catch (IllegalArgumentException iae) {
/*  394 */         throw ctxt.weirdStringException(this._valueClass, "not a valid float value");
/*      */       } }
/*  396 */     if (t == JsonToken.VALUE_NULL) {
/*  397 */       return 0.0F;
/*      */     }
/*      */     
/*  400 */     throw ctxt.mappingException(this._valueClass);
/*      */   }
/*      */   
/*      */   protected final Double _parseDouble(JsonParser jp, DeserializationContext ctxt)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/*  406 */     JsonToken t = jp.getCurrentToken();
/*      */     
/*  408 */     if ((t == JsonToken.VALUE_NUMBER_INT) || (t == JsonToken.VALUE_NUMBER_FLOAT)) {
/*  409 */       return Double.valueOf(jp.getDoubleValue());
/*      */     }
/*  411 */     if (t == JsonToken.VALUE_STRING) {
/*  412 */       String text = jp.getText().trim();
/*  413 */       if (text.length() == 0) {
/*  414 */         return null;
/*      */       }
/*  416 */       switch (text.charAt(0)) {
/*      */       case 'I': 
/*  418 */         if (("Infinity".equals(text)) || ("INF".equals(text))) {
/*  419 */           return Double.valueOf(Double.POSITIVE_INFINITY);
/*      */         }
/*      */         break;
/*      */       case 'N': 
/*  423 */         if ("NaN".equals(text)) {
/*  424 */           return Double.valueOf(NaN.0D);
/*      */         }
/*      */         break;
/*      */       case '-': 
/*  428 */         if (("-Infinity".equals(text)) || ("-INF".equals(text))) {
/*  429 */           return Double.valueOf(Double.NEGATIVE_INFINITY);
/*      */         }
/*      */         break;
/*      */       }
/*      */       try {
/*  434 */         return Double.valueOf(parseDouble(text));
/*      */       } catch (IllegalArgumentException iae) {
/*  436 */         throw ctxt.weirdStringException(this._valueClass, "not a valid Double value");
/*      */       } }
/*  438 */     if (t == JsonToken.VALUE_NULL) {
/*  439 */       return null;
/*      */     }
/*      */     
/*  442 */     throw ctxt.mappingException(this._valueClass);
/*      */   }
/*      */   
/*      */ 
/*      */   protected final double _parseDoublePrimitive(JsonParser jp, DeserializationContext ctxt)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/*  449 */     JsonToken t = jp.getCurrentToken();
/*      */     
/*  451 */     if ((t == JsonToken.VALUE_NUMBER_INT) || (t == JsonToken.VALUE_NUMBER_FLOAT)) {
/*  452 */       return jp.getDoubleValue();
/*      */     }
/*      */     
/*  455 */     if (t == JsonToken.VALUE_STRING) {
/*  456 */       String text = jp.getText().trim();
/*  457 */       if (text.length() == 0) {
/*  458 */         return 0.0D;
/*      */       }
/*  460 */       switch (text.charAt(0)) {
/*      */       case 'I': 
/*  462 */         if (("Infinity".equals(text)) || ("INF".equals(text))) {
/*  463 */           return Double.POSITIVE_INFINITY;
/*      */         }
/*      */         break;
/*      */       case 'N': 
/*  467 */         if ("NaN".equals(text)) {
/*  468 */           return NaN.0D;
/*      */         }
/*      */         break;
/*      */       case '-': 
/*  472 */         if (("-Infinity".equals(text)) || ("-INF".equals(text))) {
/*  473 */           return Double.NEGATIVE_INFINITY;
/*      */         }
/*      */         break;
/*      */       }
/*      */       try {
/*  478 */         return parseDouble(text);
/*      */       } catch (IllegalArgumentException iae) {
/*  480 */         throw ctxt.weirdStringException(this._valueClass, "not a valid double value");
/*      */       } }
/*  482 */     if (t == JsonToken.VALUE_NULL) {
/*  483 */       return 0.0D;
/*      */     }
/*      */     
/*  486 */     throw ctxt.mappingException(this._valueClass);
/*      */   }
/*      */   
/*      */ 
/*      */   protected java.util.Date _parseDate(JsonParser jp, DeserializationContext ctxt)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/*  493 */     JsonToken t = jp.getCurrentToken();
/*      */     try {
/*  495 */       if (t == JsonToken.VALUE_NUMBER_INT) {
/*  496 */         return new java.util.Date(jp.getLongValue());
/*      */       }
/*  498 */       if (t == JsonToken.VALUE_STRING)
/*      */       {
/*      */ 
/*      */ 
/*  502 */         String str = jp.getText().trim();
/*  503 */         if (str.length() == 0) {
/*  504 */           return null;
/*      */         }
/*  506 */         return ctxt.parseDate(str);
/*      */       }
/*  508 */       throw ctxt.mappingException(this._valueClass);
/*      */     } catch (IllegalArgumentException iae) {
/*  510 */       throw ctxt.weirdStringException(this._valueClass, "not a valid representation (error: " + iae.getMessage() + ")");
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected static final double parseDouble(String numStr)
/*      */     throws NumberFormatException
/*      */   {
/*  524 */     if ("2.2250738585072012e-308".equals(numStr)) {
/*  525 */       return 2.2250738585072014E-308D;
/*      */     }
/*  527 */     return Double.parseDouble(numStr);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected JsonDeserializer<Object> findDeserializer(DeserializationConfig config, DeserializerProvider provider, JavaType type, BeanProperty property)
/*      */     throws JsonMappingException
/*      */   {
/*  551 */     JsonDeserializer<Object> deser = provider.findValueDeserializer(config, type, property);
/*  552 */     return deser;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected void handleUnknownProperty(JsonParser jp, DeserializationContext ctxt, Object instanceOrClass, String propName)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/*  582 */     if (instanceOrClass == null) {
/*  583 */       instanceOrClass = getValueClass();
/*      */     }
/*      */     
/*  586 */     if (ctxt.handleUnknownProperty(jp, this, instanceOrClass, propName)) {
/*  587 */       return;
/*      */     }
/*      */     
/*  590 */     reportUnknownProperty(ctxt, instanceOrClass, propName);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  595 */     jp.skipChildren();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   protected void reportUnknownProperty(DeserializationContext ctxt, Object instanceOrClass, String fieldName)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/*  603 */     if (ctxt.isEnabled(DeserializationConfig.Feature.FAIL_ON_UNKNOWN_PROPERTIES)) {
/*  604 */       throw ctxt.unknownFieldException(instanceOrClass, fieldName);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected static abstract class PrimitiveOrWrapperDeserializer<T>
/*      */     extends StdScalarDeserializer<T>
/*      */   {
/*      */     final T _nullValue;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     protected PrimitiveOrWrapperDeserializer(Class<T> vc, T nvl)
/*      */     {
/*  624 */       super();
/*  625 */       this._nullValue = nvl;
/*      */     }
/*      */     
/*      */     public final T getNullValue()
/*      */     {
/*  630 */       return (T)this._nullValue;
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @JacksonStdImpl
/*      */   public static final class StringDeserializer
/*      */     extends StdScalarDeserializer<String>
/*      */   {
/*      */     public StringDeserializer()
/*      */     {
/*  644 */       super();
/*      */     }
/*      */     
/*      */     public String deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/*  650 */       JsonToken curr = jp.getCurrentToken();
/*      */       
/*  652 */       if (curr == JsonToken.VALUE_STRING) {
/*  653 */         return jp.getText();
/*      */       }
/*      */       
/*  656 */       if (curr == JsonToken.VALUE_EMBEDDED_OBJECT) {
/*  657 */         Object ob = jp.getEmbeddedObject();
/*  658 */         if (ob == null) {
/*  659 */           return null;
/*      */         }
/*  661 */         if ((ob instanceof byte[])) {
/*  662 */           return Base64Variants.getDefaultVariant().encode((byte[])ob, false);
/*      */         }
/*      */         
/*  665 */         return ob.toString();
/*      */       }
/*      */       
/*  668 */       if (curr.isScalarValue()) {
/*  669 */         return jp.getText();
/*      */       }
/*  671 */       throw ctxt.mappingException(this._valueClass);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public String deserializeWithType(JsonParser jp, DeserializationContext ctxt, TypeDeserializer typeDeserializer)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/*  681 */       return deserialize(jp, ctxt);
/*      */     }
/*      */   }
/*      */   
/*      */   @JacksonStdImpl
/*      */   public static final class ClassDeserializer extends StdScalarDeserializer<Class<?>>
/*      */   {
/*      */     public ClassDeserializer() {
/*  689 */       super();
/*      */     }
/*      */     
/*      */     public Class<?> deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/*  695 */       JsonToken curr = jp.getCurrentToken();
/*      */       
/*  697 */       if (curr == JsonToken.VALUE_STRING) {
/*  698 */         String className = jp.getText();
/*      */         
/*  700 */         if (className.indexOf('.') < 0) {
/*  701 */           if ("int".equals(className)) return Integer.TYPE;
/*  702 */           if ("long".equals(className)) return Long.TYPE;
/*  703 */           if ("float".equals(className)) return Float.TYPE;
/*  704 */           if ("double".equals(className)) return Double.TYPE;
/*  705 */           if ("boolean".equals(className)) return Boolean.TYPE;
/*  706 */           if ("byte".equals(className)) return Byte.TYPE;
/*  707 */           if ("char".equals(className)) return Character.TYPE;
/*  708 */           if ("short".equals(className)) return Short.TYPE;
/*  709 */           if ("void".equals(className)) return Void.TYPE;
/*      */         }
/*      */         try {
/*  712 */           return Class.forName(jp.getText());
/*      */         } catch (ClassNotFoundException e) {
/*  714 */           throw ctxt.instantiationException(this._valueClass, e);
/*      */         }
/*      */       }
/*  717 */       throw ctxt.mappingException(this._valueClass);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @JacksonStdImpl
/*      */   public static final class BooleanDeserializer
/*      */     extends StdDeserializer.PrimitiveOrWrapperDeserializer<Boolean>
/*      */   {
/*      */     public BooleanDeserializer(Class<Boolean> cls, Boolean nvl)
/*      */     {
/*  733 */       super(nvl);
/*      */     }
/*      */     
/*      */ 
/*      */     public Boolean deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/*  740 */       return _parseBoolean(jp, ctxt);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public Boolean deserializeWithType(JsonParser jp, DeserializationContext ctxt, TypeDeserializer typeDeserializer)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/*  750 */       return _parseBoolean(jp, ctxt);
/*      */     }
/*      */   }
/*      */   
/*      */   @JacksonStdImpl
/*      */   public static final class ByteDeserializer
/*      */     extends StdDeserializer.PrimitiveOrWrapperDeserializer<Byte>
/*      */   {
/*      */     public ByteDeserializer(Class<Byte> cls, Byte nvl)
/*      */     {
/*  760 */       super(nvl);
/*      */     }
/*      */     
/*      */ 
/*      */     public Byte deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/*  767 */       int value = _parseIntPrimitive(jp, ctxt);
/*      */       
/*  769 */       if ((value < -128) || (value > 127)) {
/*  770 */         throw ctxt.weirdStringException(this._valueClass, "overflow, value can not be represented as 8-bit value");
/*      */       }
/*  772 */       return Byte.valueOf((byte)value);
/*      */     }
/*      */   }
/*      */   
/*      */   @JacksonStdImpl
/*      */   public static final class ShortDeserializer
/*      */     extends StdDeserializer.PrimitiveOrWrapperDeserializer<Short>
/*      */   {
/*      */     public ShortDeserializer(Class<Short> cls, Short nvl)
/*      */     {
/*  782 */       super(nvl);
/*      */     }
/*      */     
/*      */ 
/*      */     public Short deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/*  789 */       return _parseShort(jp, ctxt);
/*      */     }
/*      */   }
/*      */   
/*      */   @JacksonStdImpl
/*      */   public static final class CharacterDeserializer
/*      */     extends StdDeserializer.PrimitiveOrWrapperDeserializer<Character>
/*      */   {
/*      */     public CharacterDeserializer(Class<Character> cls, Character nvl)
/*      */     {
/*  799 */       super(nvl);
/*      */     }
/*      */     
/*      */ 
/*      */     public Character deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/*  806 */       JsonToken t = jp.getCurrentToken();
/*      */       
/*      */ 
/*  809 */       if (t == JsonToken.VALUE_NUMBER_INT) {
/*  810 */         int value = jp.getIntValue();
/*  811 */         if ((value >= 0) && (value <= 65535)) {
/*  812 */           return Character.valueOf((char)value);
/*      */         }
/*  814 */       } else if (t == JsonToken.VALUE_STRING)
/*      */       {
/*  816 */         String text = jp.getText();
/*  817 */         if (text.length() == 1) {
/*  818 */           return Character.valueOf(text.charAt(0));
/*      */         }
/*      */       }
/*  821 */       throw ctxt.mappingException(this._valueClass);
/*      */     }
/*      */   }
/*      */   
/*      */   @JacksonStdImpl
/*      */   public static final class IntegerDeserializer
/*      */     extends StdDeserializer.PrimitiveOrWrapperDeserializer<Integer>
/*      */   {
/*      */     public IntegerDeserializer(Class<Integer> cls, Integer nvl)
/*      */     {
/*  831 */       super(nvl);
/*      */     }
/*      */     
/*      */ 
/*      */     public Integer deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/*  838 */       return _parseInteger(jp, ctxt);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public Integer deserializeWithType(JsonParser jp, DeserializationContext ctxt, TypeDeserializer typeDeserializer)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/*  848 */       return _parseInteger(jp, ctxt);
/*      */     }
/*      */   }
/*      */   
/*      */   @JacksonStdImpl
/*      */   public static final class LongDeserializer
/*      */     extends StdDeserializer.PrimitiveOrWrapperDeserializer<Long>
/*      */   {
/*      */     public LongDeserializer(Class<Long> cls, Long nvl)
/*      */     {
/*  858 */       super(nvl);
/*      */     }
/*      */     
/*      */ 
/*      */     public Long deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/*  865 */       return _parseLong(jp, ctxt);
/*      */     }
/*      */   }
/*      */   
/*      */   @JacksonStdImpl
/*      */   public static final class FloatDeserializer
/*      */     extends StdDeserializer.PrimitiveOrWrapperDeserializer<Float>
/*      */   {
/*      */     public FloatDeserializer(Class<Float> cls, Float nvl)
/*      */     {
/*  875 */       super(nvl);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public Float deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/*  885 */       return _parseFloat(jp, ctxt);
/*      */     }
/*      */   }
/*      */   
/*      */   @JacksonStdImpl
/*      */   public static final class DoubleDeserializer
/*      */     extends StdDeserializer.PrimitiveOrWrapperDeserializer<Double>
/*      */   {
/*      */     public DoubleDeserializer(Class<Double> cls, Double nvl)
/*      */     {
/*  895 */       super(nvl);
/*      */     }
/*      */     
/*      */ 
/*      */     public Double deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/*  902 */       return _parseDouble(jp, ctxt);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public Double deserializeWithType(JsonParser jp, DeserializationContext ctxt, TypeDeserializer typeDeserializer)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/*  912 */       return _parseDouble(jp, ctxt);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @JacksonStdImpl
/*      */   public static final class NumberDeserializer
/*      */     extends StdScalarDeserializer<Number>
/*      */   {
/*      */     public NumberDeserializer()
/*      */     {
/*  930 */       super();
/*      */     }
/*      */     
/*      */     public Number deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/*  936 */       JsonToken t = jp.getCurrentToken();
/*  937 */       if (t == JsonToken.VALUE_NUMBER_INT) {
/*  938 */         if (ctxt.isEnabled(DeserializationConfig.Feature.USE_BIG_INTEGER_FOR_INTS)) {
/*  939 */           return jp.getBigIntegerValue();
/*      */         }
/*  941 */         return jp.getNumberValue(); }
/*  942 */       if (t == JsonToken.VALUE_NUMBER_FLOAT)
/*      */       {
/*      */ 
/*      */ 
/*  946 */         if (ctxt.isEnabled(DeserializationConfig.Feature.USE_BIG_DECIMAL_FOR_FLOATS)) {
/*  947 */           return jp.getDecimalValue();
/*      */         }
/*  949 */         return Double.valueOf(jp.getDoubleValue());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*  955 */       if (t == JsonToken.VALUE_STRING) {
/*  956 */         String text = jp.getText().trim();
/*      */         try {
/*  958 */           if (text.indexOf('.') >= 0)
/*      */           {
/*  960 */             if (ctxt.isEnabled(DeserializationConfig.Feature.USE_BIG_DECIMAL_FOR_FLOATS)) {
/*  961 */               return new BigDecimal(text);
/*      */             }
/*  963 */             return new Double(text);
/*      */           }
/*      */           
/*  966 */           if (ctxt.isEnabled(DeserializationConfig.Feature.USE_BIG_INTEGER_FOR_INTS)) {
/*  967 */             return new BigInteger(text);
/*      */           }
/*  969 */           long value = Long.parseLong(text);
/*  970 */           if ((value <= 2147483647L) && (value >= -2147483648L)) {
/*  971 */             return Integer.valueOf((int)value);
/*      */           }
/*  973 */           return Long.valueOf(value);
/*      */         } catch (IllegalArgumentException iae) {
/*  975 */           throw ctxt.weirdStringException(this._valueClass, "not a valid number");
/*      */         }
/*      */       }
/*      */       
/*  979 */       throw ctxt.mappingException(this._valueClass);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public Object deserializeWithType(JsonParser jp, DeserializationContext ctxt, TypeDeserializer typeDeserializer)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/*  993 */       switch (StdDeserializer.1.$SwitchMap$org$codehaus$jackson$JsonToken[jp.getCurrentToken().ordinal()])
/*      */       {
/*      */       case 1: 
/*      */       case 2: 
/*      */       case 3: 
/*  998 */         return deserialize(jp, ctxt);
/*      */       }
/* 1000 */       return typeDeserializer.deserializeTypedFromScalar(jp, ctxt);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public static final class AtomicBooleanDeserializer
/*      */     extends StdScalarDeserializer<AtomicBoolean>
/*      */   {
/*      */     public AtomicBooleanDeserializer()
/*      */     {
/* 1016 */       super();
/*      */     }
/*      */     
/*      */ 
/*      */     public AtomicBoolean deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/* 1023 */       return new AtomicBoolean(_parseBooleanPrimitive(jp, ctxt));
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public static class AtomicReferenceDeserializer
/*      */     extends StdScalarDeserializer<AtomicReference<?>>
/*      */     implements ResolvableDeserializer
/*      */   {
/*      */     protected final JavaType _referencedType;
/*      */     
/*      */ 
/*      */     protected final BeanProperty _property;
/*      */     
/*      */ 
/*      */     protected JsonDeserializer<?> _valueDeserializer;
/*      */     
/*      */ 
/*      */ 
/*      */     public AtomicReferenceDeserializer(JavaType referencedType, BeanProperty property)
/*      */     {
/* 1045 */       super();
/* 1046 */       this._referencedType = referencedType;
/* 1047 */       this._property = property;
/*      */     }
/*      */     
/*      */ 
/*      */     public AtomicReference<?> deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/* 1054 */       return new AtomicReference(this._valueDeserializer.deserialize(jp, ctxt));
/*      */     }
/*      */     
/*      */ 
/*      */     public void resolve(DeserializationConfig config, DeserializerProvider provider)
/*      */       throws JsonMappingException
/*      */     {
/* 1061 */       this._valueDeserializer = provider.findValueDeserializer(config, this._referencedType, this._property);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @JacksonStdImpl
/*      */   public static class BigDecimalDeserializer
/*      */     extends StdScalarDeserializer<BigDecimal>
/*      */   {
/*      */     public BigDecimalDeserializer()
/*      */     {
/* 1076 */       super();
/*      */     }
/*      */     
/*      */     public BigDecimal deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/* 1082 */       JsonToken t = jp.getCurrentToken();
/* 1083 */       if ((t == JsonToken.VALUE_NUMBER_INT) || (t == JsonToken.VALUE_NUMBER_FLOAT)) {
/* 1084 */         return jp.getDecimalValue();
/*      */       }
/*      */       
/* 1087 */       if (t == JsonToken.VALUE_STRING) {
/* 1088 */         String text = jp.getText().trim();
/* 1089 */         if (text.length() == 0) {
/* 1090 */           return null;
/*      */         }
/*      */         try {
/* 1093 */           return new BigDecimal(text);
/*      */         } catch (IllegalArgumentException iae) {
/* 1095 */           throw ctxt.weirdStringException(this._valueClass, "not a valid representation");
/*      */         }
/*      */       }
/*      */       
/* 1099 */       throw ctxt.mappingException(this._valueClass);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   @JacksonStdImpl
/*      */   public static class BigIntegerDeserializer
/*      */     extends StdScalarDeserializer<BigInteger>
/*      */   {
/*      */     public BigIntegerDeserializer()
/*      */     {
/* 1111 */       super();
/*      */     }
/*      */     
/*      */     public BigInteger deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/* 1117 */       JsonToken t = jp.getCurrentToken();
/*      */       
/*      */ 
/* 1120 */       if (t == JsonToken.VALUE_NUMBER_INT) {
/* 1121 */         switch (StdDeserializer.1.$SwitchMap$org$codehaus$jackson$JsonParser$NumberType[jp.getNumberType().ordinal()]) {
/*      */         case 1: 
/*      */         case 2: 
/* 1124 */           return BigInteger.valueOf(jp.getLongValue()); }
/*      */       } else {
/* 1126 */         if (t == JsonToken.VALUE_NUMBER_FLOAT)
/*      */         {
/*      */ 
/*      */ 
/* 1130 */           return jp.getDecimalValue().toBigInteger(); }
/* 1131 */         if (t != JsonToken.VALUE_STRING)
/*      */         {
/* 1133 */           throw ctxt.mappingException(this._valueClass); }
/*      */       }
/* 1135 */       String text = jp.getText().trim();
/* 1136 */       if (text.length() == 0) {
/* 1137 */         return null;
/*      */       }
/*      */       try {
/* 1140 */         return new BigInteger(text);
/*      */       } catch (IllegalArgumentException iae) {
/* 1142 */         throw ctxt.weirdStringException(this._valueClass, "not a valid representation");
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @JacksonStdImpl
/*      */   public static class CalendarDeserializer
/*      */     extends StdScalarDeserializer<Calendar>
/*      */   {
/*      */     Class<? extends Calendar> _calendarClass;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1163 */     public CalendarDeserializer() { this(null); }
/*      */     
/* 1165 */     public CalendarDeserializer(Class<? extends Calendar> cc) { super();
/* 1166 */       this._calendarClass = cc;
/*      */     }
/*      */     
/*      */ 
/*      */     public Calendar deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/* 1173 */       java.util.Date d = _parseDate(jp, ctxt);
/* 1174 */       if (d == null) {
/* 1175 */         return null;
/*      */       }
/* 1177 */       if (this._calendarClass == null) {
/* 1178 */         return ctxt.constructCalendar(d);
/*      */       }
/*      */       try {
/* 1181 */         Calendar c = (Calendar)this._calendarClass.newInstance();
/* 1182 */         c.setTimeInMillis(d.getTime());
/* 1183 */         return c;
/*      */       } catch (Exception e) {
/* 1185 */         throw ctxt.instantiationException(this._calendarClass, e);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public static class SqlDateDeserializer
/*      */     extends StdScalarDeserializer<java.sql.Date>
/*      */   {
/*      */     public SqlDateDeserializer()
/*      */     {
/* 1197 */       super();
/*      */     }
/*      */     
/*      */     public java.sql.Date deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/* 1203 */       java.util.Date d = _parseDate(jp, ctxt);
/* 1204 */       return d == null ? null : new java.sql.Date(d.getTime());
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public static class StackTraceElementDeserializer
/*      */     extends StdScalarDeserializer<StackTraceElement>
/*      */   {
/*      */     public StackTraceElementDeserializer()
/*      */     {
/* 1217 */       super();
/*      */     }
/*      */     
/*      */     public StackTraceElement deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/* 1223 */       JsonToken t = jp.getCurrentToken();
/*      */       
/* 1225 */       if (t == JsonToken.START_OBJECT) {
/* 1226 */         String className = "";String methodName = "";String fileName = "";
/* 1227 */         int lineNumber = -1;
/*      */         
/* 1229 */         while ((t = jp.nextValue()) != JsonToken.END_OBJECT) {
/* 1230 */           String propName = jp.getCurrentName();
/* 1231 */           if ("className".equals(propName)) {
/* 1232 */             className = jp.getText();
/* 1233 */           } else if ("fileName".equals(propName)) {
/* 1234 */             fileName = jp.getText();
/* 1235 */           } else if ("lineNumber".equals(propName)) {
/* 1236 */             if (t.isNumeric()) {
/* 1237 */               lineNumber = jp.getIntValue();
/*      */             } else {
/* 1239 */               throw JsonMappingException.from(jp, "Non-numeric token (" + t + ") for property 'lineNumber'");
/*      */             }
/* 1241 */           } else if ("methodName".equals(propName)) {
/* 1242 */             methodName = jp.getText();
/* 1243 */           } else if (!"nativeMethod".equals(propName))
/*      */           {
/*      */ 
/* 1246 */             handleUnknownProperty(jp, ctxt, this._valueClass, propName);
/*      */           }
/*      */         }
/* 1249 */         return new StackTraceElement(className, methodName, fileName, lineNumber);
/*      */       }
/* 1251 */       throw ctxt.mappingException(this._valueClass);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @JacksonStdImpl
/*      */   public static class TokenBufferDeserializer
/*      */     extends StdScalarDeserializer<TokenBuffer>
/*      */   {
/*      */     public TokenBufferDeserializer()
/*      */     {
/* 1269 */       super();
/*      */     }
/*      */     
/*      */     public TokenBuffer deserialize(JsonParser jp, DeserializationContext ctxt)
/*      */       throws IOException, JsonProcessingException
/*      */     {
/* 1275 */       TokenBuffer tb = new TokenBuffer(jp.getCodec());
/*      */       
/* 1277 */       tb.copyCurrentStructure(jp);
/* 1278 */       return tb;
/*      */     }
/*      */   }
/*      */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\deser\StdDeserializer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */