/*     */ package org.codehaus.jackson.map.ser;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.lang.reflect.Type;
/*     */ import java.text.DateFormat;
/*     */ import java.util.Date;
/*     */ import org.codehaus.jackson.JsonGenerationException;
/*     */ import org.codehaus.jackson.JsonGenerator;
/*     */ import org.codehaus.jackson.JsonNode;
/*     */ import org.codehaus.jackson.JsonProcessingException;
/*     */ import org.codehaus.jackson.map.BeanProperty;
/*     */ import org.codehaus.jackson.map.ContextualSerializer;
/*     */ import org.codehaus.jackson.map.JsonMappingException;
/*     */ import org.codehaus.jackson.map.JsonSerializer;
/*     */ import org.codehaus.jackson.map.ResolvableSerializer;
/*     */ import org.codehaus.jackson.map.SerializationConfig;
/*     */ import org.codehaus.jackson.map.SerializationConfig.Feature;
/*     */ import org.codehaus.jackson.map.SerializerFactory;
/*     */ import org.codehaus.jackson.map.SerializerProvider;
/*     */ import org.codehaus.jackson.map.TypeSerializer;
/*     */ import org.codehaus.jackson.map.ser.impl.ReadOnlyClassToSerializerMap;
/*     */ import org.codehaus.jackson.map.util.ClassUtil;
/*     */ import org.codehaus.jackson.map.util.RootNameLookup;
/*     */ import org.codehaus.jackson.node.ObjectNode;
/*     */ import org.codehaus.jackson.schema.JsonSchema;
/*     */ import org.codehaus.jackson.schema.SchemaAware;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class StdSerializerProvider
/*     */   extends SerializerProvider
/*     */ {
/*     */   static final boolean CACHE_UNKNOWN_MAPPINGS = false;
/*  49 */   public static final JsonSerializer<Object> DEFAULT_NULL_KEY_SERIALIZER = new FailingSerializer("Null key for a Map not allowed in JSON (use a converting NullKeySerializer?)");
/*     */   
/*     */ 
/*  52 */   public static final JsonSerializer<Object> DEFAULT_KEY_SERIALIZER = new StdKeySerializer();
/*     */   
/*  54 */   public static final JsonSerializer<Object> DEFAULT_UNKNOWN_SERIALIZER = new SerializerBase(Object.class)
/*     */   {
/*     */ 
/*     */     public void serialize(Object value, JsonGenerator jgen, SerializerProvider provider)
/*     */       throws IOException, JsonMappingException
/*     */     {
/*     */ 
/*  61 */       if (provider.isEnabled(SerializationConfig.Feature.FAIL_ON_EMPTY_BEANS)) {
/*  62 */         failForEmpty(value);
/*     */       }
/*     */       
/*  65 */       jgen.writeStartObject();
/*  66 */       jgen.writeEndObject();
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     public final void serializeWithType(Object value, JsonGenerator jgen, SerializerProvider provider, TypeSerializer typeSer)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/*  75 */       if (provider.isEnabled(SerializationConfig.Feature.FAIL_ON_EMPTY_BEANS)) {
/*  76 */         failForEmpty(value);
/*     */       }
/*  78 */       typeSer.writeTypePrefixForObject(value, jgen);
/*  79 */       typeSer.writeTypeSuffixForObject(value, jgen);
/*     */     }
/*     */     
/*     */     public JsonNode getSchema(SerializerProvider provider, Type typeHint) throws JsonMappingException
/*     */     {
/*  84 */       return null;
/*     */     }
/*     */     
/*     */     protected void failForEmpty(Object value) throws JsonMappingException
/*     */     {
/*  89 */       throw new JsonMappingException("No serializer found for class " + value.getClass().getName() + " and no properties discovered to create BeanSerializer (to avoid exception, disable SerializationConfig.Feature.FAIL_ON_EMPTY_BEANS) )");
/*     */     }
/*     */   };
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final SerializerFactory _serializerFactory;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final SerializerCache _serializerCache;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final RootNameLookup _rootNames;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 122 */   protected JsonSerializer<Object> _unknownTypeSerializer = DEFAULT_UNKNOWN_SERIALIZER;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 128 */   protected JsonSerializer<Object> _keySerializer = DEFAULT_KEY_SERIALIZER;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 134 */   protected JsonSerializer<Object> _nullValueSerializer = NullSerializer.instance;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 143 */   protected JsonSerializer<Object> _nullKeySerializer = DEFAULT_NULL_KEY_SERIALIZER;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final ReadOnlyClassToSerializerMap _knownSerializers;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected DateFormat _dateFormat;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public StdSerializerProvider()
/*     */   {
/* 177 */     super(null);
/* 178 */     this._serializerFactory = null;
/* 179 */     this._serializerCache = new SerializerCache();
/*     */     
/* 181 */     this._knownSerializers = null;
/* 182 */     this._rootNames = new RootNameLookup();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected StdSerializerProvider(SerializationConfig config, StdSerializerProvider src, SerializerFactory f)
/*     */   {
/* 194 */     super(config);
/* 195 */     if (config == null) {
/* 196 */       throw new NullPointerException();
/*     */     }
/* 198 */     this._serializerFactory = f;
/*     */     
/* 200 */     this._serializerCache = src._serializerCache;
/* 201 */     this._unknownTypeSerializer = src._unknownTypeSerializer;
/* 202 */     this._keySerializer = src._keySerializer;
/* 203 */     this._nullValueSerializer = src._nullValueSerializer;
/* 204 */     this._nullKeySerializer = src._nullKeySerializer;
/* 205 */     this._rootNames = src._rootNames;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 210 */     this._knownSerializers = this._serializerCache.getReadOnlyLookupMap();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected StdSerializerProvider createInstance(SerializationConfig config, SerializerFactory jsf)
/*     */   {
/* 219 */     return new StdSerializerProvider(config, this, jsf);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void setDefaultKeySerializer(JsonSerializer<Object> ks)
/*     */   {
/* 231 */     if (ks == null) {
/* 232 */       throw new IllegalArgumentException("Can not pass null JsonSerializer");
/*     */     }
/* 234 */     this._keySerializer = ks;
/*     */   }
/*     */   
/*     */ 
/*     */   public void setNullValueSerializer(JsonSerializer<Object> nvs)
/*     */   {
/* 240 */     if (nvs == null) {
/* 241 */       throw new IllegalArgumentException("Can not pass null JsonSerializer");
/*     */     }
/* 243 */     this._nullValueSerializer = nvs;
/*     */   }
/*     */   
/*     */ 
/*     */   public void setNullKeySerializer(JsonSerializer<Object> nks)
/*     */   {
/* 249 */     if (nks == null) {
/* 250 */       throw new IllegalArgumentException("Can not pass null JsonSerializer");
/*     */     }
/* 252 */     this._nullKeySerializer = nks;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final void serializeValue(SerializationConfig config, JsonGenerator jgen, Object value, SerializerFactory jsf)
/*     */     throws IOException, JsonGenerationException
/*     */   {
/* 266 */     if (jsf == null) {
/* 267 */       throw new IllegalArgumentException("Can not pass null serializerFactory");
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 274 */     StdSerializerProvider inst = createInstance(config, jsf);
/*     */     
/* 276 */     if (inst.getClass() != getClass()) {
/* 277 */       throw new IllegalStateException("Broken serializer provider: createInstance returned instance of type " + inst.getClass() + "; blueprint of type " + getClass());
/*     */     }
/*     */     
/* 280 */     inst._serializeValue(jgen, value);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public final void serializeValue(SerializationConfig config, JsonGenerator jgen, Object value, JavaType rootType, SerializerFactory jsf)
/*     */     throws IOException, JsonGenerationException
/*     */   {
/* 288 */     if (jsf == null) {
/* 289 */       throw new IllegalArgumentException("Can not pass null serializerFactory");
/*     */     }
/* 291 */     StdSerializerProvider inst = createInstance(config, jsf);
/* 292 */     if (inst.getClass() != getClass()) {
/* 293 */       throw new IllegalStateException("Broken serializer provider: createInstance returned instance of type " + inst.getClass() + "; blueprint of type " + getClass());
/*     */     }
/* 295 */     inst._serializeValue(jgen, value, rootType);
/*     */   }
/*     */   
/*     */ 
/*     */   public JsonSchema generateJsonSchema(Class<?> type, SerializationConfig config, SerializerFactory jsf)
/*     */     throws JsonMappingException
/*     */   {
/* 302 */     if (type == null) {
/* 303 */       throw new IllegalArgumentException("A class must be provided");
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 310 */     StdSerializerProvider inst = createInstance(config, jsf);
/*     */     
/* 312 */     if (inst.getClass() != getClass()) {
/* 313 */       throw new IllegalStateException("Broken serializer provider: createInstance returned instance of type " + inst.getClass() + "; blueprint of type " + getClass());
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 318 */     JsonSerializer<Object> ser = inst.findValueSerializer(type, null);
/* 319 */     JsonNode schemaNode = (ser instanceof SchemaAware) ? ((SchemaAware)ser).getSchema(inst, null) : JsonSchema.getDefaultSchemaNode();
/*     */     
/*     */ 
/* 322 */     if (!(schemaNode instanceof ObjectNode)) {
/* 323 */       throw new IllegalArgumentException("Class " + type.getName() + " would not be serialized as a JSON object and therefore has no schema");
/*     */     }
/*     */     
/*     */ 
/* 327 */     return new JsonSchema((ObjectNode)schemaNode);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public boolean hasSerializerFor(SerializationConfig config, Class<?> cls, SerializerFactory jsf)
/*     */   {
/* 334 */     return createInstance(config, jsf)._findExplicitUntypedSerializer(cls, null) != null;
/*     */   }
/*     */   
/*     */   public int cachedSerializersCount()
/*     */   {
/* 339 */     return this._serializerCache.size();
/*     */   }
/*     */   
/*     */   public void flushCachedSerializers()
/*     */   {
/* 344 */     this._serializerCache.flush();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonSerializer<Object> findValueSerializer(Class<?> valueType, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 360 */     JsonSerializer<Object> ser = this._knownSerializers.untypedValueSerializer(valueType);
/* 361 */     if (ser == null)
/*     */     {
/* 363 */       ser = this._serializerCache.untypedValueSerializer(valueType);
/* 364 */       if (ser == null)
/*     */       {
/* 366 */         ser = this._serializerCache.untypedValueSerializer(this._config.constructType(valueType));
/* 367 */         if (ser == null)
/*     */         {
/* 369 */           ser = _createAndCacheUntypedSerializer(valueType, property);
/*     */           
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 375 */           if (ser == null) {
/* 376 */             ser = getUnknownTypeSerializer(valueType);
/*     */             
/*     */ 
/*     */ 
/*     */ 
/* 381 */             return ser;
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/* 386 */     return _handleContextualResolvable(ser, property);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonSerializer<Object> findValueSerializer(JavaType valueType, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 400 */     JsonSerializer<Object> ser = this._knownSerializers.untypedValueSerializer(valueType);
/* 401 */     if (ser == null)
/*     */     {
/* 403 */       ser = this._serializerCache.untypedValueSerializer(valueType);
/* 404 */       if (ser == null)
/*     */       {
/* 406 */         ser = _createAndCacheUntypedSerializer(valueType, property);
/*     */         
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 412 */         if (ser == null) {
/* 413 */           ser = getUnknownTypeSerializer(valueType.getRawClass());
/*     */           
/*     */ 
/*     */ 
/*     */ 
/* 418 */           return ser;
/*     */         }
/*     */       }
/*     */     }
/* 422 */     return _handleContextualResolvable(ser, property);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonSerializer<Object> findTypedValueSerializer(Class<?> valueType, boolean cache, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 435 */     JsonSerializer<Object> ser = this._knownSerializers.typedValueSerializer(valueType);
/* 436 */     if (ser != null) {
/* 437 */       return ser;
/*     */     }
/*     */     
/* 440 */     ser = this._serializerCache.typedValueSerializer(valueType);
/* 441 */     if (ser != null) {
/* 442 */       return ser;
/*     */     }
/*     */     
/*     */ 
/* 446 */     ser = findValueSerializer(valueType, property);
/* 447 */     TypeSerializer typeSer = this._serializerFactory.createTypeSerializer(this._config, this._config.constructType(valueType), property);
/*     */     
/* 449 */     if (typeSer != null) {
/* 450 */       ser = new WrappedSerializer(typeSer, ser);
/*     */     }
/* 452 */     if (cache) {
/* 453 */       this._serializerCache.addTypedSerializer(valueType, ser);
/*     */     }
/* 455 */     return ser;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonSerializer<Object> findTypedValueSerializer(JavaType valueType, boolean cache, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 464 */     JsonSerializer<Object> ser = this._knownSerializers.typedValueSerializer(valueType);
/* 465 */     if (ser != null) {
/* 466 */       return ser;
/*     */     }
/*     */     
/* 469 */     ser = this._serializerCache.typedValueSerializer(valueType);
/* 470 */     if (ser != null) {
/* 471 */       return ser;
/*     */     }
/*     */     
/*     */ 
/* 475 */     ser = findValueSerializer(valueType, property);
/* 476 */     TypeSerializer typeSer = this._serializerFactory.createTypeSerializer(this._config, valueType, property);
/* 477 */     if (typeSer != null) {
/* 478 */       ser = new WrappedSerializer(typeSer, ser);
/*     */     }
/* 480 */     if (cache) {
/* 481 */       this._serializerCache.addTypedSerializer(valueType, ser);
/*     */     }
/* 483 */     return ser;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonSerializer<Object> findKeySerializer(JavaType keyType, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 497 */     JsonSerializer<Object> ser = this._serializerFactory.createKeySerializer(this._config, keyType, property);
/*     */     
/*     */ 
/*     */ 
/* 501 */     if (ser == null) {
/* 502 */       ser = this._keySerializer;
/*     */     }
/*     */     
/* 505 */     if ((ser instanceof ContextualSerializer)) {
/* 506 */       ContextualSerializer<?> contextual = (ContextualSerializer)ser;
/* 507 */       ser = contextual.createContextual(this._config, property);
/*     */     }
/* 509 */     return ser;
/*     */   }
/*     */   
/*     */   public JsonSerializer<Object> getNullKeySerializer()
/*     */   {
/* 514 */     return this._nullKeySerializer;
/*     */   }
/*     */   
/*     */   public JsonSerializer<Object> getNullValueSerializer()
/*     */   {
/* 519 */     return this._nullValueSerializer;
/*     */   }
/*     */   
/*     */   public JsonSerializer<Object> getUnknownTypeSerializer(Class<?> unknownType)
/*     */   {
/* 524 */     return this._unknownTypeSerializer;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final void defaultSerializeDateValue(long timestamp, JsonGenerator jgen)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 541 */     if (isEnabled(SerializationConfig.Feature.WRITE_DATES_AS_TIMESTAMPS)) {
/* 542 */       jgen.writeNumber(timestamp);
/*     */     } else {
/* 544 */       if (this._dateFormat == null)
/*     */       {
/* 546 */         this._dateFormat = ((DateFormat)this._config.getDateFormat().clone());
/*     */       }
/* 548 */       jgen.writeString(this._dateFormat.format(new Date(timestamp)));
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public final void defaultSerializeDateValue(Date date, JsonGenerator jgen)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 557 */     if (isEnabled(SerializationConfig.Feature.WRITE_DATES_AS_TIMESTAMPS)) {
/* 558 */       jgen.writeNumber(date.getTime());
/*     */     } else {
/* 560 */       if (this._dateFormat == null) {
/* 561 */         DateFormat blueprint = this._config.getDateFormat();
/*     */         
/* 563 */         this._dateFormat = ((DateFormat)blueprint.clone());
/*     */       }
/* 565 */       jgen.writeString(this._dateFormat.format(date));
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void _serializeValue(JsonGenerator jgen, Object value)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/*     */     boolean wrap;
/*     */     
/*     */ 
/*     */ 
/*     */     JsonSerializer<Object> ser;
/*     */     
/*     */ 
/*     */     boolean wrap;
/*     */     
/*     */ 
/* 585 */     if (value == null) {
/* 586 */       JsonSerializer<Object> ser = getNullValueSerializer();
/* 587 */       wrap = false;
/*     */     } else {
/* 589 */       Class<?> cls = value.getClass();
/*     */       
/* 591 */       ser = findTypedValueSerializer(cls, true, null);
/*     */       
/* 593 */       wrap = this._config.isEnabled(SerializationConfig.Feature.WRAP_ROOT_VALUE);
/* 594 */       if (wrap) {
/* 595 */         jgen.writeStartObject();
/* 596 */         jgen.writeFieldName(this._rootNames.findRootName(value.getClass(), this._config));
/*     */       }
/*     */     }
/*     */     try {
/* 600 */       ser.serialize(value, jgen, this);
/* 601 */       if (wrap) {
/* 602 */         jgen.writeEndObject();
/*     */       }
/*     */       
/*     */     }
/*     */     catch (IOException ioe)
/*     */     {
/* 608 */       throw ioe;
/*     */     }
/*     */     catch (Exception e) {
/* 611 */       String msg = e.getMessage();
/* 612 */       if (msg == null) {
/* 613 */         msg = "[no message for " + e.getClass().getName() + "]";
/*     */       }
/* 615 */       throw new JsonMappingException(msg, e);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected void _serializeValue(JsonGenerator jgen, Object value, JavaType rootType)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/*     */     boolean wrap;
/*     */     
/*     */ 
/*     */     JsonSerializer<Object> ser;
/*     */     
/*     */     boolean wrap;
/*     */     
/* 631 */     if (value == null) {
/* 632 */       JsonSerializer<Object> ser = getNullValueSerializer();
/* 633 */       wrap = false;
/*     */     }
/*     */     else {
/* 636 */       if (!rootType.getRawClass().isAssignableFrom(value.getClass())) {
/* 637 */         _reportIncompatibleRootType(value, rootType);
/*     */       }
/*     */       
/* 640 */       ser = findTypedValueSerializer(rootType, true, null);
/*     */       
/* 642 */       wrap = this._config.isEnabled(SerializationConfig.Feature.WRAP_ROOT_VALUE);
/* 643 */       if (wrap) {
/* 644 */         jgen.writeStartObject();
/* 645 */         jgen.writeFieldName(this._rootNames.findRootName(rootType, this._config));
/*     */       }
/*     */     }
/*     */     try {
/* 649 */       ser.serialize(value, jgen, this);
/* 650 */       if (wrap) {
/* 651 */         jgen.writeEndObject();
/*     */       }
/*     */     } catch (IOException ioe) {
/* 654 */       throw ioe;
/*     */     } catch (Exception e) {
/* 656 */       String msg = e.getMessage();
/* 657 */       if (msg == null) {
/* 658 */         msg = "[no message for " + e.getClass().getName() + "]";
/*     */       }
/* 660 */       throw new JsonMappingException(msg, e);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void _reportIncompatibleRootType(Object value, JavaType rootType)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 670 */     if (rootType.isPrimitive()) {
/* 671 */       Class<?> wrapperType = ClassUtil.wrapperType(rootType.getRawClass());
/*     */       
/* 673 */       if (wrapperType.isAssignableFrom(value.getClass())) {
/* 674 */         return;
/*     */       }
/*     */     }
/* 677 */     throw new JsonMappingException("Incompatible types: declared root type (" + rootType + ") vs " + value.getClass().getName());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonSerializer<Object> _findExplicitUntypedSerializer(Class<?> runtimeType, BeanProperty property)
/*     */   {
/* 692 */     JsonSerializer<Object> ser = this._knownSerializers.untypedValueSerializer(runtimeType);
/* 693 */     if (ser != null) {
/* 694 */       return ser;
/*     */     }
/*     */     
/* 697 */     ser = this._serializerCache.untypedValueSerializer(runtimeType);
/* 698 */     if (ser != null) {
/* 699 */       return ser;
/*     */     }
/*     */     try {
/* 702 */       return _createAndCacheUntypedSerializer(runtimeType, property);
/*     */     } catch (Exception e) {}
/* 704 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonSerializer<Object> _createAndCacheUntypedSerializer(Class<?> type, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/*     */     JsonSerializer<Object> ser;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     try
/*     */     {
/* 725 */       ser = _createUntypedSerializer(this._config.constructType(type), property);
/*     */ 
/*     */     }
/*     */     catch (IllegalArgumentException iae)
/*     */     {
/* 730 */       throw new JsonMappingException(iae.getMessage(), null, iae);
/*     */     }
/*     */     
/* 733 */     if (ser != null) {
/* 734 */       this._serializerCache.addAndResolveNonTypedSerializer(type, ser, this);
/*     */     }
/* 736 */     return ser;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected JsonSerializer<Object> _createAndCacheUntypedSerializer(JavaType type, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/*     */     JsonSerializer<Object> ser;
/*     */     
/*     */     try
/*     */     {
/* 748 */       ser = _createUntypedSerializer(type, property);
/*     */ 
/*     */     }
/*     */     catch (IllegalArgumentException iae)
/*     */     {
/* 753 */       throw new JsonMappingException(iae.getMessage(), null, iae);
/*     */     }
/*     */     
/* 756 */     if (ser != null) {
/* 757 */       this._serializerCache.addAndResolveNonTypedSerializer(type, ser, this);
/*     */     }
/* 759 */     return ser;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonSerializer<Object> _createUntypedSerializer(JavaType type, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 772 */     return this._serializerFactory.createSerializer(this._config, type, property);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonSerializer<Object> _handleContextualResolvable(JsonSerializer<Object> ser, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 783 */     if (!(ser instanceof ContextualSerializer)) {
/* 784 */       return ser;
/*     */     }
/* 786 */     JsonSerializer<Object> ctxtSer = ((ContextualSerializer)ser).createContextual(this._config, property);
/* 787 */     if (ctxtSer != ser)
/*     */     {
/* 789 */       if ((ctxtSer instanceof ResolvableSerializer)) {
/* 790 */         ((ResolvableSerializer)ctxtSer).resolve(this);
/*     */       }
/* 792 */       ser = ctxtSer;
/*     */     }
/* 794 */     return ser;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private static final class WrappedSerializer
/*     */     extends JsonSerializer<Object>
/*     */   {
/*     */     protected final TypeSerializer _typeSerializer;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final JsonSerializer<Object> _serializer;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public WrappedSerializer(TypeSerializer typeSer, JsonSerializer<Object> ser)
/*     */     {
/* 817 */       this._typeSerializer = typeSer;
/* 818 */       this._serializer = ser;
/*     */     }
/*     */     
/*     */ 
/*     */     public void serialize(Object value, JsonGenerator jgen, SerializerProvider provider)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 825 */       this._serializer.serializeWithType(value, jgen, provider, this._typeSerializer);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public void serializeWithType(Object value, JsonGenerator jgen, SerializerProvider provider, TypeSerializer typeSer)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 836 */       this._serializer.serializeWithType(value, jgen, provider, typeSer);
/*     */     }
/*     */     
/*     */     public Class<Object> handledType() {
/* 840 */       return Object.class;
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\ser\StdSerializerProvider.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */