/*     */ package org.codehaus.janino;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.util.Iterator;
/*     */ import java.util.Set;
/*     */ import org.codehaus.commons.compiler.CompileException;
/*     */ import org.codehaus.commons.compiler.Location;
/*     */ import org.codehaus.janino.util.ClassFile;
/*     */ import org.codehaus.janino.util.resource.Resource;
/*     */ import org.codehaus.janino.util.resource.ResourceFinder;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public final class JavaSourceIClassLoader
/*     */   extends IClassLoader
/*     */ {
/*     */   private static final boolean DEBUG = false;
/*     */   private ResourceFinder sourceFinder;
/*     */   private String optionalCharacterEncoding;
/*     */   private final Set unitCompilers;
/*  49 */   private UnitCompiler.ErrorHandler optionalCompileErrorHandler = null;
/*  50 */   private WarningHandler optionalWarningHandler = null;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JavaSourceIClassLoader(ResourceFinder sourceFinder, String optionalCharacterEncoding, Set unitCompilers, IClassLoader optionalParentIClassLoader)
/*     */   {
/*  65 */     super(optionalParentIClassLoader);
/*     */     
/*  67 */     this.sourceFinder = sourceFinder;
/*  68 */     this.optionalCharacterEncoding = optionalCharacterEncoding;
/*  69 */     this.unitCompilers = unitCompilers;
/*  70 */     super.postConstruct();
/*     */   }
/*     */   
/*     */   public void setSourceFinder(ResourceFinder pathResourceFinder) {
/*  74 */     this.sourceFinder = pathResourceFinder;
/*     */   }
/*     */   
/*     */   public void setCharacterEncoding(String optionalCharacterEncoding) {
/*  78 */     this.optionalCharacterEncoding = optionalCharacterEncoding;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public void setCompileErrorHandler(UnitCompiler.ErrorHandler optionalCompileErrorHandler)
/*     */   {
/*  85 */     this.optionalCompileErrorHandler = optionalCompileErrorHandler;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public void setWarningHandler(WarningHandler optionalWarningHandler)
/*     */   {
/*  93 */     this.optionalWarningHandler = optionalWarningHandler;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public IClass findIClass(String type)
/*     */     throws ClassNotFoundException
/*     */   {
/* 104 */     String className = Descriptor.toClassName(type);
/*     */     
/*     */ 
/*     */ 
/* 108 */     if (className.startsWith("java.")) { return null;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 113 */     int idx = className.indexOf('$');
/* 114 */     String topLevelClassName = idx == -1 ? className : className.substring(0, idx);
/*     */     
/*     */ 
/*     */ 
/* 118 */     for (Iterator it = this.unitCompilers.iterator(); it.hasNext();) {
/* 119 */       UnitCompiler uc = (UnitCompiler)it.next();
/* 120 */       IClass res = uc.findClass(topLevelClassName);
/* 121 */       if (res != null) {
/* 122 */         if (!className.equals(topLevelClassName)) {
/* 123 */           res = uc.findClass(className);
/* 124 */           if (res == null) return null;
/*     */         }
/* 126 */         defineIClass(res);
/* 127 */         return res;
/*     */       }
/*     */     }
/*     */     
/*     */ 
/* 132 */     Resource sourceResource = this.sourceFinder.findResource(ClassFile.getSourceResourceName(className));
/* 133 */     if (sourceResource == null) { return null;
/*     */     }
/*     */     
/*     */ 
/*     */     try
/*     */     {
/* 139 */       InputStream inputStream = sourceResource.open();
/*     */       try
/*     */       {
/* 142 */         Scanner scanner = new Scanner(sourceResource.getFileName(), inputStream, this.optionalCharacterEncoding);
/*     */         
/*     */ 
/*     */ 
/*     */ 
/* 147 */         scanner.setWarningHandler(this.optionalWarningHandler);
/* 148 */         Parser parser = new Parser(scanner);
/* 149 */         parser.setWarningHandler(this.optionalWarningHandler);
/* 150 */         Java.CompilationUnit cu = parser.parseCompilationUnit();
/*     */         try {
/* 152 */           inputStream.close();
/*     */         } catch (IOException ex) {}
/* 154 */         uc = new UnitCompiler(cu, this);
/*     */       }
/*     */       finally
/*     */       {
/*     */         try
/*     */         {
/* 152 */           inputStream.close();
/*     */         } catch (IOException ex) {} }
/*     */       UnitCompiler uc;
/* 155 */       uc.setCompileErrorHandler(this.optionalCompileErrorHandler);
/* 156 */       uc.setWarningHandler(this.optionalWarningHandler);
/*     */       
/*     */ 
/* 159 */       this.unitCompilers.add(uc);
/*     */       
/*     */ 
/* 162 */       IClass res = uc.findClass(className);
/* 163 */       if (res == null) {
/* 164 */         if (className.equals(topLevelClassName)) {
/* 165 */           throw new CompileException("Source file \"" + sourceResource.getFileName() + "\" does not declare class \"" + className + "\"", (Location)null);
/*     */         }
/*     */         
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 173 */         return null;
/*     */       }
/* 175 */       defineIClass(res);
/* 176 */       return res;
/*     */     } catch (IOException e) {
/* 178 */       throw new ClassNotFoundException("Parsing compilation unit \"" + sourceResource + "\"", e);
/*     */     } catch (CompileException e) {
/* 180 */       throw new ClassNotFoundException("Parsing compilation unit \"" + sourceResource + "\"", e);
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\janino\JavaSourceIClassLoader.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       0.7.1
 */