package prefuse.action.layout;

import java.awt.geom.Rectangle2D;
import java.util.Arrays;
import java.util.Iterator;
import prefuse.Visualization;
import prefuse.data.Table;
import prefuse.data.query.NumberRangeModel;
import prefuse.util.ArrayLib;
import prefuse.util.MathLib;
import prefuse.util.PrefuseLib;
import prefuse.util.ui.ValuedRangeModel;
import prefuse.visual.VisualItem;

public class StackedAreaChart
  extends Layout
{
  private String m_field;
  private String m_start;
  private String m_end;
  private String[] columns;
  private double[] baseline;
  private float[] poly;
  private double m_padding = 0.05D;
  private float m_threshold;
  private Rectangle2D bounds;
  private NumberRangeModel m_model;
  
  public StackedAreaChart(String paramString1, String paramString2, String[] paramArrayOfString)
  {
    this(paramString1, paramString2, paramArrayOfString, 1.0D);
  }
  
  public StackedAreaChart(String paramString1, String paramString2, String[] paramArrayOfString, double paramDouble)
  {
    super(paramString1);
    this.columns = paramArrayOfString;
    this.baseline = new double[paramArrayOfString.length];
    this.poly = new float[4 * paramArrayOfString.length];
    this.m_field = paramString2;
    this.m_start = PrefuseLib.getStartField(paramString2);
    this.m_end = PrefuseLib.getEndField(paramString2);
    setThreshold(paramDouble);
    this.m_model = new NumberRangeModel(0, 1, 0, 1);
  }
  
  public double getPaddingPercentage()
  {
    return this.m_padding;
  }
  
  public void setPaddingPercentage(double paramDouble)
  {
    if ((paramDouble < 0.0D) || (paramDouble > 1.0D)) {
      throw new IllegalArgumentException("Illegal padding percentage: " + paramDouble);
    }
    this.m_padding = paramDouble;
  }
  
  public double getThreshold()
  {
    return this.m_threshold;
  }
  
  public void setThreshold(double paramDouble)
  {
    this.m_threshold = ((float)paramDouble);
  }
  
  public ValuedRangeModel getRangeModel()
  {
    return this.m_model;
  }
  
  public void run(double paramDouble)
  {
    Arrays.fill(this.baseline, 0.0D);
    this.bounds = getLayoutBounds();
    float f1 = (float)(this.bounds.getMaxX() - this.bounds.getMinX()) / (this.columns.length - 1);
    int i = this.columns.length;
    Iterator localIterator = this.m_vis.visibleItems(this.m_group);
    while (localIterator.hasNext())
    {
      VisualItem localVisualItem1 = (VisualItem)localIterator.next();
      for (int j = 0; j < this.columns.length; j++) {
        this.baseline[j] += localVisualItem1.getDouble(this.columns[j]);
      }
    }
    double d1 = ArrayLib.max(this.baseline);
    d1 += this.m_padding * d1;
    Arrays.fill(this.baseline, this.bounds.getMaxY());
    this.m_model.setValueRange(0.0D, d1, 0.0D, d1);
    Table localTable = (Table)this.m_vis.getGroup(this.m_group);
    localIterator = localTable.tuplesReversed();
    while (localIterator.hasNext())
    {
      VisualItem localVisualItem2 = (VisualItem)localIterator.next();
      if (localVisualItem2.isVisible())
      {
        float f2 = 0.0F;
        int k = i;
        for (;;)
        {
          k--;
          if (k < 0) {
            break;
          }
          this.poly[(2 * (i - 1 - k))] = ((float)this.bounds.getMinX() + k * f1);
          this.poly[(2 * (i - 1 - k) + 1)] = ((float)this.baseline[k]);
        }
        for (k = 0; k < this.columns.length; k++)
        {
          int m = 2 * (i + k);
          double d2 = localVisualItem2.getDouble(this.columns[k]);
          this.baseline[k] -= (float)this.bounds.getHeight() * MathLib.linearInterp(d2, 0.0D, d1);
          this.poly[m] = ((float)this.bounds.getMinX() + k * f1);
          this.poly[(m + 1)] = ((float)this.baseline[k]);
          f2 = Math.max(f2, this.poly[(2 * (i - 1 - k) + 1)] - this.poly[(m + 1)]);
        }
        if (f2 < this.m_threshold) {
          localVisualItem2.setVisible(false);
        }
        setX(localVisualItem2, null, 0.0D);
        setY(localVisualItem2, null, 0.0D);
        setPolygon(localVisualItem2, this.poly);
      }
    }
  }
  
  private void setPolygon(VisualItem paramVisualItem, float[] paramArrayOfFloat)
  {
    float[] arrayOfFloat1 = getPolygon(paramVisualItem, this.m_field);
    float[] arrayOfFloat2 = getPolygon(paramVisualItem, this.m_start);
    float[] arrayOfFloat3 = getPolygon(paramVisualItem, this.m_end);
    System.arraycopy(arrayOfFloat1, 0, arrayOfFloat2, 0, arrayOfFloat1.length);
    System.arraycopy(paramArrayOfFloat, 0, arrayOfFloat1, 0, paramArrayOfFloat.length);
    System.arraycopy(paramArrayOfFloat, 0, arrayOfFloat3, 0, paramArrayOfFloat.length);
    paramVisualItem.setValidated(false);
  }
  
  private float[] getPolygon(VisualItem paramVisualItem, String paramString)
  {
    float[] arrayOfFloat = (float[])paramVisualItem.get(paramString);
    if ((arrayOfFloat == null) || (arrayOfFloat.length < 4 * this.columns.length))
    {
      int i = this.columns.length;
      float f1 = (float)(this.bounds.getMaxX() - this.bounds.getMinX()) / (i - 1);
      arrayOfFloat = new float[4 * i];
      Arrays.fill(arrayOfFloat, (float)this.bounds.getMaxY());
      for (int j = 0; j < i; j++)
      {
        float f2 = j * f1 + (float)this.bounds.getMinX();
        arrayOfFloat[(2 * (i + j))] = f2;
        arrayOfFloat[(2 * (i - 1 - j))] = f2;
      }
      paramVisualItem.set(paramString, arrayOfFloat);
    }
    return arrayOfFloat;
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\action\layout\StackedAreaChart.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */