package prefuse.action.layout.graph;

import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Arrays;
import prefuse.Display;
import prefuse.Visualization;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Schema;
import prefuse.data.tuple.TupleSet;
import prefuse.util.ArrayLib;
import prefuse.visual.NodeItem;

public class NodeLinkTreeLayout
  extends TreeLayout
{
  private int m_orientation;
  private double m_bspace = 5.0D;
  private double m_tspace = 25.0D;
  private double m_dspace = 50.0D;
  private double m_offset = 50.0D;
  private double[] m_depths = new double[10];
  private int m_maxDepth = 0;
  private double m_ax;
  private double m_ay;
  public static final String PARAMS = "_reingoldTilfordParams";
  public static final Schema PARAMS_SCHEMA = new Schema();
  
  public NodeLinkTreeLayout(String paramString)
  {
    super(paramString);
    this.m_orientation = 0;
  }
  
  public NodeLinkTreeLayout(String paramString, int paramInt, double paramDouble1, double paramDouble2, double paramDouble3)
  {
    super(paramString);
    this.m_orientation = paramInt;
    this.m_dspace = paramDouble1;
    this.m_bspace = paramDouble2;
    this.m_tspace = paramDouble3;
  }
  
  public void setOrientation(int paramInt)
  {
    if ((paramInt < 0) || (paramInt >= 5) || (paramInt == 4)) {
      throw new IllegalArgumentException("Unsupported orientation value: " + paramInt);
    }
    this.m_orientation = paramInt;
  }
  
  public int getOrientation()
  {
    return this.m_orientation;
  }
  
  public void setDepthSpacing(double paramDouble)
  {
    this.m_dspace = paramDouble;
  }
  
  public double getDepthSpacing()
  {
    return this.m_dspace;
  }
  
  public void setBreadthSpacing(double paramDouble)
  {
    this.m_bspace = paramDouble;
  }
  
  public double getBreadthSpacing()
  {
    return this.m_bspace;
  }
  
  public void setSubtreeSpacing(double paramDouble)
  {
    this.m_tspace = paramDouble;
  }
  
  public double getSubtreeSpacing()
  {
    return this.m_tspace;
  }
  
  public void setRootNodeOffset(double paramDouble)
  {
    this.m_offset = paramDouble;
  }
  
  public double getRootNodeOffset()
  {
    return this.m_offset;
  }
  
  public Point2D getLayoutAnchor()
  {
    if (this.m_anchor != null) {
      return this.m_anchor;
    }
    this.m_tmpa.setLocation(0.0D, 0.0D);
    if (this.m_vis != null)
    {
      Display localDisplay = this.m_vis.getDisplay(0);
      Rectangle2D localRectangle2D = getLayoutBounds();
      switch (this.m_orientation)
      {
      case 0: 
        this.m_tmpa.setLocation(this.m_offset, localDisplay.getHeight() / 2.0D);
        break;
      case 1: 
        this.m_tmpa.setLocation(localRectangle2D.getMaxX() - this.m_offset, localDisplay.getHeight() / 2.0D);
        break;
      case 2: 
        this.m_tmpa.setLocation(localDisplay.getWidth() / 2.0D, this.m_offset);
        break;
      case 3: 
        this.m_tmpa.setLocation(localDisplay.getWidth() / 2.0D, localRectangle2D.getMaxY() - this.m_offset);
      }
      localDisplay.getInverseTransform().transform(this.m_tmpa, this.m_tmpa);
    }
    return this.m_tmpa;
  }
  
  private double spacing(NodeItem paramNodeItem1, NodeItem paramNodeItem2, boolean paramBoolean)
  {
    int i = (this.m_orientation == 2) || (this.m_orientation == 3) ? 1 : 0;
    return (paramBoolean ? this.m_bspace : this.m_tspace) + 0.5D * (i != 0 ? paramNodeItem1.getBounds().getWidth() + paramNodeItem2.getBounds().getWidth() : paramNodeItem1.getBounds().getHeight() + paramNodeItem2.getBounds().getHeight());
  }
  
  private void updateDepths(int paramInt, NodeItem paramNodeItem)
  {
    int i = (this.m_orientation == 2) || (this.m_orientation == 3) ? 1 : 0;
    double d = i != 0 ? paramNodeItem.getBounds().getHeight() : paramNodeItem.getBounds().getWidth();
    if (this.m_depths.length <= paramInt) {
      this.m_depths = ArrayLib.resize(this.m_depths, 3 * paramInt / 2);
    }
    this.m_depths[paramInt] = Math.max(this.m_depths[paramInt], d);
    this.m_maxDepth = Math.max(this.m_maxDepth, paramInt);
  }
  
  private void determineDepths()
  {
    for (int i = 1; i < this.m_maxDepth; i++) {
      this.m_depths[i] += this.m_depths[(i - 1)] + this.m_dspace;
    }
  }
  
  public void run(double paramDouble)
  {
    Graph localGraph = (Graph)this.m_vis.getGroup(this.m_group);
    initSchema(localGraph.getNodes());
    Arrays.fill(this.m_depths, 0.0D);
    this.m_maxDepth = 0;
    Point2D localPoint2D = getLayoutAnchor();
    this.m_ax = localPoint2D.getX();
    this.m_ay = localPoint2D.getY();
    NodeItem localNodeItem = getLayoutRoot();
    Params localParams = getParams(localNodeItem);
    firstWalk(localNodeItem, 0, 1);
    determineDepths();
    secondWalk(localNodeItem, null, -localParams.prelim, 0);
  }
  
  private void firstWalk(NodeItem paramNodeItem, int paramInt1, int paramInt2)
  {
    Params localParams = getParams(paramNodeItem);
    localParams.number = paramInt1;
    updateDepths(paramInt2, paramNodeItem);
    boolean bool = paramNodeItem.isExpanded();
    NodeItem localNodeItem1;
    if ((paramNodeItem.getChildCount() == 0) || (!bool))
    {
      localNodeItem1 = (NodeItem)paramNodeItem.getPreviousSibling();
      if (localNodeItem1 == null) {
        localParams.prelim = 0.0D;
      } else {
        localParams.prelim = (getParams(localNodeItem1).prelim + spacing(localNodeItem1, paramNodeItem, true));
      }
    }
    else if (bool)
    {
      localNodeItem1 = (NodeItem)paramNodeItem.getFirstChild();
      NodeItem localNodeItem2 = (NodeItem)paramNodeItem.getLastChild();
      NodeItem localNodeItem3 = localNodeItem1;
      NodeItem localNodeItem4 = localNodeItem1;
      int i = 0;
      while (localNodeItem4 != null)
      {
        firstWalk(localNodeItem4, i, paramInt2 + 1);
        localNodeItem3 = apportion(localNodeItem4, localNodeItem3);
        i++;
        localNodeItem4 = (NodeItem)localNodeItem4.getNextSibling();
      }
      executeShifts(paramNodeItem);
      double d = 0.5D * (getParams(localNodeItem1).prelim + getParams(localNodeItem2).prelim);
      NodeItem localNodeItem5 = (NodeItem)paramNodeItem.getPreviousSibling();
      if (localNodeItem5 != null)
      {
        localParams.prelim = (getParams(localNodeItem5).prelim + spacing(localNodeItem5, paramNodeItem, true));
        localParams.mod = (localParams.prelim - d);
      }
      else
      {
        localParams.prelim = d;
      }
    }
  }
  
  private NodeItem apportion(NodeItem paramNodeItem1, NodeItem paramNodeItem2)
  {
    NodeItem localNodeItem1 = (NodeItem)paramNodeItem1.getPreviousSibling();
    if (localNodeItem1 != null)
    {
      NodeItem localNodeItem2;
      Object localObject1 = localNodeItem2 = paramNodeItem1;
      Object localObject2 = localNodeItem1;
      NodeItem localNodeItem3 = (NodeItem)((NodeItem)localObject1).getParent().getFirstChild();
      double d1 = getParams((NodeItem)localObject1).mod;
      double d3 = getParams(localNodeItem2).mod;
      double d2 = getParams((NodeItem)localObject2).mod;
      double d4 = getParams(localNodeItem3).mod;
      NodeItem localNodeItem4 = nextRight((NodeItem)localObject2);
      for (NodeItem localNodeItem5 = nextLeft((NodeItem)localObject1); (localNodeItem4 != null) && (localNodeItem5 != null); localNodeItem5 = nextLeft((NodeItem)localObject1))
      {
        localObject2 = localNodeItem4;
        localObject1 = localNodeItem5;
        localNodeItem3 = nextLeft(localNodeItem3);
        localNodeItem2 = nextRight(localNodeItem2);
        getParams(localNodeItem2).ancestor = paramNodeItem1;
        double d5 = getParams((NodeItem)localObject2).prelim + d2 - (getParams((NodeItem)localObject1).prelim + d1) + spacing((NodeItem)localObject2, (NodeItem)localObject1, false);
        if (d5 > 0.0D)
        {
          moveSubtree(ancestor((NodeItem)localObject2, paramNodeItem1, paramNodeItem2), paramNodeItem1, d5);
          d1 += d5;
          d3 += d5;
        }
        d2 += getParams((NodeItem)localObject2).mod;
        d1 += getParams((NodeItem)localObject1).mod;
        d4 += getParams(localNodeItem3).mod;
        d3 += getParams(localNodeItem2).mod;
        localNodeItem4 = nextRight((NodeItem)localObject2);
      }
      Params localParams;
      if ((localNodeItem4 != null) && (nextRight(localNodeItem2) == null))
      {
        localParams = getParams(localNodeItem2);
        localParams.thread = localNodeItem4;
        localParams.mod += d2 - d3;
      }
      if ((localNodeItem5 != null) && (nextLeft(localNodeItem3) == null))
      {
        localParams = getParams(localNodeItem3);
        localParams.thread = localNodeItem5;
        localParams.mod += d1 - d4;
        paramNodeItem2 = paramNodeItem1;
      }
    }
    return paramNodeItem2;
  }
  
  private NodeItem nextLeft(NodeItem paramNodeItem)
  {
    NodeItem localNodeItem = null;
    if (paramNodeItem.isExpanded()) {
      localNodeItem = (NodeItem)paramNodeItem.getFirstChild();
    }
    return localNodeItem != null ? localNodeItem : getParams(paramNodeItem).thread;
  }
  
  private NodeItem nextRight(NodeItem paramNodeItem)
  {
    NodeItem localNodeItem = null;
    if (paramNodeItem.isExpanded()) {
      localNodeItem = (NodeItem)paramNodeItem.getLastChild();
    }
    return localNodeItem != null ? localNodeItem : getParams(paramNodeItem).thread;
  }
  
  private void moveSubtree(NodeItem paramNodeItem1, NodeItem paramNodeItem2, double paramDouble)
  {
    Params localParams1 = getParams(paramNodeItem1);
    Params localParams2 = getParams(paramNodeItem2);
    double d = localParams2.number - localParams1.number;
    localParams2.change -= paramDouble / d;
    localParams2.shift += paramDouble;
    localParams1.change += paramDouble / d;
    localParams2.prelim += paramDouble;
    localParams2.mod += paramDouble;
  }
  
  private void executeShifts(NodeItem paramNodeItem)
  {
    double d1 = 0.0D;
    double d2 = 0.0D;
    for (NodeItem localNodeItem = (NodeItem)paramNodeItem.getLastChild(); localNodeItem != null; localNodeItem = (NodeItem)localNodeItem.getPreviousSibling())
    {
      Params localParams = getParams(localNodeItem);
      localParams.prelim += d1;
      localParams.mod += d1;
      d2 += localParams.change;
      d1 += localParams.shift + d2;
    }
  }
  
  private NodeItem ancestor(NodeItem paramNodeItem1, NodeItem paramNodeItem2, NodeItem paramNodeItem3)
  {
    NodeItem localNodeItem = (NodeItem)paramNodeItem2.getParent();
    Params localParams = getParams(paramNodeItem1);
    if (localParams.ancestor.getParent() == localNodeItem) {
      return localParams.ancestor;
    }
    return paramNodeItem3;
  }
  
  private void secondWalk(NodeItem paramNodeItem1, NodeItem paramNodeItem2, double paramDouble, int paramInt)
  {
    Params localParams = getParams(paramNodeItem1);
    setBreadth(paramNodeItem1, paramNodeItem2, localParams.prelim + paramDouble);
    setDepth(paramNodeItem1, paramNodeItem2, this.m_depths[paramInt]);
    if (paramNodeItem1.isExpanded())
    {
      paramInt++;
      for (NodeItem localNodeItem = (NodeItem)paramNodeItem1.getFirstChild(); localNodeItem != null; localNodeItem = (NodeItem)localNodeItem.getNextSibling()) {
        secondWalk(localNodeItem, paramNodeItem1, paramDouble + localParams.mod, paramInt);
      }
    }
    localParams.clear();
  }
  
  private void setBreadth(NodeItem paramNodeItem1, NodeItem paramNodeItem2, double paramDouble)
  {
    switch (this.m_orientation)
    {
    case 0: 
    case 1: 
      setY(paramNodeItem1, paramNodeItem2, this.m_ay + paramDouble);
      break;
    case 2: 
    case 3: 
      setX(paramNodeItem1, paramNodeItem2, this.m_ax + paramDouble);
      break;
    default: 
      throw new IllegalStateException();
    }
  }
  
  private void setDepth(NodeItem paramNodeItem1, NodeItem paramNodeItem2, double paramDouble)
  {
    switch (this.m_orientation)
    {
    case 0: 
      setX(paramNodeItem1, paramNodeItem2, this.m_ax + paramDouble);
      break;
    case 1: 
      setX(paramNodeItem1, paramNodeItem2, this.m_ax - paramDouble);
      break;
    case 2: 
      setY(paramNodeItem1, paramNodeItem2, this.m_ay + paramDouble);
      break;
    case 3: 
      setY(paramNodeItem1, paramNodeItem2, this.m_ay - paramDouble);
      break;
    default: 
      throw new IllegalStateException();
    }
  }
  
  protected void initSchema(TupleSet paramTupleSet)
  {
    paramTupleSet.addColumns(PARAMS_SCHEMA);
  }
  
  private Params getParams(NodeItem paramNodeItem)
  {
    Params localParams = (Params)paramNodeItem.get("_reingoldTilfordParams");
    if (localParams == null)
    {
      localParams = new Params();
      paramNodeItem.set("_reingoldTilfordParams", localParams);
    }
    if (localParams.number == -2) {
      localParams.init(paramNodeItem);
    }
    return localParams;
  }
  
  static
  {
    PARAMS_SCHEMA.addColumn("_reingoldTilfordParams", Params.class);
  }
  
  public static class Params
    implements Cloneable
  {
    double prelim;
    double mod;
    double shift;
    double change;
    int number = -2;
    NodeItem ancestor = null;
    NodeItem thread = null;
    
    public void init(NodeItem paramNodeItem)
    {
      this.ancestor = paramNodeItem;
      this.number = -1;
    }
    
    public void clear()
    {
      this.number = -2;
      this.prelim = (this.mod = this.shift = this.change = 0.0D);
      this.ancestor = (this.thread = null);
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\action\layout\graph\NodeLinkTreeLayout.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */