package prefuse.data.column;

import java.util.Arrays;
import prefuse.data.DataReadOnlyException;
import prefuse.data.DataTypeException;

public class LongColumn
  extends AbstractColumn
{
  private long[] m_values;
  private int m_size;
  
  public LongColumn()
  {
    this(0, 10, 0L);
  }
  
  public LongColumn(int paramInt)
  {
    this(paramInt, paramInt, 0L);
  }
  
  public LongColumn(int paramInt1, int paramInt2, long paramLong)
  {
    super(Long.TYPE, new Long(paramLong));
    if (paramInt2 < paramInt1) {
      throw new IllegalArgumentException("Capacity value can not be less than the row count.");
    }
    this.m_values = new long[paramInt2];
    Arrays.fill(this.m_values, paramLong);
    this.m_size = paramInt1;
  }
  
  public int getRowCount()
  {
    return this.m_size;
  }
  
  public void setMaximumRow(int paramInt)
  {
    if (paramInt > this.m_values.length)
    {
      int i = Math.max(3 * this.m_values.length / 2 + 1, paramInt);
      long[] arrayOfLong = new long[i];
      System.arraycopy(this.m_values, 0, arrayOfLong, 0, this.m_size);
      Arrays.fill(arrayOfLong, this.m_size, i, ((Long)this.m_defaultValue).longValue());
      this.m_values = arrayOfLong;
    }
    this.m_size = paramInt;
  }
  
  public Object get(int paramInt)
  {
    return new Long(getLong(paramInt));
  }
  
  public void set(Object paramObject, int paramInt)
    throws DataTypeException
  {
    if (this.m_readOnly) {
      throw new DataReadOnlyException();
    }
    if (paramObject != null)
    {
      if ((paramObject instanceof Number)) {
        setLong(((Number)paramObject).longValue(), paramInt);
      } else if ((paramObject instanceof String)) {
        setString((String)paramObject, paramInt);
      } else {
        throw new DataTypeException(paramObject.getClass());
      }
    }
    else {
      throw new DataTypeException("Column does not accept null values");
    }
  }
  
  public long getLong(int paramInt)
    throws DataTypeException
  {
    if ((paramInt < 0) || (paramInt > this.m_size)) {
      throw new IllegalArgumentException("Row index out of bounds: " + paramInt);
    }
    return this.m_values[paramInt];
  }
  
  public void setLong(long paramLong, int paramInt)
    throws DataTypeException
  {
    if (this.m_readOnly) {
      throw new DataReadOnlyException();
    }
    if ((paramInt < 0) || (paramInt >= this.m_size)) {
      throw new IllegalArgumentException("Row index out of bounds: " + paramInt);
    }
    long l = this.m_values[paramInt];
    if (l == paramLong) {
      return;
    }
    this.m_values[paramInt] = paramLong;
    fireColumnEvent(paramInt, l);
  }
  
  public String getString(int paramInt)
    throws DataTypeException
  {
    return String.valueOf(getLong(paramInt));
  }
  
  public void setString(String paramString, int paramInt)
    throws DataTypeException
  {
    setLong(Long.parseLong(paramString), paramInt);
  }
  
  public int getInt(int paramInt)
    throws DataTypeException
  {
    return (int)getLong(paramInt);
  }
  
  public float getFloat(int paramInt)
    throws DataTypeException
  {
    return (float)getLong(paramInt);
  }
  
  public double getDouble(int paramInt)
    throws DataTypeException
  {
    return getLong(paramInt);
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\column\LongColumn.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */