package prefuse.data.expression;

import java.util.Comparator;
import prefuse.data.Schema;
import prefuse.data.Tuple;
import prefuse.util.TypeLib;
import prefuse.util.collections.DefaultLiteralComparator;
import prefuse.util.collections.LiteralComparator;

public class ComparisonPredicate
  extends BinaryExpression
  implements Predicate
{
  public static final int LT = 0;
  public static final int GT = 1;
  public static final int EQ = 2;
  public static final int NEQ = 3;
  public static final int LTEQ = 4;
  public static final int GTEQ = 5;
  private Comparator m_cmp;
  
  public ComparisonPredicate(int paramInt, Expression paramExpression1, Expression paramExpression2)
  {
    this(paramInt, paramExpression1, paramExpression2, DefaultLiteralComparator.getInstance());
  }
  
  public ComparisonPredicate(int paramInt, Expression paramExpression1, Expression paramExpression2, Comparator paramComparator)
  {
    super(paramInt, 0, 5, paramExpression1, paramExpression2);
    this.m_cmp = paramComparator;
  }
  
  public Comparator getComparator()
  {
    return this.m_cmp;
  }
  
  public Class getType(Schema paramSchema)
  {
    return Boolean.TYPE;
  }
  
  public boolean getBoolean(Tuple paramTuple)
  {
    Class localClass1 = this.m_left.getType(paramTuple.getSchema());
    Class localClass2 = this.m_right.getType(paramTuple.getSchema());
    int i = 0;
    if ((TypeLib.isNumericType(localClass1)) && (TypeLib.isNumericType(localClass2)))
    {
      Class localClass3 = TypeLib.getNumericType(localClass1, localClass2);
      if (localClass3 == Integer.TYPE)
      {
        int j = this.m_left.getInt(paramTuple);
        int k = this.m_right.getInt(paramTuple);
        i = ((LiteralComparator)this.m_cmp).compare(j, k);
      }
      else if (localClass3 == Long.TYPE)
      {
        long l1 = this.m_left.getLong(paramTuple);
        long l2 = this.m_right.getLong(paramTuple);
        i = ((LiteralComparator)this.m_cmp).compare(l1, l2);
      }
      else if (localClass3 == Float.TYPE)
      {
        float f1 = this.m_left.getFloat(paramTuple);
        float f2 = this.m_right.getFloat(paramTuple);
        i = ((LiteralComparator)this.m_cmp).compare(f1, f2);
      }
      else if (localClass3 == Double.TYPE)
      {
        double d1 = this.m_left.getDouble(paramTuple);
        double d2 = this.m_right.getDouble(paramTuple);
        i = ((LiteralComparator)this.m_cmp).compare(d1, d2);
      }
      else
      {
        throw new IllegalStateException();
      }
    }
    else
    {
      i = this.m_cmp.compare(this.m_left.get(paramTuple), this.m_right.get(paramTuple));
    }
    switch (this.m_op)
    {
    case 0: 
      return i == -1;
    case 1: 
      return i == 1;
    case 2: 
      return i == 0;
    case 3: 
      return i != 0;
    case 4: 
      return i <= 0;
    case 5: 
      return i >= 0;
    }
    throw new IllegalStateException("Unknown operation.");
  }
  
  public Object get(Tuple paramTuple)
  {
    return getBoolean(paramTuple) ? Boolean.TRUE : Boolean.FALSE;
  }
  
  public String toString()
  {
    String str = "?";
    switch (this.m_op)
    {
    case 0: 
      str = "<";
      break;
    case 1: 
      str = ">";
      break;
    case 2: 
      str = "=";
      break;
    case 3: 
      str = "!=";
      break;
    case 4: 
      str = "<=";
      break;
    case 5: 
      str = ">=";
    }
    return this.m_left.toString() + ' ' + str + ' ' + this.m_right.toString();
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\expression\ComparisonPredicate.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */