package prefuse.data.io;

import java.io.InputStream;
import java.util.Date;
import java.util.HashMap;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
import prefuse.data.Graph;
import prefuse.data.Schema;
import prefuse.data.Table;
import prefuse.data.parser.DataParseException;
import prefuse.data.parser.DataParser;
import prefuse.data.parser.ParserFactory;
import prefuse.util.collections.IntIterator;

public class GraphMLReader
  extends AbstractGraphReader
  implements GraphReader
{
  public Graph readGraph(InputStream paramInputStream)
    throws DataIOException
  {
    try
    {
      SAXParserFactory localSAXParserFactory = SAXParserFactory.newInstance();
      SAXParser localSAXParser = localSAXParserFactory.newSAXParser();
      GraphMLHandler localGraphMLHandler = new GraphMLHandler();
      localSAXParser.parse(paramInputStream, localGraphMLHandler);
      return localGraphMLHandler.getGraph();
    }
    catch (Exception localException)
    {
      if ((localException instanceof DataIOException)) {
        throw ((DataIOException)localException);
      }
      throw new DataIOException(localException);
    }
  }
  
  public static class GraphMLHandler
    extends DefaultHandler
    implements GraphMLReader.Tokens
  {
    protected ParserFactory m_pf = ParserFactory.getDefaultFactory();
    protected static final String SRC = Graph.DEFAULT_SOURCE_KEY;
    protected static final String TRG = Graph.DEFAULT_TARGET_KEY;
    protected static final String SRCID = SRC + '_' + "id";
    protected static final String TRGID = TRG + '_' + "id";
    protected Schema m_nsch = new Schema();
    protected Schema m_esch = new Schema();
    protected String m_graphid;
    protected Graph m_graph = null;
    protected Table m_nodes;
    protected Table m_edges;
    protected String m_id;
    protected String m_for;
    protected String m_name;
    protected String m_type;
    protected String m_dflt;
    protected StringBuffer m_sbuf = new StringBuffer();
    private String m_key;
    private int m_row = -1;
    private Table m_table = null;
    protected HashMap m_nodeMap = new HashMap();
    protected HashMap m_idMap = new HashMap();
    private boolean m_directed = false;
    private boolean inSchema;
    
    public void startDocument()
    {
      this.m_nodeMap.clear();
      this.inSchema = true;
      this.m_esch.addColumn(SRC, Integer.TYPE);
      this.m_esch.addColumn(TRG, Integer.TYPE);
      this.m_esch.addColumn(SRCID, String.class);
      this.m_esch.addColumn(TRGID, String.class);
    }
    
    public void endDocument()
    {
      IntIterator localIntIterator = this.m_edges.rows();
      while (localIntIterator.hasNext())
      {
        int i = localIntIterator.nextInt();
        String str1 = this.m_edges.getString(i, SRCID);
        int j = ((Integer)this.m_nodeMap.get(str1)).intValue();
        this.m_edges.setInt(i, SRC, j);
        String str2 = this.m_edges.getString(i, TRGID);
        int k = ((Integer)this.m_nodeMap.get(str2)).intValue();
        this.m_edges.setInt(i, TRG, k);
      }
      this.m_edges.removeColumn(SRCID);
      this.m_edges.removeColumn(TRGID);
      this.m_graph = new Graph(this.m_nodes, this.m_edges, this.m_directed);
      if (this.m_graphid != null) {
        this.m_graph.putClientProperty("id", this.m_graphid);
      }
    }
    
    public void startElement(String paramString1, String paramString2, String paramString3, Attributes paramAttributes)
    {
      this.m_sbuf.delete(0, this.m_sbuf.length());
      String str;
      if (paramString3.equals("graph"))
      {
        str = paramAttributes.getValue("edgedefault");
        this.m_directed = "directed".equalsIgnoreCase(str);
        this.m_graphid = paramAttributes.getValue("id");
      }
      else if (paramString3.equals("key"))
      {
        if (!this.inSchema) {
          error("\"key\" elements can not occur after the first node or edge declaration.");
        }
        this.m_for = paramAttributes.getValue("for");
        this.m_id = paramAttributes.getValue("id");
        this.m_name = paramAttributes.getValue("attr.name");
        this.m_type = paramAttributes.getValue("attr.type");
      }
      else if (paramString3.equals("node"))
      {
        schemaCheck();
        this.m_row = this.m_nodes.addRow();
        str = paramAttributes.getValue("id");
        this.m_nodeMap.put(str, new Integer(this.m_row));
        this.m_table = this.m_nodes;
      }
      else if (paramString3.equals("edge"))
      {
        schemaCheck();
        this.m_row = this.m_edges.addRow();
        this.m_edges.setString(this.m_row, SRCID, paramAttributes.getValue(SRC));
        this.m_edges.setString(this.m_row, TRGID, paramAttributes.getValue(TRG));
        this.m_table = this.m_edges;
      }
      else if (paramString3.equals("data"))
      {
        this.m_key = paramAttributes.getValue("key");
      }
    }
    
    public void endElement(String paramString1, String paramString2, String paramString3)
    {
      if (paramString3.equals("default"))
      {
        this.m_dflt = this.m_sbuf.toString();
      }
      else if (paramString3.equals("key"))
      {
        addToSchema();
      }
      else if (paramString3.equals("data"))
      {
        String str1 = this.m_sbuf.toString();
        String str2 = (String)this.m_idMap.get(this.m_key);
        Class localClass = this.m_table.getColumnType(str2);
        try
        {
          Object localObject = parse(str1, localClass);
          this.m_table.set(this.m_row, str2, localObject);
        }
        catch (DataParseException localDataParseException)
        {
          error(localDataParseException);
        }
      }
      else if ((paramString3.equals("node")) || (paramString3.equals("edge")))
      {
        this.m_row = -1;
        this.m_table = null;
      }
    }
    
    public void characters(char[] paramArrayOfChar, int paramInt1, int paramInt2)
      throws SAXException
    {
      this.m_sbuf.append(paramArrayOfChar, paramInt1, paramInt2);
    }
    
    protected void schemaCheck()
    {
      if (this.inSchema)
      {
        this.m_nsch.lockSchema();
        this.m_esch.lockSchema();
        this.m_nodes = this.m_nsch.instantiate();
        this.m_edges = this.m_esch.instantiate();
        this.inSchema = false;
      }
    }
    
    protected void addToSchema()
    {
      if ((this.m_name == null) || (this.m_name.length() == 0)) {
        error("Empty key name.");
      }
      if ((this.m_type == null) || (this.m_type.length() == 0)) {
        error("Empty key type.");
      }
      try
      {
        Class localClass = parseType(this.m_type);
        Object localObject = this.m_dflt == null ? null : parse(this.m_dflt, localClass);
        if ((this.m_for == null) || (this.m_for.equals("all")))
        {
          this.m_nsch.addColumn(this.m_name, localClass, localObject);
          this.m_esch.addColumn(this.m_name, localClass, localObject);
        }
        else if (this.m_for.equals("node"))
        {
          this.m_nsch.addColumn(this.m_name, localClass, localObject);
        }
        else if (this.m_for.equals("edge"))
        {
          this.m_esch.addColumn(this.m_name, localClass, localObject);
        }
        else
        {
          error("Unrecognized \"for\" value: " + this.m_for);
        }
        this.m_idMap.put(this.m_id, this.m_name);
        this.m_dflt = null;
      }
      catch (DataParseException localDataParseException)
      {
        error(localDataParseException);
      }
    }
    
    protected Class parseType(String paramString)
    {
      paramString = paramString.toLowerCase();
      if ((paramString.equals("int")) || (paramString.equals("integer"))) {
        return Integer.TYPE;
      }
      if (paramString.equals("long")) {
        return Long.TYPE;
      }
      if (paramString.equals("float")) {
        return Float.TYPE;
      }
      if ((paramString.equals("double")) || (paramString.equals("real"))) {
        return Double.TYPE;
      }
      if (paramString.equals("boolean")) {
        return Boolean.TYPE;
      }
      if (paramString.equals("string")) {
        return String.class;
      }
      if (paramString.equals("date")) {
        return Date.class;
      }
      error("Unrecognized data type: " + paramString);
      return null;
    }
    
    protected Object parse(String paramString, Class paramClass)
      throws DataParseException
    {
      DataParser localDataParser = this.m_pf.getParser(paramClass);
      return localDataParser.parse(paramString);
    }
    
    public Graph getGraph()
    {
      return this.m_graph;
    }
    
    protected void error(String paramString)
    {
      throw new RuntimeException(paramString);
    }
    
    protected void error(Exception paramException)
    {
      throw new RuntimeException(paramException);
    }
  }
  
  public static abstract interface Tokens
  {
    public static final String ID = "id";
    public static final String GRAPH = "graph";
    public static final String EDGEDEF = "edgedefault";
    public static final String DIRECTED = "directed";
    public static final String UNDIRECTED = "undirected";
    public static final String KEY = "key";
    public static final String FOR = "for";
    public static final String ALL = "all";
    public static final String ATTRNAME = "attr.name";
    public static final String ATTRTYPE = "attr.type";
    public static final String DEFAULT = "default";
    public static final String NODE = "node";
    public static final String EDGE = "edge";
    public static final String SOURCE = "source";
    public static final String TARGET = "target";
    public static final String DATA = "data";
    public static final String TYPE = "type";
    public static final String INT = "int";
    public static final String INTEGER = "integer";
    public static final String LONG = "long";
    public static final String FLOAT = "float";
    public static final String DOUBLE = "double";
    public static final String REAL = "real";
    public static final String BOOLEAN = "boolean";
    public static final String STRING = "string";
    public static final String DATE = "date";
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\io\GraphMLReader.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */