package prefuse.data.util;

import java.util.Iterator;
import prefuse.data.Edge;
import prefuse.data.Node;
import prefuse.data.Tuple;
import prefuse.util.collections.Queue;

public class BreadthFirstIterator
  implements Iterator
{
  protected Queue m_queue = new Queue();
  protected int m_depth;
  protected int m_traversal;
  protected boolean m_includeNodes;
  protected boolean m_includeEdges;
  
  public BreadthFirstIterator() {}
  
  public BreadthFirstIterator(Node paramNode, int paramInt1, int paramInt2)
  {
    init(new Node[] { paramNode }, paramInt1, paramInt2);
  }
  
  public BreadthFirstIterator(Iterator paramIterator, int paramInt1, int paramInt2)
  {
    init(paramIterator, paramInt1, paramInt2);
  }
  
  public void init(Object paramObject, int paramInt1, int paramInt2)
  {
    this.m_queue.clear();
    this.m_depth = paramInt1;
    if ((paramInt2 < 0) || (paramInt2 >= 3)) {
      throw new IllegalArgumentException("Unrecognized traversal type: " + paramInt2);
    }
    this.m_traversal = paramInt2;
    this.m_includeNodes = ((paramInt2 == 0) || (paramInt2 == 2));
    this.m_includeEdges = ((paramInt2 == 1) || (paramInt2 == 2));
    Object localObject1;
    if (this.m_includeNodes)
    {
      if ((paramObject instanceof Node))
      {
        this.m_queue.add(paramObject, 0);
      }
      else
      {
        localObject1 = (Iterator)paramObject;
        while (((Iterator)localObject1).hasNext()) {
          this.m_queue.add(((Iterator)localObject1).next(), 0);
        }
      }
    }
    else
    {
      Object localObject2;
      Object localObject3;
      Object localObject4;
      if ((paramObject instanceof Node))
      {
        localObject1 = (Node)paramObject;
        this.m_queue.visit(localObject1, 0);
        localObject2 = getEdges((Node)localObject1);
        while (((Iterator)localObject2).hasNext())
        {
          localObject3 = (Edge)((Iterator)localObject2).next();
          localObject4 = ((Edge)localObject3).getAdjacentNode((Node)localObject1);
          this.m_queue.visit(localObject4, 1);
          if (this.m_queue.getDepth(localObject3) < 0) {
            this.m_queue.add(localObject3, 1);
          }
        }
      }
      else
      {
        localObject1 = (Iterator)paramObject;
        while (((Iterator)localObject1).hasNext())
        {
          localObject2 = (Node)((Iterator)localObject1).next();
          this.m_queue.visit(localObject2, 0);
          localObject3 = getEdges((Node)localObject2);
          while (((Iterator)localObject3).hasNext())
          {
            localObject4 = (Edge)((Iterator)localObject3).next();
            Node localNode = ((Edge)localObject4).getAdjacentNode((Node)localObject2);
            this.m_queue.visit(localNode, 1);
            if (this.m_queue.getDepth(localObject4) < 0) {
              this.m_queue.add(localObject4, 1);
            }
          }
        }
      }
    }
  }
  
  public void remove()
  {
    throw new UnsupportedOperationException();
  }
  
  public boolean hasNext()
  {
    return !this.m_queue.isEmpty();
  }
  
  protected Iterator getEdges(Node paramNode)
  {
    return paramNode.edges();
  }
  
  public int getDepth(Tuple paramTuple)
  {
    return this.m_queue.getDepth(paramTuple);
  }
  
  public Object next()
  {
    Tuple localTuple = (Tuple)this.m_queue.removeFirst();
    Object localObject1;
    Object localObject2;
    switch (this.m_traversal)
    {
    case 0: 
    case 2: 
      if ((localTuple instanceof Edge)) {
        return localTuple;
      }
      localObject1 = (Node)localTuple;
      int i = this.m_queue.getDepth(localObject1);
      Object localObject3;
      Object localObject4;
      if (i < this.m_depth)
      {
        int j = i + 1;
        localObject3 = getEdges((Node)localObject1);
        while (((Iterator)localObject3).hasNext())
        {
          localObject4 = (Edge)((Iterator)localObject3).next();
          Node localNode2 = ((Edge)localObject4).getAdjacentNode((Node)localObject1);
          if ((this.m_includeEdges) && (this.m_queue.getDepth(localObject4) < 0)) {
            this.m_queue.add(localObject4, j);
          }
          if (this.m_queue.getDepth(localNode2) < 0) {
            this.m_queue.add(localNode2, j);
          }
        }
      }
      else if ((this.m_includeEdges) && (i == this.m_depth))
      {
        localObject2 = getEdges((Node)localObject1);
        while (((Iterator)localObject2).hasNext())
        {
          localObject3 = (Edge)((Iterator)localObject2).next();
          localObject4 = ((Edge)localObject3).getAdjacentNode((Node)localObject1);
          int n = this.m_queue.getDepth(localObject4);
          if ((n > 0) && (this.m_queue.getDepth(localObject3) < 0)) {
            this.m_queue.add(localObject3, Math.min(i, n));
          }
        }
      }
      return localObject1;
    case 1: 
      localObject1 = (Edge)localTuple;
      Node localNode1 = ((Edge)localObject1).getSourceNode();
      localObject2 = ((Edge)localObject1).getTargetNode();
      int k = this.m_queue.getDepth(localNode1);
      int m = this.m_queue.getDepth(localObject2);
      if (k != m)
      {
        Node localNode3 = m > k ? localObject2 : localNode1;
        int i1 = Math.max(k, m);
        if (i1 < this.m_depth)
        {
          int i2 = i1 + 1;
          Iterator localIterator = getEdges(localNode3);
          while (localIterator.hasNext())
          {
            Edge localEdge = (Edge)localIterator.next();
            if (this.m_queue.getDepth(localEdge) < 0)
            {
              Node localNode4 = localEdge.getAdjacentNode(localNode3);
              this.m_queue.visit(localNode4, i2);
              this.m_queue.add(localEdge, i2);
            }
          }
        }
      }
      return localObject1;
    }
    throw new IllegalStateException();
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\util\BreadthFirstIterator.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */