package prefuse.util;

import java.util.Arrays;

public class MathLib
{
  public static final double TWO_PI = 6.283185307179586D;
  public static final double LOG10 = Math.log(10.0D);
  public static final double LOG2 = Math.log(2.0D);
  
  public static double log2(double paramDouble)
  {
    return Math.log(paramDouble) / LOG2;
  }
  
  public static double log10(double paramDouble)
  {
    return Math.log(paramDouble) / LOG10;
  }
  
  public static double safeLog10(double paramDouble)
  {
    int i = paramDouble < 0.0D ? 1 : 0;
    if (i != 0) {
      paramDouble = -paramDouble;
    }
    paramDouble = Math.log(paramDouble) / LOG10;
    return i != 0 ? -paramDouble : paramDouble;
  }
  
  public static double safeSqrt(double paramDouble)
  {
    return paramDouble < 0.0D ? -Math.sqrt(-paramDouble) : Math.sqrt(paramDouble);
  }
  
  public static double interp(int paramInt, double paramDouble, double[] paramArrayOfDouble)
  {
    switch (paramInt)
    {
    case 0: 
      return linearInterp(paramDouble, paramArrayOfDouble[0], paramArrayOfDouble[(paramArrayOfDouble.length - 1)]);
    case 1: 
      return logInterp(paramDouble, paramArrayOfDouble[0], paramArrayOfDouble[(paramArrayOfDouble.length - 1)]);
    case 2: 
      return sqrtInterp(paramDouble, paramArrayOfDouble[0], paramArrayOfDouble[(paramArrayOfDouble.length - 1)]);
    case 3: 
      return quantile(paramDouble, paramArrayOfDouble);
    }
    throw new IllegalArgumentException("Unrecognized scale value: " + paramInt);
  }
  
  public static double linearInterp(double paramDouble1, double paramDouble2, double paramDouble3)
  {
    return (paramDouble1 - paramDouble2) / (paramDouble3 - paramDouble2);
  }
  
  public static double logInterp(double paramDouble1, double paramDouble2, double paramDouble3)
  {
    double d = safeLog10(paramDouble2);
    return (safeLog10(paramDouble1) - d) / (safeLog10(paramDouble3) - d);
  }
  
  public static double sqrtInterp(double paramDouble1, double paramDouble2, double paramDouble3)
  {
    double d = safeSqrt(paramDouble2);
    return (safeSqrt(paramDouble1) - d) / (safeSqrt(paramDouble3) - d);
  }
  
  public static double[] quantiles(int paramInt, double[] paramArrayOfDouble)
  {
    paramArrayOfDouble = (double[])paramArrayOfDouble.clone();
    Arrays.sort(paramArrayOfDouble);
    double[] arrayOfDouble = new double[paramInt + 1];
    for (int i = 0; i <= paramInt; i++) {
      arrayOfDouble[i] = paramArrayOfDouble[((paramArrayOfDouble.length - 1) * i / paramInt)];
    }
    return arrayOfDouble;
  }
  
  public static double quantile(double paramDouble, double[] paramArrayOfDouble)
  {
    int i = 1;
    int j = paramArrayOfDouble.length;
    for (int k = j / 2; (i < j) && (paramArrayOfDouble[k] != paramDouble); k = i + (j - i) / 2) {
      if (paramArrayOfDouble[k] < paramDouble) {
        i = k + 1;
      } else {
        j = k;
      }
    }
    return k / (paramArrayOfDouble.length - 1);
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\MathLib.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */