package prefuse.util.collections;

import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

public class ObjectIntTreeMap
  extends AbstractTreeMap
  implements ObjectIntSortedMap
{
  private ObjectEntry dummy = new ObjectEntry(null, Integer.MIN_VALUE, NIL, 0);
  private Comparator cmp = null;
  
  public ObjectIntTreeMap()
  {
    this(null, false);
  }
  
  public ObjectIntTreeMap(boolean paramBoolean)
  {
    this(null, paramBoolean);
  }
  
  public ObjectIntTreeMap(Comparator paramComparator)
  {
    this(paramComparator, false);
  }
  
  public ObjectIntTreeMap(Comparator paramComparator, boolean paramBoolean)
  {
    super(null, paramBoolean);
  }
  
  public Comparator comparator()
  {
    return this.cmp;
  }
  
  public boolean containsKey(Object paramObject)
  {
    return find(paramObject, 0) != NIL;
  }
  
  public int get(Object paramObject)
  {
    AbstractTreeMap.Entry localEntry = find(paramObject, 0);
    return localEntry == NIL ? Integer.MIN_VALUE : localEntry.val;
  }
  
  public int put(Object paramObject, int paramInt)
  {
    AbstractTreeMap.Entry localEntry = this.root;
    this.lastOrder = 0;
    if (localEntry == NIL)
    {
      incrementSize(true);
      this.root = new ObjectEntry(paramObject, paramInt, NIL, this.lastOrder);
      return Integer.MIN_VALUE;
    }
    this.dummy.key = paramObject;
    this.dummy.order = Integer.MAX_VALUE;
    for (;;)
    {
      int i = compare(this.dummy, localEntry);
      if (i == 0) {
        return localEntry.setValue(paramInt);
      }
      if (i < 0)
      {
        if (localEntry.left != NIL)
        {
          localEntry = localEntry.left;
        }
        else
        {
          incrementSize(this.lastOrder == 0);
          localEntry.left = new ObjectEntry(paramObject, paramInt, localEntry, this.lastOrder);
          fixUpInsert(localEntry.left);
          return Integer.MIN_VALUE;
        }
      }
      else if (localEntry.right != NIL)
      {
        localEntry = localEntry.right;
      }
      else
      {
        incrementSize(this.lastOrder == 0);
        localEntry.right = new ObjectEntry(paramObject, paramInt, localEntry, this.lastOrder);
        fixUpInsert(localEntry.right);
        return Integer.MIN_VALUE;
      }
    }
  }
  
  public int remove(Object paramObject)
  {
    AbstractTreeMap.Entry localEntry;
    if (this.allowDuplicates) {
      localEntry = findPredecessor(paramObject, Integer.MAX_VALUE);
    } else {
      localEntry = find(paramObject, 0);
    }
    if (localEntry == NIL) {
      return Integer.MIN_VALUE;
    }
    int i = localEntry.val;
    remove(localEntry);
    return i;
  }
  
  public int remove(Object paramObject, int paramInt)
  {
    AbstractTreeMap.Entry localEntry = findCeiling(paramObject, 0);
    if ((localEntry != NIL) && (((paramObject == null) && (localEntry.getKey() != null)) || ((paramObject != null) && (!localEntry.getKey().equals(paramObject))))) {
      localEntry = successor(localEntry);
    }
    if ((localEntry == NIL) || ((paramObject == null) && (localEntry.getKey() != null)) || ((paramObject != null) && (!localEntry.getKey().equals(paramObject)))) {
      return Integer.MIN_VALUE;
    }
    while ((localEntry.val != paramInt) && (localEntry != NIL)) {
      localEntry = successor(localEntry);
    }
    if (localEntry == NIL) {
      return Integer.MIN_VALUE;
    }
    remove(localEntry);
    return paramInt;
  }
  
  public Object firstKey()
  {
    return minimum(this.root).getKey();
  }
  
  public Object lastKey()
  {
    return maximum(this.root).getKey();
  }
  
  public Iterator keyIterator()
  {
    return new AbstractTreeMap.KeyIterator(this);
  }
  
  public Iterator keyRangeIterator(Object paramObject1, boolean paramBoolean1, Object paramObject2, boolean paramBoolean2)
  {
    if ((paramObject1 == paramObject2) && ((paramObject1 == MIN_KEY) || (paramObject1 == MAX_KEY))) {
      return Collections.EMPTY_LIST.iterator();
    }
    int i = (paramObject1 == MIN_KEY) || (paramObject2 == MAX_KEY) ? 1 : 0;
    int j = (paramObject1 == MAX_KEY) || (paramObject2 == MIN_KEY) ? 1 : 0;
    AbstractTreeMap.Entry localEntry1;
    AbstractTreeMap.Entry localEntry2;
    if ((j == 0) && ((i != 0) || (this.cmp.compare(paramObject1, paramObject2) <= 0)))
    {
      localEntry1 = findCeiling(paramObject1, paramBoolean1 ? 0 : Integer.MAX_VALUE);
      localEntry2 = findCeiling(paramObject2, paramBoolean2 ? Integer.MAX_VALUE : 0);
    }
    else
    {
      localEntry1 = findCeiling(paramObject1, paramBoolean1 ? Integer.MAX_VALUE : 0);
      localEntry1 = predecessor(localEntry1);
      localEntry2 = findCeiling(paramObject2, paramBoolean2 ? 0 : Integer.MAX_VALUE);
      localEntry2 = predecessor(localEntry2);
    }
    return new AbstractTreeMap.KeyIterator(this, localEntry1, localEntry2);
  }
  
  public IntIterator valueRangeIterator(Object paramObject1, boolean paramBoolean1, Object paramObject2, boolean paramBoolean2)
  {
    return new AbstractTreeMap.ValueIterator(this, (AbstractTreeMap.EntryIterator)keyRangeIterator(paramObject1, paramBoolean1, paramObject2, paramBoolean2));
  }
  
  protected int compare(AbstractTreeMap.Entry paramEntry1, AbstractTreeMap.Entry paramEntry2)
  {
    Object localObject1 = paramEntry1.getKey();
    Object localObject2 = paramEntry2.getKey();
    if ((localObject1 == localObject2) && ((localObject1 == MIN_KEY) || (localObject1 == MAX_KEY))) {
      return 0;
    }
    if ((localObject1 == MIN_KEY) || (localObject2 == MAX_KEY)) {
      return -1;
    }
    if ((localObject1 == MAX_KEY) || (localObject2 == MIN_KEY)) {
      return 1;
    }
    int i = this.cmp.compare(paramEntry1.getKey(), paramEntry2.getKey());
    if ((this.allowDuplicates) && (i == 0))
    {
      i = paramEntry1.order > paramEntry2.order ? 1 : paramEntry1.order < paramEntry2.order ? -1 : 0;
      this.lastOrder = (1 + (i < 0 ? paramEntry1.order : paramEntry2.order));
    }
    return i;
  }
  
  private AbstractTreeMap.Entry find(Object paramObject, int paramInt)
  {
    this.dummy.key = paramObject;
    this.dummy.order = paramInt;
    AbstractTreeMap.Entry localEntry = find(this.dummy);
    this.dummy.key = null;
    return localEntry;
  }
  
  private AbstractTreeMap.Entry findPredecessor(Object paramObject, int paramInt)
  {
    this.dummy.key = paramObject;
    this.dummy.order = paramInt;
    AbstractTreeMap.Entry localEntry = findPredecessor(this.dummy);
    this.dummy.key = null;
    return localEntry;
  }
  
  private AbstractTreeMap.Entry findCeiling(Object paramObject, int paramInt)
  {
    this.dummy.key = paramObject;
    this.dummy.order = paramInt;
    AbstractTreeMap.Entry localEntry = findCeiling(this.dummy);
    this.dummy.key = null;
    return localEntry;
  }
  
  static class ObjectEntry
    extends AbstractTreeMap.Entry
  {
    Object key;
    
    public ObjectEntry(Object paramObject, int paramInt)
    {
      super();
      this.key = paramObject;
    }
    
    public ObjectEntry(Object paramObject, int paramInt1, AbstractTreeMap.Entry paramEntry, int paramInt2)
    {
      super(paramEntry, paramInt2);
      this.key = paramObject;
    }
    
    public Object getKey()
    {
      return this.key;
    }
    
    public void copyFields(AbstractTreeMap.Entry paramEntry)
    {
      super.copyFields(paramEntry);
      this.key = paramEntry.getKey();
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\collections\ObjectIntTreeMap.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */