package prefuse.util.display;

import java.awt.Container;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.geom.AffineTransform;
import java.net.URL;
import prefuse.Display;
import prefuse.util.io.IOLib;

public class BackgroundPainter
  implements PaintListener
{
  private static final double THRESH = 0.01D;
  private Image m_img;
  private boolean m_fixed;
  private boolean m_tiled;
  private AffineTransform m_identity;
  private Clip m_clip;
  
  public BackgroundPainter(String paramString, boolean paramBoolean1, boolean paramBoolean2)
  {
    URL localURL = IOLib.urlFromString(paramString);
    this.m_img = Toolkit.getDefaultToolkit().getImage(localURL);
    MediaTracker localMediaTracker = new MediaTracker(new Container());
    localMediaTracker.addImage(this.m_img, 0);
    try
    {
      localMediaTracker.waitForID(0);
    }
    catch (Exception localException)
    {
      localException.printStackTrace();
    }
    localMediaTracker.removeImage(this.m_img, 0);
    this.m_fixed = paramBoolean1;
    this.m_tiled = paramBoolean2;
  }
  
  public BackgroundPainter(Image paramImage, boolean paramBoolean1, boolean paramBoolean2)
  {
    this.m_img = paramImage;
    this.m_fixed = paramBoolean1;
    this.m_tiled = paramBoolean2;
  }
  
  public void prePaint(Display paramDisplay, Graphics2D paramGraphics2D)
  {
    AffineTransform localAffineTransform = paramGraphics2D.getTransform();
    boolean bool = isTranslation(localAffineTransform);
    int i;
    int j;
    int m;
    int n;
    int i1;
    int i2;
    int i3;
    int i4;
    int i5;
    if ((this.m_fixed) || (bool))
    {
      i = this.m_fixed ? 0 : (int)localAffineTransform.getTranslateX();
      j = this.m_fixed ? 0 : (int)localAffineTransform.getTranslateY();
      paramGraphics2D.setTransform(getIdentity());
      if (this.m_tiled)
      {
        int k = paramDisplay.getWidth();
        m = this.m_img.getWidth(null);
        n = paramDisplay.getHeight();
        i1 = this.m_img.getHeight(null);
        i2 = this.m_fixed ? 0 : i % m;
        i3 = this.m_fixed ? 0 : j % i1;
        if (i2 > 0) {
          i2 -= m;
        }
        if (i3 > 0) {
          i3 -= i1;
        }
        i4 = i2;
        while (i4 < k - i2)
        {
          i5 = i3;
          while (i5 < n - i3)
          {
            paramGraphics2D.drawImage(this.m_img, i4, i5, null);
            i5 += i1;
          }
          i4 += m;
        }
      }
      else
      {
        paramGraphics2D.drawImage(this.m_img, i, j, null);
      }
      paramGraphics2D.setTransform(localAffineTransform);
    }
    else if (this.m_tiled)
    {
      i = this.m_img.getWidth(null);
      j = this.m_img.getHeight(null);
      Clip localClip = getClip();
      localClip.setClip(0.0D, 0.0D, paramDisplay.getWidth(), paramDisplay.getHeight());
      localClip.transform(paramDisplay.getInverseTransform());
      m = (int)Math.ceil(localClip.getWidth());
      n = (int)Math.ceil(localClip.getHeight());
      i1 = (int)localClip.getMinX();
      i2 = (int)localClip.getMinY();
      i3 = i1 % i + i;
      i4 = i2 % j + j;
      i1 -= i3;
      m += i3;
      i2 -= i4;
      n += i4;
      i5 = i1;
      while (i5 < i1 + m)
      {
        int i6 = i2;
        while (i6 < i2 + n)
        {
          paramGraphics2D.drawImage(this.m_img, i5, i6, null);
          i6 += j;
        }
        i5 += i;
      }
    }
    else
    {
      paramGraphics2D.drawImage(this.m_img, 0, 0, null);
    }
  }
  
  private static boolean isTranslation(AffineTransform paramAffineTransform)
  {
    return (Math.abs(paramAffineTransform.getScaleX() - 1.0D) < 0.01D) && (Math.abs(paramAffineTransform.getScaleY() - 1.0D) < 0.01D) && (Math.abs(paramAffineTransform.getShearX()) < 0.01D) && (Math.abs(paramAffineTransform.getShearY()) < 0.01D);
  }
  
  private AffineTransform getIdentity()
  {
    if (this.m_identity == null) {
      this.m_identity = new AffineTransform();
    }
    return this.m_identity;
  }
  
  private Clip getClip()
  {
    if (this.m_clip == null) {
      this.m_clip = new Clip();
    }
    return this.m_clip;
  }
  
  public void postPaint(Display paramDisplay, Graphics2D paramGraphics2D) {}
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\display\BackgroundPainter.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */