package prefuse.visual;

import java.awt.BasicStroke;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import prefuse.Visualization;
import prefuse.data.Schema;
import prefuse.data.Tuple;
import prefuse.render.Renderer;
import prefuse.util.PrefuseLib;

public abstract interface VisualItem
  extends Tuple
{
  public static final String VALIDATED = PrefuseLib.FIELD_PREFIX + "validated";
  public static final String VISIBLE = PrefuseLib.FIELD_PREFIX + "visible";
  public static final String STARTVISIBLE = PrefuseLib.getStartField(VISIBLE);
  public static final String ENDVISIBLE = PrefuseLib.getEndField(VISIBLE);
  public static final String INTERACTIVE = PrefuseLib.FIELD_PREFIX + "interactive";
  public static final String EXPANDED = PrefuseLib.FIELD_PREFIX + "expanded";
  public static final String FIXED = PrefuseLib.FIELD_PREFIX + "fixed";
  public static final String HIGHLIGHT = PrefuseLib.FIELD_PREFIX + "highlight";
  public static final String HOVER = PrefuseLib.FIELD_PREFIX + "hover";
  public static final String X = PrefuseLib.FIELD_PREFIX + "x";
  public static final String Y = PrefuseLib.FIELD_PREFIX + "y";
  public static final String STARTX = PrefuseLib.getStartField(X);
  public static final String STARTY = PrefuseLib.getStartField(Y);
  public static final String ENDX = PrefuseLib.getEndField(X);
  public static final String ENDY = PrefuseLib.getEndField(Y);
  public static final String BOUNDS = PrefuseLib.FIELD_PREFIX + "bounds";
  public static final String STROKECOLOR = PrefuseLib.FIELD_PREFIX + "strokeColor";
  public static final String STARTSTROKECOLOR = PrefuseLib.getStartField(STROKECOLOR);
  public static final String ENDSTROKECOLOR = PrefuseLib.getEndField(STROKECOLOR);
  public static final String FILLCOLOR = PrefuseLib.FIELD_PREFIX + "fillColor";
  public static final String STARTFILLCOLOR = PrefuseLib.getStartField(FILLCOLOR);
  public static final String ENDFILLCOLOR = PrefuseLib.getEndField(FILLCOLOR);
  public static final String TEXTCOLOR = PrefuseLib.FIELD_PREFIX + "textColor";
  public static final String STARTTEXTCOLOR = PrefuseLib.getStartField(TEXTCOLOR);
  public static final String ENDTEXTCOLOR = PrefuseLib.getEndField(TEXTCOLOR);
  public static final String SIZE = PrefuseLib.FIELD_PREFIX + "size";
  public static final String STARTSIZE = PrefuseLib.getStartField(SIZE);
  public static final String ENDSIZE = PrefuseLib.getEndField(SIZE);
  public static final String SHAPE = PrefuseLib.FIELD_PREFIX + "shape";
  public static final String STROKE = PrefuseLib.FIELD_PREFIX + "stroke";
  public static final String FONT = PrefuseLib.FIELD_PREFIX + "font";
  public static final String STARTFONT = PrefuseLib.getStartField(FONT);
  public static final String ENDFONT = PrefuseLib.getEndField(FONT);
  public static final String DOI = PrefuseLib.FIELD_PREFIX + "doi";
  public static final Schema SCHEMA = PrefuseLib.getVisualItemSchema().lockSchema();
  public static final int IDX_VALIDATED = SCHEMA.getColumnIndex(VALIDATED);
  public static final String LABEL = PrefuseLib.FIELD_PREFIX + "label";
  public static final String VALUE = PrefuseLib.FIELD_PREFIX + "value";
  public static final String POLYGON = PrefuseLib.FIELD_PREFIX + "polygon";
  public static final String STARTPOLYGON = PrefuseLib.getStartField(POLYGON);
  public static final String ENDPOLYGON = PrefuseLib.getStartField(POLYGON);
  public static final String X2 = PrefuseLib.FIELD_PREFIX + "x2";
  public static final String Y2 = PrefuseLib.FIELD_PREFIX + "y2";
  public static final String STARTX2 = PrefuseLib.getStartField(X2);
  public static final String STARTY2 = PrefuseLib.getStartField(Y2);
  public static final String ENDX2 = PrefuseLib.getEndField(X2);
  public static final String ENDY2 = PrefuseLib.getEndField(Y2);
  
  public abstract Visualization getVisualization();
  
  public abstract String getGroup();
  
  public abstract boolean isInGroup(String paramString);
  
  public abstract void render(Graphics2D paramGraphics2D);
  
  public abstract Renderer getRenderer();
  
  public abstract Rectangle2D validateBounds();
  
  public abstract boolean isValidated();
  
  public abstract void setValidated(boolean paramBoolean);
  
  public abstract boolean isVisible();
  
  public abstract void setVisible(boolean paramBoolean);
  
  public abstract boolean isStartVisible();
  
  public abstract void setStartVisible(boolean paramBoolean);
  
  public abstract boolean isEndVisible();
  
  public abstract void setEndVisible(boolean paramBoolean);
  
  public abstract boolean isInteractive();
  
  public abstract void setInteractive(boolean paramBoolean);
  
  public abstract boolean isExpanded();
  
  public abstract void setExpanded(boolean paramBoolean);
  
  public abstract boolean isFixed();
  
  public abstract void setFixed(boolean paramBoolean);
  
  public abstract boolean isHighlighted();
  
  public abstract void setHighlighted(boolean paramBoolean);
  
  public abstract boolean isHover();
  
  public abstract void setHover(boolean paramBoolean);
  
  public abstract double getX();
  
  public abstract void setX(double paramDouble);
  
  public abstract double getY();
  
  public abstract void setY(double paramDouble);
  
  public abstract double getStartX();
  
  public abstract void setStartX(double paramDouble);
  
  public abstract double getStartY();
  
  public abstract void setStartY(double paramDouble);
  
  public abstract double getEndX();
  
  public abstract void setEndX(double paramDouble);
  
  public abstract double getEndY();
  
  public abstract void setEndY(double paramDouble);
  
  public abstract Rectangle2D getBounds();
  
  public abstract void setBounds(double paramDouble1, double paramDouble2, double paramDouble3, double paramDouble4);
  
  public abstract int getStrokeColor();
  
  public abstract void setStrokeColor(int paramInt);
  
  public abstract int getStartStrokeColor();
  
  public abstract void setStartStrokeColor(int paramInt);
  
  public abstract int getEndStrokeColor();
  
  public abstract void setEndStrokeColor(int paramInt);
  
  public abstract int getFillColor();
  
  public abstract void setFillColor(int paramInt);
  
  public abstract int getStartFillColor();
  
  public abstract void setStartFillColor(int paramInt);
  
  public abstract int getEndFillColor();
  
  public abstract void setEndFillColor(int paramInt);
  
  public abstract int getTextColor();
  
  public abstract void setTextColor(int paramInt);
  
  public abstract int getStartTextColor();
  
  public abstract void setStartTextColor(int paramInt);
  
  public abstract int getEndTextColor();
  
  public abstract void setEndTextColor(int paramInt);
  
  public abstract double getSize();
  
  public abstract void setSize(double paramDouble);
  
  public abstract double getStartSize();
  
  public abstract void setStartSize(double paramDouble);
  
  public abstract double getEndSize();
  
  public abstract void setEndSize(double paramDouble);
  
  public abstract int getShape();
  
  public abstract void setShape(int paramInt);
  
  public abstract BasicStroke getStroke();
  
  public abstract void setStroke(BasicStroke paramBasicStroke);
  
  public abstract Font getFont();
  
  public abstract void setFont(Font paramFont);
  
  public abstract Font getStartFont();
  
  public abstract void setStartFont(Font paramFont);
  
  public abstract Font getEndFont();
  
  public abstract void setEndFont(Font paramFont);
  
  public abstract double getDOI();
  
  public abstract void setDOI(double paramDouble);
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\visual\VisualItem.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */